﻿using System;
using System.Linq;

namespace Xamarin.Examples.Demo.Showcase.VitalSignsMonitor
{
    public class DefaultVitalSignsDataProvider : DataProviderBase<VitalSignsData>
    {
        private const float SampleRate = 800f;

        private int _currentIndex = 0;
        private int _totalIndex = 0;
        private bool _isATrace = false;

        private readonly double[] _xValues;
        private readonly double[] _ecgHeartRate;
        private readonly double[] _bloodPressure;
        private readonly double[] _bloodVolume;
        private readonly double[] _bloodOxygenation;

        public DefaultVitalSignsDataProvider() : base(TimeSpan.FromMilliseconds(1))
        {
            var lines = VitalSignsTraces.Split("\r\n");

            var count = lines.Length;

            _xValues = new double[count];
            _ecgHeartRate = new double[count];
            _bloodPressure = new double[count];
            _bloodVolume = new double[count];
            _bloodOxygenation = new double[count];

            for (var i = 0; i < lines.Length; i++)
            {
                var split = lines[i].Split(',');

                _xValues[i] = Double.Parse(split[0]);
                _ecgHeartRate[i] = Double.Parse(split[1]);
                _bloodPressure[i] = Double.Parse(split[2]);
                _bloodVolume[i] = Double.Parse(split[3]);
                _bloodOxygenation[i] = Double.Parse(split[4]);
            }
        }

        protected override VitalSignsData OnNext()
        {
            if (_currentIndex >= _xValues.Length)
                _currentIndex = 0;

            var time = _totalIndex / SampleRate % 10;
            var ecgHeartRate = _ecgHeartRate[_currentIndex];
            var bloodPressure = _bloodPressure[_currentIndex];
            var bloodVolume = _bloodVolume[_currentIndex];
            var bloodOxygenation = _bloodOxygenation[_currentIndex];

            var data = new VitalSignsData(time, ecgHeartRate, bloodPressure, bloodVolume, bloodOxygenation, _isATrace);

            _currentIndex++;
            _totalIndex++;

            if (_totalIndex % 8000 == 0)
                _isATrace = !_isATrace;

            return data;
        }

        public (double, double) EcgHeartRateRange
        {
            get
            {
                return GetMinMax(_ecgHeartRate);
            }
        }

        public (double, double) BloodPressureRange
        {
            get 
            {
               return GetMinMax(_bloodPressure);
            }
        }

        public (double, double) BloodVolumeRange
        {
            get
            {
                return GetMinMax(_bloodVolume);
            }
        }

        public (double, double) BloodOxygenationRange
        {
            get
            {
                return GetMinMax(_bloodOxygenation);
            }
        }

        private static (double, double) GetMinMax(double[] values)
        {
            var min = values.Min();
            var max = values.Max();
            var growBy = Math.Abs(max - min) * 0.1;
            return (min - growBy, max + growBy);
        }

        private const string VitalSignsTraces = "3.12833,0.873118,0.625403,0.209285,0.100243\r\n5.61393,0.873034,0.625572,0.209349,0.100496\r\n8.98114,0.87292,0.626935,0.209435,0.100839\r\n9.31104,0.872909,0.627069,0.209443,0.100872\r\n9.37361,0.872889,0.627094,0.209445,0.100878\r\n13.1333,0.871736,0.628617,0.209541,0.101261\r\n13.156,0.871729,0.630139,0.209542,0.101264\r\n16.7508,0.870627,0.631595,0.209634,0.10163\r\n16.7621,0.871388,0.631599,0.209634,0.101631\r\n16.9157,0.871325,0.631662,0.209638,0.101646\r\n18.0817,0.870847,0.632016,0.209668,0.101765\r\n20.209,0.869976,0.632664,0.209722,0.101982\r\n20.4763,0.869866,0.632745,0.209729,0.102023\r\n20.6697,0.869787,0.632804,0.209734,0.102052\r\n24.1905,0.868345,0.63423,0.209824,0.102589\r\n24.4294,0.868308,0.634326,0.20983,0.102626\r\n27.6999,0.867808,0.635651,0.209914,0.103124\r\n28.1891,0.867733,0.635849,0.209927,0.103174\r\n31.6473,0.8704315,0.63725,0.210015,0.103526\r\n31.9488,0.8701074,0.637372,0.210023,0.103557\r\n33.0351,0.86973985,0.637812,0.210051,0.103668\r\n35.3615,0.8693673,0.638754,0.210051,0.103905\r\n35.7084,0.86889605,0.638894,0.210051,0.10394\r\n38.9278,0.8684216,0.639385,0.210052,0.104268\r\n39.0643,0.86804285,0.639406,0.210052,0.104282\r\n39.0757,0.8675123,0.639408,0.210052,0.104283\r\n42.7671,0.86711295,0.639971,0.210052,0.104659\r\n42.7898,0.86665865,0.639975,0.210052,0.104661\r\n43.1994,0.8660722,0.640037,0.210052,0.104703\r\n46.4813,0.86549985,0.641366,0.210053,0.105037\r\n46.959,0.8649501,0.64156,0.210053,0.105085\r\n47.9829,0.86436135,0.641974,0.210053,0.10519\r\n50.1556,0.86377575,0.642854,0.210127,0.105411\r\n50.1954,0.8632461,0.64287,0.210128,0.105415\r\n50.7187,0.8626203,0.643082,0.210146,0.105468\r\n53.8982,0.86205755,0.64437,0.210255,0.105792\r\n53.9096,0.8614751,0.644375,0.210255,0.105793\r\n54.4784,0.8607904,0.644605,0.210274,0.105851\r\n57.601,0.86018795,0.645555,0.210381,0.106169\r\n57.6238,0.8595858,0.645562,0.210382,0.106171\r\n58.2324,0.8589965,0.645747,0.210402,0.106233\r\n59.1993,0.8583415,0.645944,0.210435,0.106332\r\n61.3209,0.85778445,0.646375,0.210543,0.106548\r\n61.3835,0.8571568,0.646387,0.210547,0.106554\r\n65.0351,0.8564697,0.647129,0.210733,0.106926\r\n65.729,0.855842,0.64727,0.210768,0.106997\r\n68.7379,0.8551763,0.648489,0.210922,0.107303\r\n68.7493,0.85451235,0.648493,0.210922,0.107304\r\n69.4887,0.85392275,0.648793,0.21096,0.10738\r\n72.4634,0.8533399,0.649998,0.211112,0.107682\r\n72.6113,0.85276495,0.650058,0.21112,0.107698\r\n73.2484,0.8522159,0.650316,0.211152,0.107762\r\n74.1584,0.8516263,0.650684,0.211199,0.107855\r\n76.1776,0.8510358,0.651502,0.211199,0.108061\r\n77.008,0.85048425,0.651838,0.211199,0.108145\r\n79.8918,0.84989255,0.653006,0.211199,0.108439\r\n80.0966,0.84931745,0.653089,0.211199,0.10846\r\n80.7677,0.84883275,0.653361,0.2112,0.108528\r\n84.5217,0.8482717,0.654503,0.2112,0.10891\r\n84.5445,0.84770405,0.656025,0.2112,0.108913\r\n87.3429,0.84714655,0.657158,0.211201,0.109198\r\n88.3042,0.8466449,0.657548,0.211201,0.109295\r\n89.1061,0.84609465,0.657872,0.211201,0.109377\r\n91.0514,0.84565725,0.65866,0.2113,0.109575\r\n91.0628,0.8452359,0.658665,0.211301,0.109576\r\n91.3244,0.84489215,0.658771,0.211314,0.109603\r\n92.0638,0.84445475,0.65907,0.211352,0.109678\r\n94.7769,0.8440218,0.660169,0.21149,0.109954\r\n95.8235,0.84358445,0.660593,0.211544,0.110061\r\n102.228,0.84314905,0.661894,0.21187,0.110713\r\n102.552,0.8427241,0.66196,0.211887,0.110746\r\n103.32,0.8423152,0.662116,0.211926,0.110824\r\n104.065,0.84193745,0.662343,0.211964,0.1109\r\n107.074,0.8415552,0.663258,0.212067,0.111207\r\n113.427,0.8412329,0.664549,0.212283,0.111854\r\n113.78,0.84091475,0.664621,0.212295,0.111889\r\n114.571,0.84060525,0.664782,0.212322,0.11197\r\n115.282,0.84034185,0.665069,0.212346,0.112042\r\n118.33,0.84008615,0.666304,0.212424,0.112353\r\n121.265,0.8398611,0.667493,0.212499,0.112652\r\n122.09,0.8396703,0.667827,0.212521,0.112708\r\n125.844,0.83950655,0.668969,0.212617,0.112963\r\n128.387,0.83939475,0.669999,0.212682,0.113136\r\n129.604,0.839246,0.670492,0.212713,0.113218\r\n130.235,0.8391064,0.670747,0.212729,0.113261\r\n132.488,0.83898345,0.671659,0.212882,0.113414\r\n133.363,0.8389161,0.672014,0.212942,0.113504\r\n133.386,0.83887005,0.673536,0.212943,0.113506\r\n139.603,0.83889405,0.673959,0.213366,0.114139\r\n141.457,0.83892495,0.674085,0.213492,0.114328\r\n143.715,0.8389607,0.674238,0.213607,0.114558\r\n144.608,0.83900535,0.674299,0.213653,0.114618\r\n147.083,0.83907925,0.674425,0.213779,0.114787\r\n154.562,0.8391752,0.674807,0.214161,0.115295\r\n154.938,0.83927215,0.674826,0.21418,0.11532\r\n156.416,0.83937105,0.674902,0.214255,0.115421\r\n159.567,0.8394766,0.675062,0.214469,0.115635\r\n166.16,0.83957625,0.674839,0.214917,0.116083\r\n167.639,0.83967085,0.674789,0.215018,0.116196\r\n169.516,0.83976335,0.674726,0.215114,0.11634\r\n170.773,0.83984635,0.674684,0.215178,0.116436\r\n181.124,0.83992285,0.674158,0.215706,0.117227\r\n182.598,0.8399973,0.674083,0.215781,0.117377\r\n184.463,0.84007065,0.673988,0.215971,0.117567\r\n185.709,0.84014025,0.673925,0.216098,0.117694\r\n188.61,0.84020835,0.673851,0.216393,0.117989\r\n190.083,0.8402765,0.673814,0.216543,0.118089\r\n199.411,0.84033455,0.673578,0.217019,0.118723\r\n199.832,0.84038895,0.673567,0.217041,0.118752\r\n200.651,0.8404432,0.673547,0.217083,0.118794\r\n205.042,0.8405039,0.673099,0.217307,0.119018\r\n211.845,0.84056145,0.672407,0.217769,0.119365\r\n214.364,0.84060215,0.672236,0.21794,0.119493\r\n214.791,0.84064985,0.672207,0.217969,0.119515\r\n216.264,0.840705,0.672107,0.218069,0.119615\r\n218.107,0.84076875,0.671982,0.218163,0.119741\r\n223.044,0.84083055,0.671647,0.218415,0.120076\r\n225.587,0.8408919,0.671475,0.218545,0.120249\r\n226.013,0.84096855,0.671446,0.218567,0.120278\r\n231.223,0.84105545,0.671093,0.218833,0.120632\r\n234.243,0.841151,0.670888,0.21914,0.120837\r\n236.809,0.8412487,0.670364,0.219401,0.121012\r\n237.235,0.8413691,0.670277,0.219445,0.121041\r\n241.695,0.84149335,0.669367,0.219899,0.121344\r\n242.451,0.841646,0.669316,0.219976,0.121395\r\n248.037,0.84180185,0.668937,0.220356,0.121775\r\n248.457,0.84199145,0.668909,0.220384,0.121803\r\n252.894,0.8421865,0.668608,0.220686,0.12203\r\n253.673,0.84239465,0.668449,0.220739,0.12207\r\n255.527,0.84260945,0.66807,0.220928,0.122164\r\n260.345,0.84282335,0.667087,0.221418,0.12241\r\n263.417,0.84303785,0.666617,0.221731,0.122567\r\n264.901,0.84325755,0.66639,0.221882,0.122668\r\n267.802,0.8434766,0.665946,0.222079,0.122865\r\n270.487,0.84368635,0.665536,0.222262,0.123047\r\n274.223,0.8438835,0.664964,0.222516,0.123301\r\n274.639,0.84407415,0.664901,0.222544,0.123329\r\n275.259,0.84426335,0.664806,0.222586,0.123351\r\n276.123,0.8444349,0.664718,0.222645,0.12338\r\n281.697,0.84459095,0.66415,0.223212,0.12357\r\n285.855,0.84473745,0.663727,0.223636,0.123712\r\n286.452,0.8448807,0.663666,0.223697,0.123752\r\n287.351,0.84499295,0.663483,0.223788,0.123813\r\n293.904,0.8450979,0.662145,0.224233,0.124259\r\n296.639,0.8451744,0.661727,0.224419,0.124445\r\n297.077,0.8452477,0.66166,0.224449,0.124474\r\n298.573,0.84528715,0.661431,0.224551,0.124551\r\n301.36,0.8453207,0.661005,0.224835,0.124693\r\n305.074,0.84534015,0.659483,0.225213,0.124883\r\n309.801,0.8453473,0.65876,0.225694,0.125124\r\n311.587,0.84534665,0.658487,0.225876,0.125215\r\n312.036,0.8453422,0.658419,0.225922,0.125238\r\n312.531,0.8453304,0.658343,0.225972,0.125255\r\n321.029,0.8453118,0.657478,0.226837,0.125544\r\n323.253,0.84529315,0.657251,0.227064,0.12562\r\n323.725,0.8452745,0.657203,0.227112,0.125632\r\n326.535,0.845257,0.656052,0.227398,0.125704\r\n327.439,0.8452409,0.655681,0.22749,0.125727\r\n328.514,0.8452281,0.655241,0.227599,0.125755\r\n331.153,0.8452296,0.65416,0.227957,0.125822\r\n338.206,0.8452324,0.651634,0.228914,0.126003\r\n338.587,0.8452363,0.651498,0.228966,0.126016\r\n338.61,0.84524995,0.65302,0.228969,0.126017\r\n339.748,0.8452693,0.652082,0.229123,0.126055\r\n341.488,0.84528865,0.650647,0.229242,0.126115\r\n342.302,0.84530805,0.649976,0.229297,0.126142\r\n346.016,0.84532745,0.648454,0.229549,0.126269\r\n348.962,0.8453468,0.647248,0.22975,0.126369\r\n349.423,0.8453661,0.647059,0.229781,0.126385\r\n349.73,0.8453854,0.646933,0.229802,0.126393\r\n350.97,0.84540475,0.646425,0.229886,0.126425\r\n353.444,0.8454241,0.645411,0.230263,0.126488\r\n356.436,0.8454434,0.644186,0.23072,0.126565\r\n357.158,0.84546265,0.64389,0.23083,0.126583\r\n358.461,0.84548195,0.643356,0.231029,0.126616\r\n360.872,0.84550125,0.642368,0.231519,0.126678\r\n364.376,0.84551945,0.641833,0.232231,0.126768\r\n365.957,0.84553625,0.641591,0.232552,0.126808\r\n368.329,0.8455497,0.641228,0.232793,0.126869\r\n371.384,0.8455558,0.639977,0.233104,0.126947\r\n372.043,0.8455745,0.639707,0.233172,0.126964\r\n377.185,0.8455987,0.638657,0.233695,0.127096\r\n379.33,0.84562185,0.638219,0.234022,0.127151\r\n379.495,0.84565225,0.638186,0.234047,0.127156\r\n383.214,0.8456864,0.637045,0.234615,0.127283\r\n384.676,0.84572475,0.636446,0.234838,0.127333\r\n386.343,0.84576755,0.635763,0.235008,0.12739\r\n386.929,0.84583795,0.635523,0.235067,0.12741\r\n390.546,0.8459089,0.632541,0.235436,0.127533\r\n390.62,0.84598005,0.63248,0.235443,0.127535\r\n390.643,0.84607545,0.634002,0.235445,0.127535\r\n393.834,0.84617595,0.631371,0.23577,0.127617\r\n394.334,0.8462924,0.630958,0.235821,0.12763\r\n395.904,0.84643065,0.630315,0.235981,0.12767\r\n398.048,0.8466022,0.629437,0.236308,0.127725\r\n401.33,0.84679275,0.628092,0.236809,0.127809\r\n401.336,0.8469876,0.62809,0.23681,0.127809\r\n401.763,0.84719905,0.627915,0.236875,0.12782\r\n403.395,0.84742765,0.627246,0.237124,0.127862\r\n405.084,0.84765855,0.626554,0.237467,0.127905\r\n405.477,0.84787655,0.626394,0.237547,0.127915\r\n405.499,0.84814385,0.62639,0.237551,0.127916\r\n410.892,0.8484279,0.625565,0.238647,0.127916\r\n412.581,0.84870515,0.625307,0.238905,0.127917\r\n412.934,0.84900695,0.625253,0.238959,0.127917\r\n416.648,0.84931775,0.623732,0.239525,0.127917\r\n418.383,0.849637,0.623377,0.23979,0.127918\r\n420.072,0.8499315,0.623033,0.240047,0.127918\r\n420.447,0.8502533,0.622956,0.240105,0.127918\r\n424.099,0.85058845,0.622211,0.240662,0.127918\r\n425.873,0.85091215,0.621939,0.240932,0.127919\r\n427.563,0.85127135,0.621681,0.241276,0.127919\r\n431.556,0.851633,0.62107,0.242087,0.12792\r\n433.37,0.85199005,0.620327,0.242456,0.12792\r\n435.059,0.8523518,0.619635,0.242713,0.12792\r\n435.071,0.85269655,0.61963,0.242715,0.12792\r\n435.27,0.8530382,0.619549,0.242745,0.12792\r\n435.395,0.85339805,0.61953,0.242764,0.12792\r\n438.819,0.8537615,0.619006,0.243287,0.128037\r\n440.861,0.8541399,0.618694,0.243598,0.128106\r\n442.727,0.85455455,0.618408,0.243883,0.12817\r\n446.316,0.8549154,0.616938,0.24443,0.128292\r\n446.441,0.8552604,0.616887,0.244449,0.128297\r\n446.611,0.85561595,0.616852,0.244475,0.128302\r\n448.352,0.8559848,0.616497,0.244741,0.128303\r\n453.807,0.85634215,0.615383,0.245481,0.128303\r\n453.892,0.856761,0.615366,0.245492,0.128303\r\n457.566,0.85718725,0.614804,0.245991,0.128304\r\n459.585,0.85759465,0.614495,0.246265,0.128304\r\n461.349,0.85798955,0.614225,0.246534,0.128305\r\n461.559,0.85838945,0.614139,0.246566,0.128305\r\n465.063,0.858759,0.612704,0.2471,0.128305\r\n467.076,0.8591189,0.612396,0.247407,0.128305\r\n468.794,0.85948055,0.612133,0.247669,0.128306\r\n472.52,0.85980425,0.611564,0.248238,0.128306\r\n472.537,0.86013115,0.611557,0.24824,0.128306\r\n474.567,0.86046105,0.610725,0.24855,0.128307\r\n476.234,0.86075685,0.610042,0.248889,0.128307\r\n476.507,0.86104085,0.609986,0.248944,0.128307\r\n480.016,0.8613163,0.60927,0.249657,0.128307\r\n482.064,0.86156355,0.608852,0.250073,0.128308\r\n483.685,0.86182875,0.608521,0.250238,0.128308\r\n487.405,0.86211155,0.60738,0.250617,0.128309\r\n491.119,0.86238175,0.605859,0.250995,0.128309\r\n491.244,0.86261035,0.605833,0.251008,0.128309\r\n491.455,0.8628372,0.60579,0.25103,0.128309\r\n493.292,0.8629897,0.605415,0.251217,0.128263\r\n498.57,0.86313195,0.604338,0.252022,0.128129\r\n500.783,0.86326525,0.603431,0.252359,0.128073\r\n502.284,0.8633975,0.602816,0.252967,0.128035\r\n504.542,0.8635119,0.602124,0.253882,0.127978\r\n506.004,0.86361605,0.601675,0.254031,0.127941\r\n506.192,0.8637104,0.601637,0.25405,0.127936\r\n506.397,0.8637499,0.601595,0.254071,0.127931\r\n513.455,0.86378825,0.600154,0.25479,0.127932\r\n513.654,0.8637998,0.600073,0.25481,0.127932\r\n515.77,0.86379285,0.599206,0.255025,0.127932\r\n517.169,0.8637473,0.598633,0.25531,0.127933\r\n520.884,0.8636905,0.597111,0.256064,0.127933\r\n521.105,0.8636209,0.597043,0.256109,0.127933\r\n521.344,0.8635234,0.59697,0.256158,0.127933\r\n523.267,0.8633933,0.59638,0.256548,0.127934\r\n524.603,0.863242,0.59597,0.256752,0.127934\r\n524.831,0.86307105,0.595924,0.256787,0.127934\r\n528.562,0.8628833,0.595162,0.257356,0.127934\r\n530.758,0.86268565,0.594714,0.257691,0.127935\r\n532.055,0.86245955,0.594449,0.257889,0.127935\r\n535.769,0.86223155,0.592928,0.258456,0.127935\r\n536.019,0.8620007,0.59289,0.258494,0.127935\r\n536.292,0.8617499,0.592848,0.258535,0.127936\r\n538.249,0.8614483,0.592549,0.258834,0.127869\r\n543.226,0.8611442,0.591787,0.259845,0.127701\r\n543.47,0.8608179,0.591737,0.259895,0.127693\r\n545.745,0.86050965,0.591273,0.260357,0.127616\r\n547.497,0.86017,0.590915,0.260535,0.127557\r\n550.677,0.85984945,0.590266,0.260859,0.127557\r\n550.921,0.85948995,0.590166,0.260884,0.127557\r\n554.391,0.8591645,0.588745,0.261237,0.127558\r\n554.641,0.85879355,0.588707,0.261263,0.127558\r\n556.973,0.8583967,0.58835,0.2615,0.127558\r\n558.355,0.85805755,0.588139,0.261711,0.127558\r\n558.367,0.8576787,0.588137,0.261713,0.127558\r\n561.848,0.85729295,0.587605,0.262244,0.127559\r\n562.445,0.85689525,0.58736,0.262335,0.127559\r\n564.464,0.8564975,0.586533,0.262643,0.127508\r\n565.562,0.8561059,0.586083,0.26281,0.12748\r\n565.806,0.8556957,0.586046,0.262848,0.127474\r\n571.955,0.85527925,0.585105,0.263786,0.127318\r\n573.019,0.85486375,0.584943,0.26393,0.127291\r\n573.263,0.8544261,0.584893,0.263963,0.127285\r\n577.387,0.8540078,0.584051,0.264523,0.127181\r\n580.47,0.8535895,0.583422,0.264941,0.127181\r\n580.714,0.85317555,0.583384,0.264974,0.127181\r\n583.188,0.8527432,0.583006,0.265309,0.127182\r\n587.926,0.85234145,0.582281,0.266032,0.127182\r\n588.171,0.8519166,0.582256,0.26607,0.127182\r\n590.679,0.8515325,0.582001,0.266452,0.127183\r\n591.897,0.8511478,0.581877,0.266576,0.127183\r\n592.335,0.85077735,0.581832,0.266621,0.127183\r\n595.622,0.8504233,0.581498,0.266955,0.1271\r\n599.12,0.85002175,0.581141,0.267312,0.127011\r\n601.907,0.8496569,0.580858,0.267595,0.126941\r\n606.822,0.84930125,0.580357,0.268345,0.126816\r\n607.277,0.8489766,0.580311,0.268414,0.126805\r\n609.398,0.8486688,0.580095,0.268738,0.126805\r\n610.314,0.8483648,0.580002,0.268831,0.126805\r\n617.771,0.8481078,0.578861,0.269591,0.126806\r\n620.626,0.847859,0.578424,0.269881,0.126807\r\n621.769,0.84761145,0.57825,0.270056,0.126807\r\n622.224,0.8474022,0.57818,0.270125,0.126807\r\n625.228,0.84722075,0.577721,0.270583,0.126706\r\n628.117,0.84703505,0.577574,0.271024,0.126608\r\n633.429,0.8468655,0.577304,0.272103,0.126428\r\n635.614,0.8467308,0.577193,0.272547,0.126429\r\n636.711,0.84658625,0.577137,0.272715,0.126429\r\n640.164,0.84645735,0.576962,0.273241,0.126429\r\n643.104,0.8463246,0.576763,0.27369,0.12643\r\n647.911,0.84618675,0.576437,0.274667,0.12643\r\n648.377,0.8460369,0.576405,0.274761,0.12643\r\n650.601,0.84588335,0.576254,0.275213,0.126374\r\n651.363,0.84573385,0.576203,0.275329,0.126355\r\n651.625,0.8456391,0.576185,0.275369,0.126348\r\n651.636,0.84544925,0.576184,0.275371,0.126348\r\n655.345,0.84514605,0.575933,0.275937,0.126254\r\n658.092,0.84480395,0.575746,0.276356,0.126185\r\n659.036,0.8445379,0.575682,0.276548,0.126161\r\n659.059,0.8441274,0.575681,0.276552,0.12616\r\n662.563,0.8437091,0.575443,0.277264,0.126071\r\n662.75,0.84327975,0.575437,0.277302,0.126067\r\n663.319,0.84280775,0.575418,0.277418,0.126052\r\n665.589,0.84232145,0.575341,0.277879,0.125976\r\n666.465,0.84187995,0.575311,0.278013,0.125946\r\n666.476,0.84132705,0.575311,0.278014,0.125946\r\n670.179,0.84071615,0.575186,0.278579,0.12582\r\n673.08,0.84010285,0.575088,0.279022,0.125722\r\n673.768,0.83949695,0.575065,0.279162,0.125699\r\n673.893,0.8389009,0.575058,0.279187,0.125695\r\n674.524,0.83825265,0.575026,0.279315,0.125673\r\n680.576,0.8376421,0.574719,0.280545,0.125674\r\n681.344,0.83697045,0.57468,0.280856,0.125674\r\n684.336,0.8362858,0.574528,0.282068,0.125675\r\n685.053,0.8356011,0.574491,0.282177,0.125675\r\n685.064,0.83493695,0.574491,0.282179,0.125675\r\n688.704,0.83438455,0.574306,0.282734,0.125676\r\n688.778,0.83387285,0.574301,0.282745,0.125676\r\n689.472,0.83329105,0.574254,0.282851,0.125676\r\n691.827,0.832865,0.574094,0.28321,0.125616\r\n696.258,0.83252055,0.573794,0.283661,0.125504\r\n699.904,0.83219005,0.573546,0.284033,0.125412\r\n700.017,0.8319821,0.573535,0.284044,0.125409\r\n700.04,0.83185315,0.573533,0.284047,0.125408\r\n703.055,0.8317012,0.573226,0.284354,0.125332\r\n703.8,0.8317015,0.57315,0.284404,0.125313\r\n703.811,0.83179925,0.573149,0.284405,0.125313\r\n703.822,0.8319872,0.573147,0.284406,0.125312\r\n703.845,0.8321892,0.573145,0.284407,0.125312\r\n704.414,0.83246225,0.573087,0.284446,0.125297\r\n707.605,0.83287315,0.572762,0.284663,0.125217\r\n707.628,0.833368,0.57276,0.284664,0.125216\r\n707.65,0.8339746,0.572758,0.284666,0.125216\r\n711.097,0.83460265,0.572407,0.2849,0.125128\r\n711.399,0.8352307,0.572386,0.284921,0.125121\r\n711.41,0.8360094,0.572385,0.284921,0.12512\r\n711.433,0.83684725,0.572384,0.284923,0.12512\r\n714.277,0.83770185,0.572191,0.285116,0.125048\r\n715.193,0.83862655,0.572129,0.285256,0.125025\r\n715.215,0.83961605,0.572127,0.285259,0.125024\r\n715.238,0.84056975,0.572126,0.285263,0.125023\r\n715.249,0.8415592,0.572125,0.285265,0.125023\r\n715.261,0.8425487,0.572124,0.285266,0.125023\r\n719.02,0.84355325,0.571869,0.28584,0.124928\r\n719.043,0.84461885,0.571868,0.285843,0.124927\r\n719.066,0.84572245,0.571866,0.285847,0.124927\r\n719.089,0.8467907,0.571865,0.28585,0.124926\r\n719.356,0.84780735,0.571847,0.285891,0.124919\r\n721.768,0.8488179,0.571683,0.286259,0.12492\r\n722.297,0.8497693,0.571647,0.286331,0.12492\r\n722.837,0.8506827,0.571592,0.286404,0.12492\r\n722.848,0.8515961,0.571591,0.286406,0.12492\r\n722.871,0.85255055,0.571589,0.286409,0.12492\r\n722.894,0.85357805,0.571586,0.286412,0.12492\r\n726.654,0.8546436,0.571204,0.286922,0.12492\r\n726.676,0.8556711,0.571201,0.286925,0.12492\r\n726.688,0.85667905,0.5712,0.286926,0.12492\r\n726.699,0.8577446,0.571199,0.286928,0.12492\r\n730.459,0.85881015,0.570816,0.287438,0.124921\r\n730.482,0.8598757,0.570814,0.287441,0.124921\r\n730.504,0.8609793,0.570811,0.287444,0.124921\r\n730.527,0.86204525,0.570809,0.287447,0.124921\r\n730.567,0.8630598,0.570805,0.287453,0.124921\r\n733.001,0.8640032,0.570557,0.287783,0.124859\r\n733.49,0.8648785,0.570507,0.287857,0.124847\r\n734.275,0.86571575,0.570387,0.287977,0.124827\r\n734.287,0.8666264,0.570386,0.287979,0.124827\r\n734.309,0.86758865,0.570382,0.287982,0.124826\r\n734.332,0.8686216,0.570379,0.287986,0.124826\r\n734.355,0.8697252,0.570375,0.287989,0.124825\r\n734.378,0.87082875,0.570372,0.287993,0.124824\r\n734.389,0.87189425,0.57037,0.287995,0.124824\r\n734.4,0.8729598,0.570368,0.287996,0.124824\r\n738.16,0.87402535,0.569793,0.28857,0.124729\r\n738.183,0.8750909,0.56979,0.288573,0.124728\r\n738.206,0.8761945,0.569786,0.288577,0.124728\r\n738.228,0.8772981,0.569783,0.28858,0.124727\r\n738.24,0.8783636,0.569781,0.288582,0.124727\r\n738.251,0.8793984,0.569779,0.288584,0.124726\r\n740.492,0.8803663,0.569437,0.288925,0.12467\r\n740.947,0.88127965,0.569367,0.288956,0.124658\r\n742.011,0.8822687,0.569204,0.289029,0.124631\r\n742.034,0.8833091,0.569201,0.28903,0.124631\r\n742.056,0.88442065,0.569197,0.289032,0.12463\r\n742.079,0.88556225,0.569194,0.289033,0.12463\r\n742.09,0.88670385,0.569192,0.289034,0.124629\r\n742.102,0.88784545,0.56919,0.289035,0.124629\r\n742.125,0.8889795,0.569187,0.289036,0.124628\r\n745.509,0.89004505,0.568669,0.289266,0.124543\r\n745.884,0.8911106,0.568612,0.289292,0.124533\r\n745.907,0.8922142,0.568608,0.289294,0.124533\r\n745.93,0.8933178,0.568605,0.289295,0.124532\r\n745.941,0.8943833,0.568603,0.289296,0.124532\r\n745.953,0.8954488,0.568601,0.289297,0.124531\r\n745.975,0.8965143,0.568598,0.289298,0.124531\r\n745.998,0.8975798,0.568595,0.2893,0.12453\r\n746.021,0.89865555,0.568591,0.289301,0.12453\r\n748.404,0.8997211,0.568227,0.289463,0.124469\r\n749.78,0.9007793,0.568086,0.289557,0.124435\r\n749.792,0.90186635,0.568085,0.289558,0.124434\r\n749.803,0.90300795,0.568084,0.289558,0.124434\r\n749.826,0.90414955,0.568082,0.28956,0.124433\r\n749.849,0.90529115,0.568079,0.289561,0.124433\r\n749.871,0.90643275,0.568077,0.289563,0.124432\r\n749.894,0.90757435,0.568075,0.289565,0.124432\r\n749.906,0.90871595,0.568074,0.289565,0.124431\r\n749.917,0.90981785,0.568072,0.289566,0.124431\r\n751.714,0.91089095,0.567889,0.289688,0.124386\r\n753.677,0.91203255,0.56769,0.289888,0.124336\r\n753.699,0.91317415,0.567687,0.28989,0.124335\r\n753.722,0.91431575,0.567685,0.289893,0.124335\r\n753.745,0.91545735,0.567683,0.289895,0.124334\r\n753.756,0.91659895,0.567682,0.289896,0.124334\r\n753.768,0.9177406,0.56768,0.289897,0.124334\r\n757.527,0.91888225,0.567298,0.29028,0.124238\r\n757.55,0.9200239,0.567295,0.290282,0.124238\r\n757.573,0.92115825,0.567293,0.290285,0.124237\r\n759.598,0.92228205,0.567087,0.290491,0.124186\r\n760.451,0.92338565,0.566913,0.290578,0.124164\r\n761.333,0.92448925,0.566733,0.290668,0.124135\r\n761.344,0.9255548,0.56673,0.290669,0.124134\r\n761.355,0.9265764,0.566728,0.29067,0.124134\r\n762.942,0.9275659,0.566404,0.290831,0.12408\r\n765.115,0.9285554,0.565961,0.291163,0.124007\r\n765.138,0.92958295,0.565956,0.291166,0.124006\r\n765.16,0.93061065,0.565951,0.29117,0.124005\r\n767.049,0.93163985,0.565566,0.291458,0.123941\r\n768.92,0.93266735,0.565184,0.291743,0.123878\r\n768.931,0.9336568,0.565181,0.291745,0.123878\r\n768.943,0.9345473,0.565179,0.291747,0.123877\r\n770.433,0.93534295,0.564875,0.291974,0.123827\r\n771.656,0.9360851,0.564625,0.292469,0.123786\r\n772.663,0.936694,0.56442,0.292877,0.12376\r\n774.193,0.93720765,0.564108,0.293497,0.123721\r\n774.5,0.93753125,0.564045,0.293559,0.123714\r\n776.331,0.93785485,0.563483,0.293931,0.123667\r\n776.354,0.93821355,0.563476,0.293936,0.123667\r\n776.377,0.9384011,0.563469,0.293941,0.123666\r\n778.22,0.93842025,0.562904,0.294315,0.123619\r\n780.023,0.93847745,0.562536,0.294681,0.123574\r\n780.046,0.93853465,0.562531,0.294686,0.123573\r\n780.057,0.9383971,0.562529,0.294688,0.123573\r\n781.689,0.9380167,0.562196,0.29502,0.123532\r\n783.686,0.93759825,0.561788,0.295325,0.123481\r\n783.697,0.9371798,0.561786,0.295326,0.123481\r\n783.72,0.93678025,0.561781,0.29533,0.12348\r\n783.737,0.93615015,0.561778,0.295332,0.12348\r\n785.671,0.93538965,0.561383,0.295627,0.123431\r\n786.598,0.9345146,0.561241,0.295769,0.123407\r\n787.366,0.9337385,0.561124,0.295886,0.123381\r\n787.389,0.9330573,0.56112,0.295889,0.123381\r\n787.411,0.9321672,0.561117,0.295893,0.12338\r\n789.18,0.93114005,0.560846,0.296163,0.12332\r\n791.012,0.9302081,0.560566,0.296442,0.123258\r\n791.035,0.92942815,0.560563,0.296446,0.123257\r\n791.046,0.92861015,0.560561,0.296447,0.123257\r\n791.057,0.92779215,0.560559,0.296449,0.123257\r\n791.08,0.9268231,0.560556,0.296453,0.123256\r\n793.128,0.9257768,0.560242,0.296765,0.123187\r\n794.658,0.92469245,0.560008,0.296998,0.123135\r\n794.669,0.9236081,0.560007,0.297,0.123135\r\n794.68,0.92276245,0.560005,0.297002,0.123134\r\n794.703,0.92219175,0.560002,0.297005,0.123133\r\n794.726,0.9212717,0.559998,0.297009,0.123133\r\n796.671,0.9200944,0.559701,0.297305,0.123067\r\n797.803,0.9187817,0.559527,0.297478,0.123029\r\n798.292,0.9176997,0.559453,0.297553,0.123029\r\n798.304,0.91678615,0.559451,0.297554,0.123029\r\n798.326,0.9160252,0.559447,0.297558,0.123029\r\n798.349,0.91526425,0.559444,0.297561,0.123029\r\n798.372,0.9140233,0.55944,0.297565,0.123029\r\n800.584,0.9127637,0.559102,0.297902,0.123029\r\n801.915,0.9117744,0.55883,0.298105,0.123029\r\n801.927,0.9107851,0.558828,0.298107,0.123029\r\n801.95,0.9098339,0.558823,0.298111,0.123029\r\n801.972,0.90892075,0.558819,0.298114,0.123029\r\n801.995,0.9080076,0.558814,0.298118,0.123029\r\n802.018,0.906959,0.558809,0.298121,0.123029\r\n804.162,0.9058556,0.558372,0.298448,0.12303\r\n805.539,0.9047522,0.558091,0.298728,0.12303\r\n805.55,0.90368685,0.558088,0.29873,0.12303\r\n805.573,0.9026215,0.558084,0.298735,0.12303\r\n805.595,0.90155615,0.558079,0.298739,0.12303\r\n805.618,0.9008782,0.558075,0.298744,0.12303\r\n805.641,0.89986215,0.55807,0.298749,0.12303\r\n808.036,0.89868265,0.557581,0.299235,0.12303\r\n809.15,0.89750315,0.557411,0.299462,0.12303\r\n809.162,0.8962856,0.557409,0.299464,0.12303\r\n809.173,0.89506805,0.557407,0.299466,0.12303\r\n809.196,0.8938505,0.557404,0.299471,0.12303\r\n809.219,0.8931129,0.5574,0.299476,0.12303\r\n809.241,0.8926944,0.557397,0.29948,0.12303\r\n809.264,0.8917186,0.557393,0.299485,0.12303\r\n811.659,0.8904299,0.557027,0.299971,0.123031\r\n812.751,0.8890601,0.55686,0.300193,0.123031\r\n812.774,0.88765225,0.556857,0.300198,0.12303\r\n812.785,0.88620635,0.556855,0.3002,0.12303\r\n812.796,0.8852769,0.556853,0.300202,0.12303\r\n812.819,0.88470615,0.55685,0.300207,0.123029\r\n812.842,0.88417345,0.556846,0.300212,0.123029\r\n812.865,0.88308495,0.556843,0.300216,0.123028\r\n815.492,0.8817532,0.556441,0.30075,0.122961\r\n816.408,0.8803834,0.556254,0.300936,0.122938\r\n816.42,0.8790136,0.556251,0.300939,0.122938\r\n816.442,0.87823915,0.556247,0.300943,0.122937\r\n816.465,0.87778255,0.556242,0.300948,0.122937\r\n816.488,0.8769496,0.556238,0.300953,0.122936\r\n819.155,0.87596035,0.555693,0.301495,0.122869\r\n820.088,0.8749711,0.555503,0.301637,0.122845\r\n820.111,0.87398185,0.555498,0.30164,0.122845\r\n820.134,0.87256015,0.555493,0.301644,0.122844\r\n822.943,0.8709621,0.55492,0.302072,0.122773\r\n823.734,0.8699594,0.554799,0.302193,0.122753\r\n823.757,0.86923155,0.554795,0.302196,0.122752\r\n823.768,0.8685467,0.554794,0.302198,0.122752\r\n823.78,0.8678999,0.554792,0.3022,0.122752\r\n823.802,0.8667916,0.554788,0.302203,0.122751\r\n826.646,0.86549795,0.554353,0.302637,0.122679\r\n827.38,0.86416625,0.554241,0.302786,0.122661\r\n827.391,0.8627965,0.55424,0.302789,0.12266\r\n827.403,0.86198255,0.554238,0.302791,0.12266\r\n827.426,0.86141185,0.554234,0.302796,0.122659\r\n827.448,0.86076125,0.554231,0.3028,0.122659\r\n827.693,0.85957085,0.554193,0.30285,0.122653\r\n830.4,0.8582011,0.553779,0.3034,0.122469\r\n831.003,0.8567933,0.553687,0.303523,0.122428\r\n831.015,0.85576185,0.553685,0.303525,0.122427\r\n831.026,0.85492475,0.553684,0.303527,0.122427\r\n831.049,0.8540877,0.55368,0.303532,0.122425\r\n831.071,0.85325065,0.553677,0.303536,0.122424\r\n831.094,0.8522478,0.553673,0.303541,0.122422\r\n834.143,0.85125855,0.553207,0.30416,0.122215\r\n834.649,0.8502693,0.55313,0.304238,0.122181\r\n834.672,0.8493181,0.553126,0.304241,0.122179\r\n834.695,0.84840495,0.553123,0.304245,0.122178\r\n834.717,0.8470175,0.553119,0.304248,0.122176\r\n837.857,0.8459951,0.552639,0.304727,0.121963\r\n838.318,0.84515805,0.552545,0.304797,0.121932\r\n838.341,0.84435905,0.55254,0.304801,0.121931\r\n838.363,0.84356005,0.552536,0.304804,0.121929\r\n838.375,0.8426136,0.552533,0.304806,0.121928\r\n838.892,0.84121325,0.552428,0.304885,0.121893\r\n841.634,0.83980925,0.551868,0.305303,0.121824\r\n841.964,0.83894505,0.551801,0.305353,0.121816\r\n841.986,0.83822215,0.551796,0.305357,0.121815\r\n841.998,0.83749925,0.551794,0.305359,0.121815\r\n842.009,0.8368144,0.551791,0.30536,0.121814\r\n842.032,0.8357094,0.551787,0.305364,0.121814\r\n845.308,0.83449185,0.551118,0.305864,0.121731\r\n845.655,0.8332743,0.551065,0.305917,0.121722\r\n845.678,0.83245475,0.551061,0.30592,0.121722\r\n849.125,0.83171285,0.550534,0.306446,0.121634\r\n849.375,0.83092925,0.550496,0.306471,0.121628\r\n852.765,0.8300732,0.549978,0.306816,0.121542\r\n853.123,0.8291483,0.549941,0.306853,0.121533\r\n853.834,0.82865245,0.549869,0.306925,0.121515\r\n856.872,0.82826795,0.549559,0.307235,0.121412\r\n860.353,0.8278115,0.549205,0.307589,0.121295\r\n860.626,0.8273128,0.549177,0.307608,0.121285\r\n863.959,0.82678705,0.548838,0.307834,0.121173\r\n865.039,0.82636395,0.548728,0.307908,0.121136\r\n868.122,0.826345,0.548414,0.308117,0.121058\r\n871.575,0.82639885,0.548062,0.308352,0.120971\r\n875.152,0.8263801,0.547698,0.308716,0.12088\r\n875.613,0.82636765,0.547627,0.308763,0.120869\r\n879.981,0.82634385,0.546959,0.309208,0.120758\r\n882.609,0.82624595,0.546557,0.309475,0.120691\r\n882.803,0.8264952,0.546538,0.309495,0.120687\r\n883.11,0.8268189,0.546507,0.309542,0.120679\r\n883.121,0.8270848,0.546505,0.309544,0.120679\r\n890.294,0.82747565,0.545775,0.310638,0.120497\r\n890.601,0.8278999,0.545744,0.310669,0.120489\r\n893.803,0.8282983,0.545418,0.310995,0.120408\r\n894.923,0.8287172,0.545189,0.311109,0.12038\r\n898.092,0.82914495,0.544542,0.311432,0.120273\r\n901.254,0.82954325,0.543897,0.311754,0.120166\r\n901.521,0.82990615,0.543815,0.311781,0.120157\r\n904.974,0.8302708,0.542756,0.312133,0.12004\r\n906.129,0.8306145,0.541804,0.31225,0.120001\r\n908.665,0.8309473,0.539712,0.312508,0.119937\r\n908.688,0.83125205,0.541234,0.312511,0.119936\r\n909.319,0.8315479,0.541008,0.312575,0.11992\r\n912.749,0.8318336,0.53978,0.312924,0.119833\r\n916.122,0.8321368,0.538572,0.313439,0.119748\r\n920.24,0.8323972,0.538152,0.314067,0.119644\r\n920.547,0.8326327,0.538121,0.314088,0.119636\r\n921.071,0.8328874,0.538068,0.314123,0.119623\r\n927.316,0.8331531,0.537432,0.314548,0.119412\r\n931.462,0.8333818,0.536798,0.31483,0.119271\r\n931.769,0.83359475,0.536751,0.314861,0.119261\r\n932.276,0.83381965,0.536673,0.314913,0.119244\r\n934.773,0.8340239,0.536291,0.315167,0.119181\r\n935.512,0.8342538,0.536178,0.315242,0.119162\r\n942.229,0.8344619,0.535151,0.315926,0.118992\r\n942.69,0.83465545,0.534946,0.315973,0.11898\r\n942.997,0.83486195,0.53481,0.316004,0.118972\r\n947.218,0.8350718,0.532936,0.316434,0.118866\r\n953.366,0.8352776,0.530206,0.31706,0.118658\r\n953.918,0.83547155,0.530122,0.317116,0.118639\r\n954.219,0.8356868,0.530076,0.317137,0.118629\r\n958.423,0.835911,0.529433,0.317422,0.118487\r\n960.823,0.83613975,0.529066,0.317586,0.118426\r\n965.14,0.8363529,0.528184,0.317879,0.118317\r\n965.447,0.83655465,0.528122,0.317895,0.118309\r\n968.274,0.83677215,0.527545,0.318039,0.118237\r\n973.365,0.8369991,0.527026,0.318299,0.118109\r\n976.67,0.8372191,0.52669,0.318467,0.117885\r\n979.468,0.8374282,0.526405,0.31861,0.117695\r\n980.099,0.837659,0.526308,0.318642,0.117652\r\n984.564,0.83789925,0.525625,0.318794,0.117349\r\n986.925,0.8381317,0.525264,0.318875,0.117289\r\n987.897,0.8383778,0.525165,0.318908,0.117265\r\n991.316,0.83859135,0.524817,0.319025,0.117178\r\n991.64,0.8388191,0.524784,0.319033,0.11717\r\n998.118,0.83904785,0.524125,0.319199,0.117006\r\n999.12,0.8392642,0.523971,0.319224,0.116981\r\n999.506,0.83949055,0.523912,0.319234,0.116971\r\n1005.58,0.8397186,0.522984,0.31939,0.116766\r\n1006.27,0.83996635,0.522842,0.319407,0.116742\r\n1010.35,0.84019455,0.52201,0.319512,0.116605\r\n1010.71,0.84042235,0.521936,0.319521,0.116592\r\n1013.03,0.8406661,0.521463,0.31958,0.116534\r\n1020.48,0.84091205,0.520323,0.319771,0.116345\r\n1021.22,0.8411502,0.52021,0.31979,0.116326\r\n1021.58,0.84140215,0.520156,0.319808,0.116317\r\n1025.65,0.8416603,0.519532,0.320016,0.116214\r\n1027.94,0.8419128,0.519182,0.320133,0.116137\r\n1029.07,0.84217525,0.518721,0.32019,0.116099\r\n1031.65,0.84243805,0.517661,0.320322,0.116011\r\n1036.18,0.84269225,0.5172,0.320553,0.115858\r\n1036.86,0.842959,0.517131,0.320554,0.115835\r\n1040.29,0.8432343,0.516781,0.320554,0.115661\r\n1042.85,0.8435201,0.516521,0.320554,0.115531\r\n1044.04,0.8438084,0.516399,0.320555,0.11547\r\n1047.78,0.8441198,0.516018,0.320555,0.11528\r\n1050.31,0.84443755,0.515761,0.320556,0.115152\r\n1051.13,0.8447312,0.515678,0.320556,0.11511\r\n1051.79,0.8450567,0.51561,0.320578,0.115076\r\n1055.28,0.8454376,0.515255,0.320697,0.114959\r\n1059.04,0.8458354,0.514872,0.320825,0.114831\r\n1061.5,0.84622785,0.514621,0.320909,0.114748\r\n1062.35,0.84658735,0.514492,0.320938,0.11472\r\n1063,0.84697,0.514392,0.320971,0.114698\r\n1066.53,0.84737555,0.513852,0.321152,0.114608\r\n1068.96,0.8478015,0.513481,0.321275,0.114547\r\n1074.03,0.84822665,0.512965,0.321534,0.114418\r\n1077.3,0.8486456,0.512631,0.321701,0.114335\r\n1077.77,0.8490531,0.512584,0.321713,0.114324\r\n1077.94,0.84946,0.512566,0.321718,0.114319\r\n1080.15,0.84987735,0.512341,0.321774,0.114245\r\n1081.52,0.8503534,0.512387,0.321809,0.114198\r\n1089.02,0.8508045,0.512643,0.322001,0.113945\r\n1089.15,0.8512604,0.512647,0.322004,0.113941\r\n1091.37,0.851704,0.512723,0.322061,0.113828\r\n1092.26,0.85215435,0.512814,0.322084,0.113782\r\n1096.51,0.85267255,0.513246,0.322085,0.113567\r\n1100.27,0.85320765,0.513629,0.322085,0.113376\r\n1102.6,0.8537224,0.513866,0.322086,0.113257\r\n1104.08,0.85419275,0.513917,0.322086,0.113182\r\n1107.21,0.85465,0.514023,0.322086,0.113076\r\n1107.76,0.8551389,0.514042,0.322105,0.113057\r\n1111.51,0.8556418,0.51417,0.322233,0.112931\r\n1111.52,0.85614115,0.51417,0.322233,0.11293\r\n1113.82,0.85663075,0.514249,0.322312,0.112853\r\n1115.29,0.8571007,0.514548,0.322362,0.112803\r\n1118.42,0.85756,0.515185,0.322469,0.112644\r\n1119.01,0.85804025,0.515305,0.322514,0.112614\r\n1121.31,0.85857165,0.515772,0.322689,0.112497\r\n1126.51,0.859098,0.516565,0.323087,0.112233\r\n1128.8,0.85962135,0.516915,0.323262,0.112116\r\n1130.22,0.86009265,0.517204,0.323371,0.112044\r\n1133.39,0.8605676,0.517846,0.323612,0.112045\r\n1134,0.86107215,0.517971,0.323633,0.112045\r\n1136.3,0.86159955,0.518438,0.323712,0.112045\r\n1137.76,0.8621319,0.518661,0.323761,0.112045\r\n1141.44,0.86258885,0.519221,0.323887,0.112046\r\n1141.51,0.86306015,0.519233,0.323889,0.112042\r\n1141.52,0.8635337,0.519234,0.32389,0.112042\r\n1143.79,0.8639817,0.51958,0.323967,0.111926\r\n1144.6,0.86445765,0.519705,0.323995,0.111885\r\n1149.01,0.86491855,0.520377,0.32422,0.111661\r\n1151.28,0.8654122,0.520723,0.324335,0.111546\r\n1156.37,0.8658839,0.521414,0.324595,0.111287\r\n1156.51,0.8663639,0.521432,0.324602,0.111283\r\n1159.56,0.8668346,0.521847,0.324758,0.111179\r\n1162.51,0.8673106,0.522247,0.324859,0.111079\r\n1164,0.8678092,0.522699,0.324909,0.111029\r\n1166.27,0.86828145,0.523389,0.324986,0.110953\r\n1167.58,0.86873995,0.523655,0.325031,0.110908\r\n1167.76,0.8692183,0.523692,0.325037,0.110899\r\n1170.78,0.8696799,0.524306,0.32514,0.110746\r\n1171.51,0.87015975,0.524453,0.325177,0.110709\r\n1173.77,0.8705933,0.524912,0.325293,0.110594\r\n1173.79,0.8709975,0.526434,0.325294,0.110593\r\n1173.81,0.87136455,0.527956,0.325295,0.110592\r\n1173.83,0.8717453,0.529479,0.325296,0.11059\r\n1175.26,0.87213535,0.530054,0.325369,0.110518\r\n1177.59,0.87250855,0.531001,0.325488,0.110399\r\n1177.62,0.872876,0.532523,0.325489,0.110398\r\n1177.64,0.87323775,0.534045,0.32549,0.110397\r\n1179.02,0.8735946,0.534603,0.32556,0.110327\r\n1181.4,0.8739082,0.535568,0.325682,0.110206\r\n1182.51,0.8742406,0.536019,0.325739,0.11015\r\n1185.16,0.87455415,0.537091,0.325874,0.11006\r\n1185.74,0.874851,0.537267,0.325903,0.110041\r\n1186.51,0.8751853,0.537501,0.325943,0.110015\r\n1188.91,0.8754741,0.538233,0.326065,0.109933\r\n1188.94,0.87576335,0.539755,0.326067,0.109932\r\n1190.27,0.87606745,0.540294,0.326134,0.109887\r\n1192.69,0.876341,0.541277,0.326258,0.109805\r\n1192.72,0.8766148,0.5428,0.32626,0.109805\r\n1193.72,0.87688675,0.543205,0.326311,0.109771\r\n1196.48,0.8771682,0.544322,0.326451,0.109584\r\n1197.76,0.87747925,0.544582,0.326517,0.109497\r\n1200.7,0.8777983,0.54518,0.326667,0.109298\r\n1201.51,0.878128,0.545344,0.326667,0.109243\r\n1203.97,0.8784106,0.545845,0.326667,0.109075\r\n1204,0.8787021,0.547367,0.326667,0.109074\r\n1204.92,0.87899655,0.547741,0.326667,0.109011\r\n1205.25,0.8792885,0.547877,0.326667,0.109003\r\n1207.76,0.87956255,0.54889,0.326668,0.10894\r\n1207.78,0.8798663,0.550412,0.326668,0.108939\r\n1211.54,0.88016215,0.551935,0.326668,0.108844\r\n1212.75,0.8804923,0.552303,0.326669,0.108813\r\n1215.29,0.88082195,0.553077,0.326669,0.108749\r\n1215.65,0.8811159,0.55322,0.326669,0.10874\r\n1216.5,0.8814353,0.553568,0.326698,0.108718\r\n1219.05,0.88173595,0.5546,0.326785,0.108654\r\n1219.86,0.88204805,0.554927,0.326813,0.108633\r\n1222.81,0.88236025,0.556122,0.326913,0.108533\r\n1222.83,0.88267665,0.557644,0.326914,0.108533\r\n1224,0.88299175,0.558116,0.326954,0.108493\r\n1226.59,0.8832973,0.559167,0.327042,0.108405\r\n1226.62,0.88357485,0.560689,0.327043,0.108405\r\n1226.86,0.883871,0.560788,0.327051,0.108396\r\n1230.38,0.88414225,0.562212,0.327052,0.108278\r\n1230.4,0.8844183,0.563734,0.327052,0.108277\r\n1231.07,0.88468825,0.564003,0.327052,0.108254\r\n1231.49,0.8849684,0.564176,0.327052,0.108233\r\n1234.16,0.88522325,0.565256,0.327052,0.108097\r\n1234.18,0.885497,0.566778,0.327052,0.108096\r\n1237.94,0.88573265,0.568301,0.327053,0.107905\r\n1237.96,0.88596115,0.569823,0.327053,0.107904\r\n1238.98,0.88616495,0.570233,0.327053,0.107852\r\n1241.72,0.8863635,0.571346,0.327054,0.107713\r\n1241.75,0.88654935,0.572868,0.327054,0.107712\r\n1241.81,0.8867139,0.572881,0.327054,0.107709\r\n1242.72,0.88688145,0.573065,0.327054,0.107662\r\n1246,0.88703,0.573732,0.327054,0.107496\r\n1249.24,0.88717835,0.574391,0.327055,0.107386\r\n1249.27,0.88731495,0.575913,0.327055,0.107385\r\n1249.29,0.8874446,0.577435,0.327055,0.107384\r\n1253.05,0.8875741,0.578958,0.327055,0.107257\r\n1253.07,0.8877047,0.58048,0.327055,0.107257\r\n1253.94,0.88780855,0.580745,0.327055,0.107227\r\n1256.76,0.8879122,0.581601,0.327056,0.107132\r\n1256.83,0.8880108,0.581622,0.327056,0.10713\r\n1256.85,0.88810615,0.583144,0.327056,0.107129\r\n1257.21,0.8881879,0.583289,0.327056,0.107117\r\n1260.61,0.8882696,0.584667,0.327056,0.107002\r\n1264.37,0.88833215,0.586189,0.327057,0.106875\r\n1265.15,0.88833365,0.586507,0.327057,0.106848\r\n1268.13,0.8883242,0.587712,0.327058,0.106748\r\n1268.41,0.8882914,0.587827,0.327058,0.106738\r\n1268.87,0.8882368,0.588012,0.327058,0.106715\r\n1271.7,0.8881805,0.589161,0.327058,0.106571\r\n1271.89,0.88809085,0.589235,0.327058,0.106561\r\n1275.65,0.8879848,0.590757,0.327059,0.10637\r\n1276.32,0.88783635,0.590963,0.327059,0.106336\r\n1279.4,0.88768125,0.591899,0.327059,0.10618\r\n1280.05,0.88749425,0.592162,0.327059,0.106147\r\n1283.16,0.88729255,0.593422,0.32706,0.105989\r\n1283.35,0.88708645,0.593498,0.32706,0.105979\r\n1283.77,0.88681845,0.593671,0.32706,0.105965\r\n1286.65,0.88654495,0.594836,0.32706,0.105867\r\n1286.92,0.8862598,0.594945,0.32706,0.105858\r\n1287.49,0.8859176,0.595251,0.327061,0.105839\r\n1291.21,0.88554125,0.597256,0.327061,0.105713\r\n1294.55,0.88512745,0.599058,0.327062,0.1056\r\n1298.22,0.88470915,0.601035,0.327062,0.105414\r\n1298.66,0.8842002,0.601212,0.327062,0.105392\r\n1301.6,0.88374045,0.602403,0.327063,0.105242\r\n1301.98,0.88327475,0.602557,0.327053,0.105223\r\n1302.36,0.88285645,0.602712,0.327043,0.105204\r\n1302.37,0.8823909,0.602716,0.327043,0.105203\r\n1305.74,0.8819196,0.60408,0.326958,0.105032\r\n1306.09,0.88140735,0.604221,0.326949,0.105014\r\n1309.49,0.8809001,0.605598,0.326863,0.104842\r\n1309.5,0.88042375,0.605603,0.326863,0.104841\r\n1309.52,0.8799483,0.607125,0.326862,0.10484\r\n1309.8,0.8793793,0.607238,0.326855,0.104831\r\n1313.28,0.87881135,0.608647,0.326767,0.104713\r\n1313.31,0.8782408,0.610169,0.326766,0.104713\r\n1313.5,0.8777863,0.610248,0.326761,0.104706\r\n1313.52,0.87722825,0.610255,0.326761,0.104705\r\n1316.54,0.876663,0.61148,0.326684,0.104603\r\n1317.07,0.876154,0.611692,0.326684,0.104585\r\n1317.09,0.8756785,0.613214,0.326684,0.104585\r\n1317.11,0.8752366,0.614356,0.326684,0.104584\r\n1317.21,0.87472785,0.614399,0.326685,0.10458\r\n1320.69,0.87430625,0.615809,0.326685,0.104463\r\n1320.87,0.8738959,0.615878,0.326685,0.104458\r\n1320.89,0.8734965,0.6174,0.326685,0.104458\r\n1320.93,0.8730971,0.617417,0.326685,0.104457\r\n1320.94,0.8725764,0.617421,0.326685,0.104457\r\n1324.62,0.87213895,0.618912,0.326686,0.104363\r\n1324.64,0.87177095,0.618921,0.326686,0.104363\r\n1324.65,0.8714022,0.618923,0.326686,0.104363\r\n1324.67,0.87088295,0.620445,0.326686,0.104362\r\n1328.33,0.8703664,0.621929,0.326686,0.104269\r\n1328.43,0.8699745,0.621968,0.326686,0.104267\r\n1328.45,0.86948985,0.62349,0.326686,0.104266\r\n1331.49,0.8689953,0.62472,0.326687,0.10419\r\n1332.04,0.8684817,0.624941,0.326687,0.104176\r\n1332.05,0.86807375,0.624946,0.326687,0.104175\r\n1332.21,0.8676145,0.625013,0.326687,0.104171\r\n1335.64,0.8671535,0.625708,0.326687,0.104085\r\n1335.76,0.86661705,0.625734,0.326687,0.104076\r\n1339.48,0.8660797,0.626488,0.326688,0.103824\r\n1339.71,0.86554235,0.626536,0.326688,0.103808\r\n1343.19,0.8650028,0.627945,0.326688,0.103572\r\n1343.47,0.86440295,0.628058,0.326689,0.103554\r\n1346.44,0.8637957,0.629261,0.326689,0.103352\r\n1346.84,0.863149,0.629422,0.326689,0.103325\r\n1346.9,0.86268425,0.62945,0.326689,0.103324\r\n1347.23,0.86205655,0.629581,0.326689,0.103315\r\n1350.61,0.86146715,0.630611,0.32669,0.10323\r\n1350.62,0.8608714,0.630614,0.32669,0.103229\r\n1350.98,0.8603579,0.630723,0.32669,0.10322\r\n1354.34,0.85983915,0.632082,0.32669,0.103135\r\n1354.74,0.85925325,0.632246,0.32669,0.103125\r\n1358.05,0.85875175,0.633586,0.326691,0.103041\r\n1358.5,0.85826665,0.633768,0.326691,0.10303\r\n1358.52,0.8576849,0.63529,0.326691,0.103029\r\n1361.39,0.8570987,0.636449,0.326691,0.102957\r\n1361.77,0.8565824,0.636604,0.326682,0.102947\r\n1361.78,0.8560583,0.636608,0.326681,0.102947\r\n1362.28,0.8555448,0.636813,0.326668,0.10293\r\n1365.48,0.85498125,0.637301,0.326588,0.102822\r\n1369.78,0.8544887,0.637956,0.326479,0.102677\r\n1369.8,0.85400155,0.639478,0.326478,0.102676\r\n1369.82,0.85353555,0.641,0.326478,0.102675\r\n1372.93,0.853096,0.642258,0.326399,0.10257\r\n1372.98,0.8526487,0.642281,0.326398,0.102568\r\n1373.58,0.85220935,0.642523,0.326383,0.102528\r\n1373.6,0.85185645,0.644045,0.326382,0.102526\r\n1373.63,0.85146515,0.645567,0.326381,0.102525\r\n1373.65,0.85104005,0.647089,0.326381,0.102523\r\n1376.33,0.85067875,0.648174,0.326313,0.102341\r\n1376.65,0.8503102,0.648305,0.32628,0.102319\r\n1377.41,0.8499489,0.648611,0.326203,0.102268\r\n1380.37,0.8495886,0.649809,0.325902,0.102068\r\n1381.17,0.849198,0.650134,0.32582,0.102013\r\n1384.18,0.8488319,0.650747,0.325513,0.101809\r\n1387.52,0.8484706,0.651425,0.325173,0.101724\r\n1387.82,0.8481067,0.651485,0.325153,0.101717\r\n1388.66,0.8477404,0.651657,0.325096,0.101695\r\n1392.42,0.84741825,0.652799,0.324841,0.1016\r\n1398.72,0.8471499,0.653761,0.324414,0.101441\r\n1399.02,0.84688185,0.653806,0.324394,0.101433\r\n1399.12,0.8466141,0.653822,0.324386,0.101431\r\n1399.91,0.84638605,0.653942,0.324333,0.101377\r\n1407.41,0.84613975,0.655465,0.323825,0.100869\r\n1409.92,0.8459026,0.655849,0.323654,0.100699\r\n1410.32,0.84566585,0.65591,0.323613,0.100671\r\n1413.96,0.84543185,0.656466,0.323243,0.100548\r\n1414.9,0.84521405,0.656608,0.323148,0.100517\r\n1421.11,0.8450382,0.657241,0.322515,0.100307\r\n1421.44,0.8448673,0.657275,0.322481,0.100295\r\n1421.53,0.8447119,0.657283,0.322472,0.100292\r\n1426.12,0.84461715,0.657751,0.322004,0.100059\r\n1428.92,0.8445306,0.658177,0.32172,0.0999171\r\n1432.31,0.84447485,0.658694,0.321375,0.0997449\r\n1433.62,0.8444532,0.658894,0.321286,0.0996785\r\n1436.47,0.8444346,0.658967,0.321093,0.0995337\r\n1443.51,0.84441885,0.659147,0.320615,0.0992957\r\n1443.87,0.8444159,0.659156,0.320579,0.0992835\r\n1447.67,0.8444343,0.659254,0.320191,0.0991549\r\n1448.57,0.84445375,0.659277,0.3201,0.0991092\r\n1454.7,0.8444732,0.659278,0.319476,0.0987978\r\n1458.81,0.8444927,0.659278,0.319197,0.0985889\r\n1462.61,0.84451215,0.659279,0.31894,0.0983961\r\n1463.52,0.8445316,0.659279,0.318878,0.0983654\r\n1465.9,0.84455115,0.659036,0.318716,0.0982848\r\n1473.76,0.84457065,0.658236,0.318317,0.0980191\r\n1473.81,0.84458775,0.658231,0.318314,0.0980173\r\n1474.71,0.84458905,0.658139,0.318269,0.0979717\r\n1480.84,0.84458955,0.657724,0.317958,0.0976605\r\n1485.91,0.84459005,0.65738,0.317829,0.0974027\r\n1488.71,0.84459055,0.65719,0.317758,0.0972606\r\n1488.75,0.84459105,0.657187,0.317757,0.0972586\r\n1495.78,0.84459155,0.656711,0.317579,0.0970209\r\n1496.18,0.844592,0.656683,0.317569,0.0970072\r\n1497.11,0.8445925,0.656621,0.317546,0.0969759\r\n1499.95,0.84459295,0.656404,0.317474,0.0968797\r\n1503.66,0.84458145,0.656121,0.31738,0.0967546\r\n1510.72,0.8445692,0.655582,0.317201,0.0965157\r\n1511.16,0.84455545,0.655549,0.317171,0.0965009\r\n1512.04,0.84453685,0.655481,0.317112,0.0964562\r\n1514.86,0.84450935,0.65529,0.31692,0.0963128\r\n1521.92,0.8444801,0.654811,0.316442,0.0959543\r\n1523.24,0.84444725,0.654722,0.316352,0.0958872\r\n1526.09,0.8444097,0.654431,0.316159,0.0957422\r\n1529.8,0.84436975,0.654054,0.315907,0.0956168\r\n1530.7,0.8443237,0.653962,0.315846,0.0955864\r\n1533.12,0.84427435,0.652972,0.315682,0.0955047\r\n1534.42,0.8442176,0.652441,0.31555,0.0954609\r\n1537.3,0.8441492,0.651852,0.315257,0.0953634\r\n1541.87,0.8440746,0.65092,0.314792,0.0950537\r\n1544.31,0.8439989,0.650546,0.314543,0.0948879\r\n1544.74,0.8439104,0.65048,0.314499,0.0948589\r\n1548.5,0.84381915,0.649905,0.314116,0.094604\r\n1549.32,0.8437069,0.649779,0.314032,0.0945832\r\n1555.51,0.84359315,0.648517,0.313403,0.0944266\r\n1555.94,0.8434794,0.648429,0.313337,0.0944157\r\n1556.78,0.84337765,0.648258,0.313209,0.0943945\r\n1562.96,0.8432766,0.647312,0.312262,0.0942379\r\n1563.41,0.843177,0.647243,0.312232,0.0942265\r\n1563.44,0.8430802,0.647239,0.31223,0.0942258\r\n1564.23,0.84298285,0.647118,0.312176,0.0941991\r\n1567.95,0.84287975,0.645596,0.311924,0.0940735\r\n1570.88,0.84277825,0.644394,0.311725,0.0939743\r\n1571.66,0.8426751,0.644075,0.311673,0.0939479\r\n1574.16,0.84257305,0.643692,0.311503,0.0938633\r\n1574.65,0.8424657,0.643618,0.311404,0.093847\r\n1579.12,0.84235535,0.642934,0.310492,0.0936199\r\n1581.61,0.84224925,0.641912,0.309982,0.093493\r\n1582.83,0.84214715,0.641413,0.309796,0.0934312\r\n1585.82,0.84204575,0.640803,0.309339,0.0932795\r\n1589.07,0.8419446,0.640139,0.308842,0.0931142\r\n1589.58,0.841855,0.640035,0.308807,0.0930882\r\n1590.28,0.8417619,0.639892,0.308759,0.0930646\r\n1594,0.84168465,0.63837,0.308508,0.092939\r\n1597.02,0.841608,0.637442,0.308302,0.0928367\r\n1597.72,0.8415281,0.637229,0.308255,0.0928132\r\n1600.27,0.8414475,0.636183,0.308082,0.0927269\r\n1600.79,0.8413661,0.635971,0.308003,0.0927094\r\n1601.43,0.84127675,0.635708,0.307905,0.0926768\r\n1607.73,0.84118795,0.634423,0.306942,0.0923569\r\n1608.88,0.8411047,0.634187,0.306785,0.0922982\r\n1611.96,0.8410281,0.632924,0.306366,0.0921416\r\n1612.6,0.84094955,0.632666,0.30628,0.0921096\r\n1615.72,0.8408766,0.631384,0.305855,0.0919506\r\n1616.31,0.84080155,0.631144,0.305775,0.0919109\r\n1618.92,0.8407365,0.630345,0.305421,0.0917343\r\n1620.03,0.8406697,0.630003,0.305308,0.0916587\r\n1626.38,0.84060535,0.627403,0.304662,0.0912283\r\n1626.91,0.8405486,0.627186,0.304608,0.0911925\r\n1626.92,0.84049735,0.627179,0.304606,0.0911913\r\n1627.46,0.84044695,0.62696,0.304552,0.0911732\r\n1627.48,0.84039775,0.628482,0.304549,0.0911725\r\n1631.18,0.84035485,0.625819,0.304173,0.0910475\r\n1634.38,0.84032525,0.625165,0.303847,0.0909392\r\n1637.57,0.8402979,0.624514,0.303522,0.0908313\r\n1638.13,0.8402751,0.6244,0.303465,0.0908125\r\n1638.63,0.8402612,0.624298,0.303414,0.0907871\r\n1641.86,0.8402497,0.622975,0.303085,0.090623\r\n1642.34,0.84025885,0.622777,0.303036,0.0905984\r\n1646.06,0.84027865,0.621255,0.302658,0.0904097\r\n1648.77,0.8403058,0.620425,0.302382,0.0902722\r\n1649.78,0.8403449,0.620114,0.302279,0.0902207\r\n1653.07,0.840388,0.618767,0.301944,0.0900537\r\n1653.08,0.840435,0.61876,0.301942,0.0900532\r\n1653.49,0.8405136,0.618593,0.301901,0.0900393\r\n1657.21,0.8406146,0.617071,0.301522,0.0899138\r\n1659.96,0.84072835,0.615943,0.301242,0.0898207\r\n1660.57,0.84084625,0.615692,0.301148,0.0897999\r\n1660.92,0.8409982,0.61555,0.301095,0.0897882\r\n1664.27,0.8411821,0.615037,0.300583,0.0896749\r\n1667.42,0.84137265,0.614556,0.300102,0.0895953\r\n1668.07,0.84156555,0.614456,0.300057,0.0895787\r\n1668.38,0.84178655,0.614409,0.300036,0.089571\r\n1672.07,0.8419996,0.611366,0.299786,0.0894775\r\n1672.09,0.8422377,0.612888,0.299785,0.089477\r\n1675.56,0.84247675,0.610027,0.299549,0.0893891\r\n1675.78,0.84273635,0.609844,0.299534,0.0893835\r\n1678.61,0.8430004,0.609411,0.299342,0.0893118\r\n1679.21,0.84325275,0.60932,0.299302,0.0892967\r\n1679.31,0.843515,0.609305,0.299295,0.0892901\r\n1683.05,0.8437757,0.608732,0.299041,0.0890364\r\n1683.24,0.84406295,0.608704,0.299029,0.0890236\r\n1686.95,0.84437925,0.607182,0.298777,0.0887718\r\n1689.81,0.84469845,0.60601,0.298583,0.0885778\r\n1690.41,0.84499075,0.605766,0.298492,0.0885374\r\n1690.55,0.84526325,0.60571,0.298471,0.0885281\r\n1690.67,0.84559815,0.605661,0.298452,0.08852\r\n1697.27,0.8459374,0.604651,0.297443,0.0880723\r\n1698.04,0.8462438,0.604532,0.297364,0.0880198\r\n1698.12,0.8465713,0.60452,0.297356,0.0880144\r\n1701.61,0.846895,0.603092,0.297001,0.087778\r\n1701.8,0.84721675,0.603015,0.296981,0.0877733\r\n1701.84,0.8475481,0.602999,0.296977,0.0877723\r\n1705.55,0.84790885,0.601477,0.296599,0.0876782\r\n1708.46,0.8482513,0.600284,0.296303,0.0876045\r\n1709.27,0.84859175,0.599956,0.296221,0.0875842\r\n1709.3,0.8489415,0.59995,0.296218,0.0875835\r\n1713.04,0.84932325,0.599185,0.295836,0.0874886\r\n1713.06,0.84973075,0.599183,0.295835,0.0874883\r\n1716.55,0.8501204,0.598468,0.295479,0.0873998\r\n1716.72,0.85053055,0.598435,0.295462,0.0873886\r\n1719.66,0.85090895,0.597533,0.295163,0.0871892\r\n1720.44,0.8512591,0.597294,0.295083,0.0871364\r\n1720.55,0.8516764,0.597272,0.295072,0.0871291\r\n1727.75,0.8520937,0.595801,0.294339,0.0866404\r\n1727.89,0.85251135,0.595773,0.294325,0.0866358\r\n1728.04,0.85288335,0.595726,0.294309,0.0866306\r\n1730.85,0.85325325,0.594864,0.294023,0.0865357\r\n1731.61,0.85365025,0.594632,0.293972,0.0865101\r\n1735.32,0.85399585,0.59311,0.29372,0.0863845\r\n1735.53,0.854356,0.593078,0.293706,0.0863774\r\n1738.96,0.8547315,0.592555,0.293473,0.0862616\r\n1742.05,0.85507295,0.592081,0.293264,0.0861833\r\n1742.78,0.8554051,0.59197,0.293189,0.0861649\r\n1746.76,0.85574675,0.591157,0.292784,0.0860641\r\n1750.23,0.85608955,0.590449,0.292431,0.0859763\r\n1750.5,0.85640125,0.590407,0.292403,0.0859694\r\n1753.25,0.85667655,0.589988,0.292124,0.0859\r\n1753.9,0.8569348,0.589888,0.292079,0.0858834\r\n1757.69,0.8571926,0.589309,0.291823,0.0856266\r\n1757.98,0.85740605,0.589248,0.291803,0.0856065\r\n1764.44,0.85761135,0.587929,0.291364,0.0851684\r\n1765.1,0.85781565,0.587796,0.29132,0.0851241\r\n1765.14,0.85793305,0.587788,0.291317,0.0851231\r\n1769.18,0.8580404,0.587169,0.291043,0.0850207\r\n1772.6,0.85813845,0.586647,0.290812,0.0849343\r\n1775.64,0.85821335,0.585398,0.290605,0.0848571\r\n1776.31,0.85827305,0.585126,0.290588,0.0848403\r\n1780.04,0.85829495,0.584555,0.290494,0.0847458\r\n1783.77,0.8583143,0.583985,0.2904,0.0844932\r\n1784.12,0.85828335,0.583931,0.290391,0.0844693\r\n1790.59,0.8582334,0.582942,0.290227,0.0840308\r\n1791.22,0.85817965,0.582845,0.290227,0.0839876\r\n1791.24,0.8580849,0.582841,0.290227,0.0839865\r\n1795.31,0.8579468,0.58201,0.290228,0.0838488\r\n1798.67,0.8577834,0.581324,0.290228,0.0837352\r\n1801.8,0.85760805,0.580687,0.290229,0.0836296\r\n1802.45,0.85742855,0.580554,0.290245,0.0836077\r\n1802.77,0.85720515,0.580488,0.290253,0.0835995\r\n1806.13,0.8569772,0.579803,0.290339,0.0835145\r\n1806.5,0.856737,0.57969,0.290349,0.0835052\r\n1809.85,0.85649515,0.578662,0.290435,0.0834204\r\n1810.22,0.85620355,0.578585,0.290444,0.0834109\r\n1816.75,0.85592145,0.577253,0.290611,0.0832456\r\n1817.3,0.8556473,0.577141,0.290625,0.0832317\r\n1817.39,0.8553787,0.577113,0.290628,0.0832294\r\n1817.68,0.8550777,0.577024,0.290635,0.0832098\r\n1821.02,0.8547442,0.576,0.290721,0.0829834\r\n1825.13,0.85439465,0.575161,0.290826,0.0827046\r\n1828.47,0.8540451,0.574479,0.290911,0.0824782\r\n1828.59,0.8537046,0.574461,0.290914,0.0824701\r\n1831.7,0.85336065,0.573984,0.290994,0.0823118\r\n1832.59,0.85297415,0.573849,0.291017,0.082267\r\n1835.92,0.8525559,0.573339,0.291102,0.0820974\r\n1840.03,0.8521824,0.572501,0.291207,0.0818888\r\n1840.04,0.85178155,0.572499,0.291207,0.0818885\r\n1843.38,0.8513834,0.571818,0.291293,0.0817189\r\n1843.52,0.8509842,0.571788,0.291297,0.0817113\r\n1843.76,0.8505896,0.57174,0.291303,0.0817035\r\n1846.66,0.8501562,0.571148,0.291377,0.0816054\r\n1850.83,0.84975315,0.570297,0.291484,0.0814644\r\n1851.21,0.849327,0.570239,0.291493,0.0814516\r\n1854.73,0.8489236,0.5697,0.291584,0.0813325\r\n1858.28,0.84852145,0.569157,0.291675,0.0812123\r\n1858.66,0.84808995,0.569098,0.291684,0.0811995\r\n1861.61,0.8476185,0.568648,0.29176,0.0810999\r\n1865.74,0.84717915,0.568016,0.29204,0.0809603\r\n1865.93,0.8467799,0.567996,0.292054,0.0809537\r\n1866.12,0.84635485,0.567978,0.292066,0.0809445\r\n1872.83,0.84592505,0.567294,0.292523,0.0806032\r\n1873.57,0.8454848,0.567219,0.292541,0.0805657\r\n1876.93,0.8450475,0.566876,0.292627,0.0803949\r\n1877.3,0.8446347,0.566821,0.292637,0.0803764\r\n1880.87,0.8442735,0.566274,0.292728,0.080195\r\n1881.03,0.84385895,0.56625,0.292732,0.0801896\r\n1884.39,0.843461,0.565736,0.292818,0.0800759\r\n1887.79,0.84305825,0.565736,0.292905,0.0799611\r\n1888.48,0.84264685,0.565736,0.292952,0.0799377\r\n1892.07,0.8422416,0.565737,0.293197,0.0798161\r\n1899.01,0.84187775,0.565738,0.293668,0.079346\r\n1899.34,0.8415166,0.565738,0.293702,0.0793237\r\n1899.68,0.8411641,0.565721,0.293736,0.0793005\r\n1903.27,0.8408033,0.565538,0.294102,0.0790568\r\n1910.24,0.8404422,0.565184,0.294811,0.0788806\r\n1910.87,0.84009965,0.565152,0.294844,0.0788645\r\n1914.27,0.83978585,0.564979,0.295017,0.0787784\r\n1918.22,0.839461,0.56498,0.295218,0.0786786\r\n1922.07,0.83910615,0.564981,0.295415,0.0785482\r\n1925.2,0.8387965,0.564981,0.295575,0.0784426\r\n1925.8,0.8384838,0.564981,0.295636,0.0784222\r\n1929.22,0.8382049,0.564982,0.295985,0.0783065\r\n1929.42,0.83791995,0.564982,0.296005,0.0782998\r\n1933.27,0.83762095,0.564982,0.296396,0.0781044\r\n1936.42,0.83727895,0.564983,0.296718,0.0779441\r\n1940.72,0.83690055,0.564983,0.29701,0.0777259\r\n1944.17,0.83655295,0.564984,0.297244,0.0775505\r\n1944.36,0.8362109,0.564984,0.297257,0.077541\r\n1944.43,0.83583175,0.564984,0.297262,0.0775385\r\n1947.65,0.83546835,0.564984,0.297481,0.0774299\r\n1948.15,0.83503,0.564984,0.297558,0.0774127\r\n1951.87,0.8345258,0.564985,0.298124,0.0772872\r\n1955.14,0.8340312,0.564986,0.298623,0.0771766\r\n1955.56,0.8335717,0.564986,0.298652,0.0771622\r\n1955.58,0.8331535,0.564986,0.298653,0.0771613\r\n1955.59,0.8325645,0.564986,0.298654,0.0771607\r\n1959.12,0.83198095,0.564986,0.298894,0.0769816\r\n1959.27,0.83148655,0.564982,0.298904,0.0769738\r\n1959.29,0.83087185,0.564982,0.298906,0.0769726\r\n1962.99,0.8301593,0.564888,0.299157,0.0767851\r\n1966.36,0.8294453,0.564803,0.299386,0.0766138\r\n1966.69,0.82876985,0.564795,0.299403,0.076597\r\n1966.7,0.82802815,0.564794,0.299403,0.0765964\r\n1970.41,0.8273173,0.5647,0.299593,0.0764078\r\n1970.5,0.8265957,0.564698,0.299597,0.0764034\r\n1974.06,0.8259437,0.564608,0.299779,0.0762831\r\n1974.13,0.8253685,0.564603,0.299783,0.0762806\r\n1981.32,0.824798,0.564116,0.300149,0.0760377\r\n1981.63,0.8243529,0.564095,0.300181,0.0760271\r\n1981.65,0.8239166,0.564093,0.300184,0.0760263\r\n1981.7,0.82362835,0.56409,0.300189,0.0760246\r\n1985.26,0.82341025,0.563849,0.300551,0.075844\r\n1985.41,0.8232389,0.563833,0.300566,0.0758362\r\n1985.42,0.82310595,0.563832,0.300568,0.0758356\r\n1985.44,0.82301105,0.563831,0.300569,0.0758351\r\n1985.46,0.82317455,0.563828,0.300571,0.0758339\r\n1985.48,0.8234221,0.563826,0.300573,0.0758328\r\n1985.5,0.8236697,0.563824,0.300576,0.0758316\r\n1989.26,0.8241075,0.563441,0.300958,0.0756406\r\n1989.27,0.82468735,0.56344,0.30096,0.07564\r\n1989.29,0.82535345,0.563439,0.300961,0.0756395\r\n1989.31,0.8260576,0.563436,0.300963,0.0756383\r\n1989.33,0.82690285,0.563434,0.300965,0.0756371\r\n1992.55,0.8277605,0.563107,0.301293,0.0754739\r\n1993.09,0.82874885,0.563051,0.30133,0.0754462\r\n1993.11,0.82977635,0.563049,0.301331,0.075445\r\n1993.13,0.830769,0.563048,0.301332,0.0754444\r\n1993.14,0.83183455,0.563046,0.301333,0.0754438\r\n1993.16,0.83289065,0.563044,0.301334,0.0754427\r\n1996.45,0.8339495,0.562709,0.301558,0.0752754\r\n1996.64,0.83494205,0.56268,0.301571,0.0752659\r\n1996.92,0.83600755,0.562637,0.30159,0.0752564\r\n1996.94,0.8371111,0.562634,0.301591,0.0752557\r\n1996.96,0.83821465,0.56263,0.301593,0.0752549\r\n1996.98,0.8392802,0.562629,0.301594,0.0752545\r\n2000.72,0.84034575,0.562055,0.301849,0.0751278\r\n2000.75,0.8414113,0.562052,0.30185,0.075127\r\n2000.77,0.8424768,0.562049,0.301852,0.0751262\r\n2000.79,0.84358035,0.562045,0.301853,0.0751255\r\n2000.82,0.84467585,0.562042,0.301855,0.0751247\r\n2003.77,0.8456967,0.56159,0.302055,0.0750249\r\n2003.91,0.8466481,0.561568,0.30207,0.0750201\r\n2004.56,0.8475725,0.561502,0.302136,0.074998\r\n2004.58,0.84856195,0.561501,0.302138,0.0749976\r\n2004.6,0.8495514,0.561498,0.30214,0.0749968\r\n2004.62,0.8505789,0.561496,0.302142,0.0749961\r\n2004.64,0.85163315,0.561494,0.302145,0.0749953\r\n2007.85,0.8526226,0.561168,0.302471,0.074887\r\n2008.4,0.85358345,0.561111,0.302527,0.0748682\r\n2008.41,0.85457855,0.56111,0.302529,0.0748678\r\n2008.43,0.8556441,0.561108,0.30253,0.0748674\r\n2008.45,0.85670965,0.561106,0.302532,0.0748667\r\n2008.47,0.85777525,0.561104,0.302534,0.0748659\r\n2012.23,0.8588789,0.560721,0.302917,0.0747388\r\n2012.25,0.85996345,0.560719,0.302919,0.074738\r\n2012.26,0.86100995,0.560718,0.30292,0.0747378\r\n2012.27,0.86205645,0.560717,0.302921,0.0747374\r\n2012.29,0.8630816,0.560715,0.302924,0.0747367\r\n2015,0.8640328,0.560439,0.303199,0.0746453\r\n2015.1,0.86497325,0.560428,0.303206,0.0746417\r\n2016.05,0.86598835,0.560283,0.303271,0.0746096\r\n2016.08,0.8670729,0.56028,0.303272,0.0746088\r\n2016.1,0.86815745,0.560276,0.303274,0.074608\r\n2016.11,0.86920395,0.560275,0.303274,0.0746076\r\n2016.12,0.8702337,0.560273,0.303275,0.0746073\r\n2019.05,0.87120415,0.559825,0.303474,0.0745082\r\n2019.88,0.87218585,0.559698,0.303531,0.074466\r\n2019.9,0.87323235,0.559694,0.303532,0.0744649\r\n2019.93,0.8743169,0.559691,0.303534,0.0744637\r\n2019.95,0.87540145,0.559687,0.303535,0.0744626\r\n2019.96,0.87644795,0.559686,0.303536,0.074462\r\n2019.97,0.87747075,0.559684,0.303537,0.0744614\r\n2022.56,0.87844115,0.559288,0.303713,0.0743299\r\n2023.73,0.87941155,0.559109,0.303792,0.0742704\r\n2023.75,0.88043905,0.559105,0.303794,0.0742693\r\n2023.78,0.8815046,0.559102,0.303796,0.0742681\r\n2023.8,0.8825321,0.559098,0.303797,0.074267\r\n2023.81,0.8835429,0.559097,0.303798,0.0742664\r\n2023.82,0.88462765,0.559095,0.303799,0.0742658\r\n2023.85,0.8857412,0.559091,0.3038,0.0742646\r\n2026.22,0.88680675,0.558729,0.303961,0.0741442\r\n2027.61,0.8878723,0.558516,0.304032,0.0740737\r\n2027.63,0.8889759,0.558513,0.304033,0.0740725\r\n2027.65,0.8900795,0.55851,0.304035,0.0740713\r\n2027.66,0.8911618,0.558508,0.304035,0.0740708\r\n2027.67,0.8923034,0.558506,0.304036,0.0740702\r\n2027.7,0.8934159,0.558503,0.304037,0.074069\r\n2030.02,0.89448145,0.558148,0.304155,0.0739511\r\n2031.46,0.895547,0.557854,0.304229,0.073878\r\n2031.48,0.8966506,0.557849,0.30423,0.0738769\r\n2031.5,0.8977542,0.557845,0.304231,0.0738757\r\n2031.51,0.89884345,0.557842,0.304232,0.0738752\r\n2031.52,0.89998505,0.55784,0.304232,0.0738746\r\n2031.55,0.90112665,0.557835,0.304233,0.0738734\r\n2031.57,0.9022411,0.557831,0.304234,0.0738723\r\n2033.99,0.90330665,0.557337,0.304358,0.0737495\r\n2035.33,0.90441025,0.557063,0.304426,0.0736585\r\n2035.35,0.90551385,0.557059,0.304427,0.0736569\r\n2035.36,0.9065794,0.557056,0.304428,0.0736561\r\n2035.38,0.907673,0.557054,0.304429,0.0736554\r\n2035.4,0.9088146,0.557049,0.30443,0.0736538\r\n2035.42,0.90992155,0.557045,0.304431,0.0736523\r\n2037.47,0.9109871,0.556627,0.304535,0.0735135\r\n2039.18,0.9120907,0.556365,0.304623,0.0733974\r\n2039.2,0.9131943,0.556361,0.304624,0.0733958\r\n2039.21,0.91425985,0.55636,0.304625,0.0733951\r\n2039.23,0.9153179,0.556358,0.304625,0.0733943\r\n2041.18,0.91638345,0.55606,0.304725,0.073262\r\n2042.99,0.91733315,0.555783,0.304848,0.0731394\r\n2044.93,0.9182014,0.555486,0.304979,0.0730079\r\n2045.19,0.9190006,0.555379,0.304997,0.0729901\r\n2046.7,0.91970195,0.554759,0.3051,0.072939\r\n2048.64,0.92027285,0.553965,0.305232,0.0728734\r\n2050.4,0.92080575,0.553695,0.305352,0.0728138\r\n2050.41,0.92116615,0.553693,0.305353,0.0728134\r\n2052.4,0.9213946,0.55339,0.305487,0.0727463\r\n2054.08,0.92162305,0.553132,0.305602,0.0726894\r\n2054.11,0.92188955,0.553129,0.305603,0.0726886\r\n2054.13,0.92178095,0.553125,0.305605,0.0726879\r\n2056.1,0.9215456,0.552824,0.305739,0.0726213\r\n2056.39,0.92101305,0.552764,0.305759,0.0726113\r\n2057.68,0.92051515,0.5525,0.305847,0.0725786\r\n2057.71,0.92005865,0.552496,0.305848,0.0725781\r\n2057.73,0.91960215,0.552491,0.30585,0.0725775\r\n2057.75,0.91914565,0.552487,0.305851,0.0725769\r\n2057.76,0.91868915,0.552484,0.305852,0.0725766\r\n2057.77,0.91826925,0.552482,0.305853,0.0725763\r\n2057.8,0.9170517,0.552477,0.305854,0.0725758\r\n2061.31,0.91595,0.551761,0.306093,0.0724869\r\n2061.33,0.91492975,0.551756,0.306094,0.0724863\r\n2061.35,0.9140166,0.551752,0.306096,0.0724858\r\n2061.37,0.9132013,0.551747,0.306097,0.0724852\r\n2061.39,0.9124784,0.551745,0.306098,0.0724849\r\n2061.4,0.9113492,0.551742,0.306099,0.0724846\r\n2063.55,0.9103309,0.551303,0.306245,0.0724302\r\n2063.62,0.9091894,0.551281,0.30625,0.0724283\r\n2064.95,0.90804795,0.550873,0.306318,0.0723946\r\n2064.97,0.9069065,0.550866,0.306319,0.0723941\r\n2065,0.90614015,0.550859,0.30632,0.0723935\r\n2065.01,0.9054625,0.550855,0.306321,0.0723932\r\n2065.02,0.9049133,0.550852,0.306322,0.0723929\r\n2067.27,0.90412035,0.550162,0.306436,0.072336\r\n2071.33,0.90320725,0.549332,0.306644,0.0722331\r\n2072.4,0.9020862,0.549114,0.306698,0.0721606\r\n2074.72,0.90084965,0.548642,0.306816,0.0720036\r\n2076.08,0.89965115,0.548434,0.306886,0.0719115\r\n2076.1,0.89841465,0.54843,0.306887,0.0719099\r\n2076.11,0.8979772,0.548428,0.306888,0.0719091\r\n2076.12,0.89709775,0.548427,0.306888,0.0719084\r\n2078.58,0.8959754,0.548051,0.307013,0.0717418\r\n2079.7,0.89485305,0.547879,0.30709,0.0716658\r\n2079.72,0.8937307,0.547876,0.307091,0.0716643\r\n2079.73,0.89260835,0.547874,0.307092,0.0716635\r\n2079.75,0.8919303,0.547873,0.307093,0.0716627\r\n2079.77,0.891341,0.547869,0.307094,0.0716612\r\n2079.79,0.89051755,0.547866,0.307096,0.0716596\r\n2082.17,0.88954845,0.547501,0.307258,0.071498\r\n2082.53,0.88834995,0.547428,0.307282,0.0714737\r\n2083.32,0.8871895,0.547267,0.307336,0.0714537\r\n2083.35,0.88602905,0.547262,0.307337,0.0714532\r\n2083.36,0.8849993,0.54726,0.307338,0.0714529\r\n2083.37,0.8842133,0.547257,0.307339,0.0714526\r\n2083.39,0.88354745,0.547253,0.307341,0.071452\r\n2083.41,0.88232855,0.547248,0.307342,0.0714514\r\n2086.95,0.8812632,0.546527,0.307582,0.071362\r\n2086.97,0.8801598,0.546522,0.307584,0.0713614\r\n2086.98,0.87905635,0.54652,0.307584,0.0713612\r\n2086.99,0.87799095,0.546518,0.307585,0.0713609\r\n2087.02,0.87736755,0.546513,0.307587,0.0713603\r\n2087.04,0.87646775,0.546508,0.307588,0.0713597\r\n2089.63,0.87546165,0.54598,0.307764,0.0712942\r\n2089.8,0.8742821,0.545944,0.307776,0.0712897\r\n2090.59,0.87310255,0.545783,0.307817,0.0712697\r\n2090.6,0.87188495,0.54578,0.307817,0.0712694\r\n2090.62,0.87066735,0.545778,0.307818,0.0712691\r\n2090.64,0.86998795,0.545773,0.307819,0.0712686\r\n2090.66,0.86876925,0.545769,0.30782,0.071268\r\n2094.22,0.86774195,0.545043,0.308001,0.071178\r\n2094.23,0.8666766,0.545041,0.308002,0.0711777\r\n2094.24,0.8656493,0.545039,0.308003,0.0711774\r\n2094.26,0.86466005,0.545034,0.308004,0.0711769\r\n2094.28,0.8636708,0.545029,0.308005,0.0711763\r\n2094.31,0.8619404,0.545025,0.308006,0.0711757\r\n2097.08,0.8607993,0.544459,0.308147,0.0711056\r\n2097.48,0.85950565,0.544398,0.308168,0.0710955\r\n2097.79,0.85825005,0.54435,0.308184,0.0710847\r\n2097.82,0.8570325,0.544346,0.308185,0.071084\r\n2097.84,0.8557769,0.544343,0.308186,0.0710832\r\n2097.85,0.85448325,0.544341,0.308187,0.0710828\r\n2097.86,0.8537089,0.544339,0.308187,0.0710824\r\n2097.88,0.85304085,0.544336,0.308189,0.0710817\r\n2097.91,0.8525843,0.544332,0.30819,0.0710809\r\n2097.93,0.8512526,0.544329,0.308191,0.0710801\r\n2101.42,0.84995895,0.543795,0.308369,0.0709622\r\n2101.44,0.8486653,0.543792,0.30837,0.0709615\r\n2101.46,0.8473336,0.543789,0.308371,0.0709607\r\n2101.47,0.84664875,0.543787,0.308372,0.0709603\r\n2101.49,0.8460019,0.543785,0.308372,0.0709599\r\n2101.51,0.84473025,0.543782,0.308374,0.0709592\r\n2104.53,0.8433384,0.543319,0.308528,0.0708569\r\n2104.76,0.8418164,0.543284,0.30854,0.0708492\r\n2105.04,0.8402944,0.543241,0.30854,0.0708398\r\n2105.06,0.83951355,0.543238,0.30854,0.070839\r\n2105.09,0.83886635,0.543234,0.30854,0.0708382\r\n2105.1,0.83833365,0.543233,0.30854,0.0708378\r\n2105.11,0.83780095,0.543231,0.30854,0.0708375\r\n2105.13,0.83650725,0.543227,0.30854,0.0708367\r\n2108.66,0.8352326,0.542687,0.30854,0.0707173\r\n2108.68,0.8339389,0.542685,0.30854,0.0707167\r\n2108.69,0.8326452,0.542684,0.30854,0.0707164\r\n2108.71,0.83131345,0.54268,0.30854,0.0707152\r\n2108.72,0.82994365,0.542679,0.30854,0.0707147\r\n2108.73,0.82948705,0.542677,0.30854,0.0707141\r\n2108.75,0.82850835,0.542673,0.30854,0.0707129\r\n2111.99,0.82740495,0.542178,0.308541,0.0705485\r\n2112.34,0.82630155,0.542106,0.308541,0.0705306\r\n2112.35,0.8252362,0.542104,0.308541,0.07053\r\n2112.38,0.8239616,0.542099,0.308541,0.0705289\r\n2116.07,0.8232994,0.541345,0.308541,0.0703411\r\n2119.44,0.82268335,0.540657,0.308542,0.07017\r\n2119.71,0.82215075,0.540617,0.308542,0.0701564\r\n2123.57,0.8215426,0.540027,0.308542,0.0699605\r\n2123.62,0.8208917,0.540019,0.308542,0.0699579\r\n2126.9,0.820245,0.539517,0.308543,0.069847\r\n2131.06,0.8195854,0.53888,0.308544,0.0697062\r\n2134.36,0.818852,0.538377,0.308544,0.0695949\r\n2134.66,0.81880475,0.538315,0.308544,0.0695847\r\n2134.82,0.81872885,0.538281,0.308544,0.0695791\r\n2138.55,0.818667,0.53752,0.308545,0.069453\r\n2141.81,0.81857195,0.536856,0.308545,0.069343\r\n2146.03,0.81843905,0.53621,0.308546,0.0692003\r\n2146.05,0.8182763,0.536207,0.308546,0.0691992\r\n2149.26,0.81803835,0.535715,0.308546,0.0690359\r\n2149.6,0.8182564,0.535681,0.308546,0.0690186\r\n2153.53,0.81856115,0.535281,0.30828,0.0688192\r\n2157.28,0.818844,0.534899,0.308026,0.0686288\r\n2160.46,0.81905255,0.534575,0.307811,0.0684673\r\n2160.8,0.81939505,0.534505,0.307787,0.0684496\r\n2160.96,0.81974725,0.534472,0.307783,0.0684415\r\n2167.91,0.82010045,0.533054,0.307607,0.0682068\r\n2168.51,0.8204429,0.533014,0.307592,0.0681866\r\n2172.17,0.820803,0.532766,0.307499,0.0680627\r\n2175.75,0.821131,0.532523,0.307409,0.0679722\r\n2175.99,0.8214247,0.532506,0.307396,0.067966\r\n2179.11,0.82170275,0.532295,0.307238,0.0678871\r\n2182.82,0.8219894,0.530774,0.307049,0.0677931\r\n2187.11,0.8222829,0.530118,0.306831,0.0676845\r\n2190.28,0.822549,0.529633,0.306671,0.0674697\r\n2190.68,0.8227827,0.529571,0.30665,0.0674423\r\n2190.95,0.8229966,0.529531,0.306641,0.0674242\r\n2197.74,0.82321225,0.528493,0.306412,0.0669641\r\n2198.31,0.82343185,0.528376,0.306392,0.0669252\r\n2201.89,0.8236515,0.527645,0.306271,0.0668346\r\n2202.17,0.82386885,0.527587,0.306271,0.0668274\r\n2205.19,0.8240595,0.526972,0.306272,0.0667511\r\n2209.65,0.8242644,0.526289,0.306272,0.0666381\r\n2212.64,0.82447365,0.525831,0.306273,0.0665623\r\n2213.25,0.8246836,0.525645,0.306273,0.0665469\r\n2213.4,0.8248699,0.525599,0.306273,0.0665419\r\n2216.84,0.82507905,0.524546,0.306273,0.0664258\r\n2224.46,0.8252701,0.522208,0.306662,0.0661681\r\n2224.62,0.82545775,0.522157,0.306671,0.0661597\r\n2227.52,0.82566585,0.521268,0.306819,0.0660124\r\n2231.79,0.82588225,0.520614,0.307037,0.0657954\r\n2234.98,0.82609355,0.520127,0.307253,0.0656336\r\n2235.85,0.826306,0.520038,0.307312,0.0655894\r\n2239.39,0.8265227,0.519678,0.307553,0.0654093\r\n2243.02,0.82674905,0.519309,0.3078,0.0652869\r\n2246.17,0.8269778,0.518987,0.30788,0.0651801\r\n2247.07,0.8272014,0.51885,0.307903,0.0651498\r\n2250.6,0.8274385,0.51831,0.307994,0.0650305\r\n2253.63,0.82768555,0.517847,0.308071,0.0648765\r\n2257.97,0.8279333,0.516961,0.308182,0.0646561\r\n2258.3,0.82820165,0.516894,0.308191,0.0646393\r\n2261.08,0.8284684,0.516326,0.308262,0.064498\r\n2262.05,0.82871365,0.516178,0.308287,0.0644489\r\n2265.53,0.8289604,0.515645,0.308376,0.0642718\r\n2268.54,0.8292082,0.515185,0.308453,0.0641702\r\n2269.53,0.82946505,0.51478,0.308478,0.0641368\r\n2272.25,0.82970115,0.513664,0.308548,0.0640447\r\n2272.92,0.8299658,0.51363,0.308565,0.064022\r\n2276.74,0.83021775,0.513436,0.308695,0.063893\r\n2277.02,0.83048425,0.513422,0.308705,0.0638835\r\n2284.14,0.83075005,0.51306,0.308947,0.0636428\r\n2287.19,0.8310153,0.512905,0.308948,0.0635397\r\n2287.94,0.83127005,0.512854,0.308948,0.0635141\r\n2288.25,0.83156955,0.512833,0.308948,0.0635065\r\n2295.74,0.83187185,0.512326,0.308949,0.0633169\r\n2298.39,0.83217465,0.512146,0.30895,0.0632498\r\n2299.09,0.83247885,0.512128,0.30895,0.0632321\r\n2302.89,0.8327884,0.512032,0.309047,0.0631359\r\n2306.97,0.8331139,0.511929,0.309151,0.0628594\r\n2310.71,0.8334511,0.511834,0.309247,0.0626053\r\n2313.33,0.8337636,0.511768,0.309314,0.0624279\r\n2314.04,0.83406655,0.51175,0.309332,0.0623797\r\n2314.09,0.83436245,0.511748,0.309335,0.0623766\r\n2314.46,0.83472685,0.511739,0.309354,0.0623672\r\n2321.95,0.8351369,0.511549,0.309736,0.0621774\r\n2328.27,0.83553175,0.511389,0.310059,0.0620175\r\n2329,0.8359217,0.511389,0.310096,0.061999\r\n2329.03,0.8362857,0.51139,0.310096,0.0619983\r\n2329.44,0.83672375,0.51139,0.310096,0.0619843\r\n2336.94,0.83714495,0.511391,0.310097,0.0617309\r\n2339.48,0.83755635,0.511391,0.310097,0.0616449\r\n2340.23,0.8379922,0.51141,0.310097,0.0616195\r\n2343.95,0.8384302,0.511505,0.310098,0.0614308\r\n2344.43,0.8388492,0.511518,0.310114,0.0614063\r\n2348.18,0.83929275,0.511614,0.310242,0.0612159\r\n2351.93,0.8397519,0.51171,0.31037,0.0610255\r\n2354.44,0.84019725,0.511774,0.310455,0.060898\r\n2355.16,0.84062185,0.511811,0.31048,0.060861\r\n2355.17,0.8410408,0.511811,0.31048,0.0608608\r\n2359.42,0.84152255,0.512028,0.310589,0.0607171\r\n2363.18,0.84203315,0.51222,0.310685,0.06059\r\n2366.37,0.8425742,0.512383,0.310767,0.0604819\r\n2369.4,0.84312075,0.512537,0.310845,0.0604055\r\n2370.12,0.8435968,0.512611,0.310863,0.0603872\r\n2370.67,0.8440818,0.512668,0.310877,0.0603731\r\n2378.17,0.8445993,0.51343,0.311069,0.0601835\r\n2378.18,0.845129,0.513431,0.311069,0.0601832\r\n2380.62,0.84565905,0.51368,0.311132,0.0601213\r\n2381.32,0.84611605,0.513751,0.31115,0.0601037\r\n2381.92,0.84658565,0.513813,0.311165,0.0600833\r\n2385.07,0.84708035,0.514133,0.311246,0.0599768\r\n2388.11,0.84756005,0.514443,0.311401,0.0598741\r\n2389.42,0.8480676,0.514576,0.311468,0.0598299\r\n2392.52,0.84858165,0.514892,0.311626,0.0597249\r\n2396.91,0.8490945,0.515339,0.31185,0.0595018\r\n2399.34,0.84959875,0.515586,0.311974,0.0593787\r\n2400.03,0.85010715,0.515657,0.312009,0.0593435\r\n2400.67,0.8506845,0.515721,0.312031,0.0593111\r\n2407.46,0.8512366,0.516413,0.312262,0.0589661\r\n2408.16,0.85173695,0.516485,0.312286,0.0589662\r\n2410.56,0.8522118,0.516729,0.312368,0.0589666\r\n2411.25,0.8526627,0.517006,0.312391,0.0589667\r\n2411.91,0.85313075,0.517275,0.312425,0.0589668\r\n2414.32,0.85360675,0.518252,0.312548,0.0589672\r\n2415.66,0.85406245,0.518793,0.312617,0.0589674\r\n2418.08,0.85448045,0.519774,0.31274,0.0589677\r\n2418.11,0.85489455,0.521297,0.312741,0.0589677\r\n2418.67,0.8553165,0.521411,0.31277,0.0589678\r\n2419.41,0.8557982,0.521562,0.312808,0.05893\r\n2425.6,0.85625405,0.52282,0.313124,0.0586156\r\n2426.21,0.8566862,0.523003,0.313155,0.058585\r\n2426.91,0.8571237,0.523218,0.313155,0.0585492\r\n2429.36,0.85755835,0.523962,0.313155,0.0584249\r\n2433.12,0.85794855,0.525484,0.313156,0.0582339\r\n2433.14,0.85830545,0.527007,0.313156,0.0582328\r\n2433.61,0.8586579,0.527101,0.313156,0.0582091\r\n2434.4,0.85906395,0.527263,0.313156,0.0581822\r\n2440.64,0.85940625,0.52853,0.313157,0.0579714\r\n2441.15,0.859749,0.528739,0.313157,0.0579539\r\n2441.9,0.8600927,0.529041,0.313138,0.0579287\r\n2444.4,0.8604337,0.530052,0.313075,0.0578443\r\n2444.81,0.8607764,0.530221,0.313064,0.0578303\r\n2445.65,0.86113265,0.530559,0.313043,0.0578091\r\n2445.66,0.86149435,0.530564,0.313043,0.0578088\r\n2448.16,0.8618479,0.531575,0.31298,0.0577456\r\n2451.91,0.8622105,0.533098,0.312885,0.0576504\r\n2453.15,0.8625945,0.533286,0.312853,0.0576192\r\n2456.1,0.86300075,0.533735,0.312779,0.0575446\r\n2459.41,0.8633444,0.53424,0.312554,0.0574608\r\n2459.43,0.8636852,0.535762,0.312553,0.0574602\r\n2459.75,0.8640279,0.535894,0.312531,0.057452\r\n2460.65,0.86437135,0.536255,0.31247,0.0574218\r\n2463.19,0.8647148,0.537285,0.312298,0.0573359\r\n2466.95,0.86505825,0.538808,0.312043,0.0572088\r\n2466.97,0.86540065,0.54033,0.312041,0.057208\r\n2467.29,0.86574335,0.540379,0.312019,0.057197\r\n2468.14,0.8660514,0.540508,0.311977,0.0571686\r\n2470.96,0.8663898,0.540938,0.311833,0.0570732\r\n2474.46,0.8667325,0.541473,0.311655,0.0568952\r\n2475.63,0.8670879,0.541947,0.311596,0.0568357\r\n2475.64,0.86746525,0.541952,0.311595,0.0568351\r\n2478.22,0.86783435,0.542995,0.311464,0.0567042\r\n2478.24,0.8681771,0.544517,0.311463,0.0567031\r\n2479.39,0.8685207,0.544983,0.311405,0.0566447\r\n2482,0.8688358,0.54604,0.311272,0.0565121\r\n2482.03,0.86914305,0.547562,0.311271,0.0565109\r\n2482.23,0.8694483,0.547604,0.311261,0.0565005\r\n2485.89,0.8697275,0.548348,0.311012,0.0563144\r\n2486.88,0.87003325,0.548549,0.310945,0.056281\r\n2489.52,0.87033595,0.549085,0.310766,0.0561918\r\n2489.55,0.87066775,0.550607,0.310765,0.056191\r\n2493.31,0.8709804,0.55213,0.31051,0.0560639\r\n2493.33,0.8712654,0.553652,0.310508,0.0560631\r\n2493.43,0.8715599,0.553693,0.310501,0.0560597\r\n2494.38,0.87187255,0.554078,0.310453,0.0560275\r\n2497.09,0.8721789,0.555175,0.310315,0.055936\r\n2497.1,0.872493,0.555176,0.310315,0.0559356\r\n2501.87,0.872799,0.555904,0.310072,0.0558148\r\n2504.58,0.8731313,0.556317,0.309935,0.0557463\r\n2508.34,0.87342575,0.55784,0.309744,0.0556511\r\n2508.36,0.87369405,0.559362,0.309743,0.0556505\r\n2508.37,0.87397235,0.559364,0.309743,0.0556504\r\n2509.37,0.87425255,0.55977,0.309675,0.0556251\r\n2512.04,0.87451325,0.560852,0.309493,0.0555574\r\n2512.12,0.8747788,0.560885,0.309488,0.0555547\r\n2513.11,0.87508085,0.561086,0.309421,0.0555212\r\n2516.86,0.8753825,0.561847,0.309167,0.0553945\r\n2519.57,0.8756771,0.562397,0.308983,0.055303\r\n2519.62,0.87598175,0.562408,0.308983,0.0553013\r\n2523.25,0.8762875,0.563878,0.308984,0.0551786\r\n2523.38,0.87655525,0.56393,0.308984,0.0551719\r\n2523.39,0.87683255,0.565072,0.308984,0.0551711\r\n2524.36,0.87713015,0.565461,0.308984,0.0551222\r\n2527.15,0.87744675,0.566595,0.308984,0.0549801\r\n2530.91,0.8777429,0.568117,0.308985,0.0547891\r\n2530.94,0.87804585,0.569639,0.308985,0.0547879\r\n2531.85,0.87830335,0.570008,0.308985,0.0547417\r\n2534.51,0.87853275,0.571088,0.308985,0.0546062\r\n2534.7,0.8787239,0.571162,0.308981,0.0545969\r\n2534.72,0.8789031,0.572684,0.30898,0.0545958\r\n2538.18,0.87908125,0.574087,0.308893,0.0544198\r\n2538.48,0.8792491,0.574207,0.308885,0.0544199\r\n2538.5,0.8793906,0.575729,0.308885,0.0544199\r\n2542.26,0.87952905,0.577251,0.308789,0.0544204\r\n2543.05,0.8796321,0.577569,0.308769,0.0544206\r\n2546.02,0.87966205,0.578774,0.308694,0.054421\r\n2549.46,0.8796611,0.580165,0.308607,0.0544215\r\n2549.78,0.87965945,0.580297,0.308585,0.0544216\r\n2549.8,0.87961385,0.581819,0.308584,0.0544216\r\n2550.5,0.87951295,0.5821,0.308537,0.0544217\r\n2553.13,0.87940305,0.583166,0.308358,0.0544221\r\n2553.56,0.8792829,0.583341,0.308329,0.0544075\r\n2553.59,0.8791286,0.584864,0.308327,0.0544067\r\n2554.21,0.8789837,0.585117,0.308285,0.0543856\r\n2554.22,0.8787585,0.585122,0.308284,0.0543852\r\n2557.35,0.8785258,0.586386,0.308072,0.0542796\r\n2557.37,0.87828775,0.587908,0.308071,0.0542788\r\n2557.93,0.87802245,0.587994,0.308033,0.0542598\r\n2560.66,0.87771965,0.58841,0.307848,0.0541677\r\n2564.34,0.87742325,0.588971,0.307755,0.0540433\r\n2564.86,0.87712765,0.589051,0.307741,0.0540256\r\n2564.88,0.87682705,0.590573,0.307741,0.0540248\r\n2565.38,0.87647245,0.590776,0.307728,0.0540079\r\n2568.64,0.87611115,0.592096,0.307646,0.0538977\r\n2569.1,0.87567835,0.59228,0.307634,0.0538823\r\n2572.4,0.8752799,0.593618,0.307551,0.0537706\r\n2572.42,0.87487325,0.59514,0.30755,0.0537698\r\n2572.8,0.87450485,0.595217,0.30754,0.0537571\r\n2572.81,0.8740876,0.595219,0.30754,0.0537568\r\n2575.54,0.87372315,0.595774,0.307471,0.0536645\r\n2575.6,0.87334855,0.595785,0.30747,0.0536616\r\n2576.52,0.87290485,0.595974,0.307407,0.0536145\r\n2579.92,0.8724735,0.596664,0.307177,0.053442\r\n2579.94,0.87199805,0.598186,0.307175,0.0534408\r\n2580.24,0.87153175,0.598305,0.307155,0.0534258\r\n2583.7,0.87106555,0.599708,0.30692,0.0532498\r\n2583.73,0.8706133,0.60123,0.306919,0.0532487\r\n2583.75,0.8701847,0.602752,0.306917,0.0532475\r\n2583.95,0.86969125,0.602835,0.306903,0.0532371\r\n2586.8,0.8691775,0.603987,0.30671,0.0530926\r\n2587.51,0.86865885,0.604275,0.306674,0.0530565\r\n2587.65,0.8682214,0.604333,0.306667,0.0530493\r\n2587.67,0.8677351,0.604342,0.306666,0.0530482\r\n2590.48,0.86722555,0.605478,0.306523,0.0529057\r\n2591.27,0.8668139,0.605798,0.306483,0.0529058\r\n2591.36,0.866441,0.60588,0.306478,0.0529058\r\n2591.38,0.8659655,0.60589,0.306478,0.0529058\r\n2595.08,0.8653759,0.609055,0.30629,0.0529064\r\n2598.79,0.8648413,0.612229,0.306101,0.052907\r\n2598.84,0.86424855,0.612268,0.306099,0.052907\r\n2601.73,0.86365895,0.61344,0.305952,0.0529074\r\n2602.51,0.86313515,0.613753,0.305932,0.0529075\r\n2602.6,0.86249525,0.61379,0.30593,0.0529075\r\n2605.42,0.8618295,0.614935,0.305858,0.052908\r\n2606.2,0.8612728,0.615248,0.305839,0.0528818\r\n2606.21,0.86075465,0.615253,0.305838,0.0528814\r\n2606.22,0.8602313,0.615258,0.305838,0.0528811\r\n2606.36,0.8597112,0.615313,0.305834,0.0528764\r\n2606.38,0.8591479,0.616835,0.305834,0.0528757\r\n2609.91,0.8586056,0.618266,0.305745,0.0527563\r\n2610.14,0.8580679,0.618358,0.305739,0.0527486\r\n2610.16,0.85738315,0.61988,0.305738,0.0527478\r\n2613.63,0.85680795,0.621283,0.30565,0.0526307\r\n2613.92,0.8562649,0.621402,0.305643,0.0526207\r\n2613.95,0.85572535,0.622924,0.305642,0.0526199\r\n2613.97,0.8550932,0.624447,0.305642,0.0526192\r\n2616.63,0.8545743,0.625525,0.305574,0.0525292\r\n2616.68,0.8541178,0.625543,0.305573,0.0525269\r\n2617.34,0.85365235,0.625813,0.305528,0.052493\r\n2617.73,0.8532456,0.625969,0.305502,0.0524734\r\n2617.75,0.8527701,0.627491,0.3055,0.0524722\r\n2621.05,0.8523364,0.627995,0.305277,0.0523047\r\n2621.06,0.85194315,0.627996,0.305276,0.0523041\r\n2624.77,0.85157225,0.628563,0.305024,0.0521154\r\n2625.24,0.8511628,0.628634,0.304993,0.0520917\r\n2625.26,0.85066185,0.630156,0.304991,0.0520906\r\n2627.87,0.85015395,0.631213,0.304814,0.0519579\r\n2628.49,0.84963605,0.631462,0.304783,0.0519267\r\n2629.02,0.8492633,0.631679,0.304756,0.0518996\r\n2629.05,0.84884355,0.633201,0.304754,0.0518984\r\n2631.57,0.84841125,0.634221,0.304626,0.0517704\r\n2632.2,0.84803755,0.634479,0.304594,0.0517489\r\n2632.81,0.8476062,0.634723,0.304563,0.0517285\r\n2635.92,0.847162,0.635983,0.304405,0.0516233\r\n2636.57,0.84665545,0.636246,0.304372,0.0516014\r\n2639.63,0.84617145,0.636869,0.304217,0.0514977\r\n2642.77,0.8456879,0.637507,0.304057,0.0513916\r\n2642.81,0.84521245,0.637515,0.304055,0.0513906\r\n2643.36,0.8447433,0.637626,0.304018,0.0513768\r\n2644.06,0.84425935,0.637769,0.30397,0.0513589\r\n2647.09,0.8438451,0.63869,0.303765,0.0512823\r\n2647.82,0.84333165,0.638911,0.303716,0.0512639\r\n2650.8,0.84287795,0.640221,0.303513,0.0511883\r\n2654.01,0.84241495,0.641628,0.303296,0.0511071\r\n2657.71,0.8419453,0.643251,0.302919,0.0510134\r\n2659.1,0.8415144,0.64386,0.302778,0.0509782\r\n2662,0.84108725,0.645032,0.302483,0.0509049\r\n2662.86,0.84064715,0.645382,0.302395,0.050883\r\n2665.2,0.84019305,0.646332,0.302156,0.0508237\r\n2666.62,0.83976175,0.646905,0.30206,0.0507879\r\n2669.45,0.8393435,0.64805,0.301868,0.0507163\r\n2670.38,0.8389376,0.648428,0.301805,0.0506927\r\n2672.66,0.8385954,0.64912,0.301651,0.0506351\r\n2674.13,0.83826655,0.64957,0.301551,0.0505851\r\n2676.4,0.8380005,0.650489,0.301397,0.0505084\r\n2677.89,0.83776645,0.651093,0.301321,0.050458\r\n2677.92,0.8375329,0.652615,0.30132,0.0504572\r\n2683.86,0.8373049,0.65322,0.301018,0.0502563\r\n2684.39,0.83708055,0.653274,0.300991,0.0502428\r\n2689.14,0.8368716,0.653758,0.300749,0.0501225\r\n2691.34,0.83667755,0.654093,0.300638,0.050067\r\n2691.87,0.8365447,0.654173,0.300602,0.0500536\r\n2696.64,0.8364282,0.654901,0.300279,0.0499329\r\n2698.8,0.83633055,0.655011,0.300132,0.0498781\r\n2699.34,0.83624545,0.655039,0.300095,0.0498598\r\n2702.54,0.8361878,0.655202,0.299879,0.0497517\r\n2710.01,0.83613995,0.655583,0.299118,0.0494992\r\n2710.56,0.8361173,0.655611,0.299062,0.0494853\r\n2711.59,0.83611185,0.655664,0.298957,0.0494591\r\n2713.73,0.83614685,0.655519,0.298739,0.0494049\r\n2722.79,0.83618455,0.654905,0.298124,0.0491756\r\n2724.93,0.8362223,0.65476,0.297979,0.0491214\r\n2724.95,0.83626075,0.654758,0.297978,0.049121\r\n2725.51,0.836328,0.65472,0.29794,0.049102\r\n2733.99,0.8364025,0.654145,0.297365,0.0488153\r\n2736.13,0.8364771,0.654,0.29722,0.048743\r\n2736.15,0.8365536,0.653999,0.297218,0.0487422\r\n2736.73,0.8366517,0.653959,0.297159,0.0487275\r\n2745.19,0.83675515,0.653386,0.296297,0.0485134\r\n2747.33,0.83686825,0.653223,0.29608,0.0484594\r\n2751.1,0.83698285,0.652935,0.295696,0.0483639\r\n2751.69,0.83709745,0.65289,0.295635,0.0483488\r\n2758.52,0.8372121,0.652369,0.29494,0.048176\r\n2759.17,0.83732915,0.65232,0.294874,0.0481596\r\n2760.12,0.83745585,0.652247,0.294777,0.0481354\r\n2766.04,0.83757135,0.651342,0.294175,0.0479857\r\n2766.65,0.8376871,0.651249,0.294113,0.0479858\r\n2767.58,0.8378043,0.651106,0.294018,0.0479859\r\n2769.71,0.83792555,0.65078,0.293801,0.0479863\r\n2775.04,0.83803805,0.649966,0.293258,0.0479871\r\n2777.25,0.8381525,0.649514,0.293033,0.0479874\r\n2780.91,0.8382678,0.648768,0.292661,0.0478016\r\n2781.6,0.8383824,0.648626,0.29259,0.0477664\r\n2782.49,0.8384682,0.648445,0.2925,0.0477213\r\n2788.37,0.83854675,0.647545,0.291901,0.0474225\r\n2789.94,0.83862525,0.647304,0.291794,0.0473425\r\n2792.19,0.8387018,0.646617,0.291642,0.0472286\r\n2792.81,0.8387524,0.646425,0.2916,0.0472075\r\n2793.66,0.83876755,0.646163,0.291542,0.0471786\r\n2799.57,0.83875355,0.645361,0.291141,0.0469791\r\n2803.39,0.83873495,0.644842,0.290752,0.0468498\r\n2804.01,0.8387134,0.644758,0.29069,0.0468499\r\n2804.85,0.8386792,0.644643,0.290603,0.0468501\r\n2808.57,0.8386352,0.643502,0.290225,0.0468506\r\n2810.76,0.838561,0.643055,0.290002,0.046851\r\n2815.2,0.8384811,0.642148,0.289322,0.0468516\r\n2816.02,0.83838885,0.641981,0.289197,0.0468518\r\n2818.22,0.8382932,0.641082,0.288861,0.0468521\r\n2818.34,0.83818365,0.641033,0.288852,0.0468521\r\n2819.74,0.83805635,0.64046,0.288745,0.0468167\r\n2822.67,0.83792035,0.639561,0.288522,0.0467425\r\n2823.46,0.8377685,0.639319,0.288461,0.0467225\r\n2826.4,0.83759365,0.638718,0.288237,0.0466481\r\n2830.91,0.83740735,0.637798,0.287893,0.0465339\r\n2833.15,0.8372204,0.637455,0.287722,0.0464772\r\n2833.28,0.83701145,0.637435,0.287709,0.0464739\r\n2837.59,0.83679855,0.636776,0.28727,0.0463281\r\n2838.37,0.83657855,0.636657,0.287191,0.046302\r\n2840.61,0.83638115,0.635737,0.286962,0.046226\r\n2842.08,0.836191,0.635136,0.286813,0.0461764\r\n2844.49,0.83598205,0.634768,0.286568,0.0460951\r\n2848.79,0.83577345,0.63411,0.28613,0.0459862\r\n2849.54,0.83556975,0.633995,0.286053,0.0459672\r\n2851.81,0.83536855,0.633066,0.285822,0.0459097\r\n2853.25,0.8351615,0.632474,0.285724,0.0458732\r\n2859.43,0.83495675,0.631213,0.285305,0.0457168\r\n2859.99,0.8347595,0.631099,0.285268,0.0457027\r\n2860.7,0.83455115,0.630953,0.285219,0.0456846\r\n2863.01,0.83432365,0.630601,0.285063,0.0456262\r\n2868.16,0.8340957,0.629813,0.284538,0.0454958\r\n2871.18,0.83386825,0.628575,0.284231,0.0454193\r\n2871.87,0.8336439,0.628291,0.28416,0.0454018\r\n2874.2,0.8334417,0.627577,0.283923,0.0453429\r\n2874.37,0.83324815,0.627525,0.283911,0.0453386\r\n2874.91,0.83305525,0.627359,0.283875,0.0453386\r\n2875.59,0.83288435,0.62715,0.283829,0.0453388\r\n2882.38,0.8327191,0.625764,0.283368,0.0453398\r\n2883.04,0.8325713,0.625629,0.283323,0.0453399\r\n2885.4,0.83244095,0.625269,0.283164,0.0453402\r\n2889.32,0.83232585,0.62467,0.282765,0.0453408\r\n2890.5,0.8322482,0.624489,0.282644,0.0453008\r\n2896.59,0.832175,0.623245,0.282024,0.0450949\r\n2897.34,0.8321126,0.623092,0.281986,0.0450695\r\n2897.95,0.8320682,0.622968,0.281955,0.0450489\r\n2900.52,0.8320696,0.622443,0.281824,0.044962\r\n2905.4,0.832089,0.621447,0.281576,0.0448385\r\n2908.57,0.8321147,0.619165,0.281415,0.0447583\r\n2909.1,0.8321523,0.618784,0.281388,0.0447449\r\n2909.12,0.8322284,0.620306,0.281387,0.0447443\r\n2911.53,0.83232625,0.617312,0.281265,0.0446834\r\n2912.79,0.8324278,0.61574,0.281179,0.0446515\r\n2912.81,0.83255465,0.617262,0.281178,0.0446509\r\n2915.46,0.83268765,0.615077,0.280998,0.0445838\r\n2916.51,0.8328402,0.614219,0.280928,0.0445486\r\n2919.8,0.83299595,0.61287,0.280704,0.0444373\r\n2920.22,0.8331708,0.612697,0.280676,0.044423\r\n2922.73,0.83335315,0.61167,0.280506,0.0443382\r\n2923.93,0.83355035,0.611176,0.280465,0.0442975\r\n2926.67,0.83374225,0.610055,0.280372,0.044205\r\n2927.29,0.833933,0.609801,0.280352,0.0441893\r\n2927.65,0.83412545,0.609654,0.280339,0.0441802\r\n2931.03,0.83431935,0.608268,0.280225,0.0440945\r\n2931.36,0.83453625,0.608133,0.280214,0.0440862\r\n2933.93,0.83474865,0.607344,0.280127,0.0440211\r\n2934.78,0.8349385,0.607084,0.280105,0.0439997\r\n2935.08,0.83514055,0.606992,0.280098,0.043992\r\n2938.8,0.8353614,0.60547,0.280004,0.043898\r\n2941.61,0.83558715,0.604317,0.279933,0.0438267\r\n2942.27,0.83580225,0.604046,0.279916,0.0438268\r\n2942.51,0.8360362,0.603949,0.27991,0.0438269\r\n2946.23,0.83628795,0.602808,0.279816,0.0438274\r\n2948.88,0.83655765,0.602268,0.279749,0.0438278\r\n2953.51,0.8368234,0.601323,0.279749,0.0438285\r\n2953.68,0.8370943,0.601287,0.279749,0.0438286\r\n2956.56,0.83736835,0.600108,0.27975,0.043829\r\n2957.4,0.8376507,0.599765,0.27975,0.0438078\r\n2961,0.83792475,0.59829,0.279751,0.0437167\r\n2961.11,0.83819865,0.598244,0.279751,0.0437138\r\n2963.82,0.8384755,0.597412,0.279751,0.0436451\r\n2964.83,0.8387784,0.597103,0.279802,0.0436197\r\n2968.49,0.8390644,0.594087,0.279989,0.0435271\r\n2968.52,0.8393473,0.594059,0.279991,0.0435262\r\n2968.54,0.83963415,0.595581,0.279992,0.0435256\r\n2971.5,0.83991985,0.593143,0.280143,0.0434508\r\n2972.24,0.8402413,0.592538,0.28018,0.043426\r\n2972.25,0.84060935,0.592533,0.280181,0.0434256\r\n2975.95,0.8409571,0.591016,0.28037,0.0433004\r\n2976.01,0.841356,0.590993,0.280373,0.0432985\r\n2978.78,0.84177115,0.589856,0.280514,0.0432046\r\n2979.66,0.84216315,0.589495,0.280574,0.0431748\r\n2979.77,0.84255965,0.589458,0.280581,0.0431714\r\n2982.71,0.8429582,0.588405,0.280781,0.0430719\r\n2987.1,0.8433556,0.586832,0.281079,0.0430726\r\n2987.12,0.8437321,0.588354,0.281081,0.0430726\r\n2987.26,0.84412085,0.588242,0.28109,0.0430726\r\n2990,0.84448765,0.585977,0.281277,0.0430731\r\n2990.81,0.84489995,0.585311,0.281318,0.0430732\r\n2997.65,0.8452957,0.583914,0.281667,0.0430742\r\n2998.26,0.8456853,0.58379,0.281698,0.0430588\r\n2998.49,0.8460661,0.583755,0.28171,0.0430531\r\n3002.23,0.84647425,0.583183,0.281901,0.0429583\r\n3004.96,0.84688985,0.582765,0.28204,0.0428892\r\n3005.72,0.84727805,0.582649,0.282092,0.0428701\r\n3005.98,0.84770925,0.582623,0.28211,0.0428634\r\n3012.6,0.8481091,0.581949,0.282559,0.042696\r\n3013.47,0.84846155,0.58186,0.282619,0.0426664\r\n3016.19,0.84882025,0.581584,0.282803,0.0425746\r\n3016.91,0.84916395,0.581509,0.282853,0.04255\r\n3020.97,0.8495329,0.580889,0.283128,0.0424129\r\n3023.8,0.8499084,0.580456,0.283321,0.0423172\r\n3024.37,0.8502668,0.580369,0.283359,0.0423173\r\n3024.72,0.85059635,0.580321,0.283383,0.0423173\r\n3027.41,0.85093955,0.579956,0.283566,0.0423177\r\n3035.56,0.85128235,0.578849,0.283982,0.042319\r\n3035.94,0.85159945,0.57879,0.284001,0.042319\r\n3038.75,0.85190185,0.578361,0.284145,0.0423194\r\n3042.37,0.8521562,0.577807,0.284329,0.04232\r\n3043.02,0.8524054,0.577708,0.284373,0.0423201\r\n3043.41,0.8526151,0.577648,0.2844,0.0423202\r\n3047.13,0.85279435,0.577078,0.284653,0.0423207\r\n3050.47,0.85293535,0.576568,0.28488,0.0423212\r\n3053.59,0.85306825,0.57625,0.285092,0.0423217\r\n3053.7,0.8531828,0.576239,0.285097,0.0423217\r\n3058.33,0.8532214,0.575767,0.285334,0.0422044\r\n3061.67,0.8532218,0.575428,0.285504,0.04212\r\n3065.79,0.8531759,0.575008,0.285714,0.0420156\r\n3068.55,0.853122,0.574727,0.285855,0.0419458\r\n3068.64,0.8529982,0.574718,0.285861,0.0419435\r\n3072.86,0.8528033,0.574288,0.286148,0.0418008\r\n3076.98,0.85255695,0.573658,0.286428,0.0416616\r\n3079.77,0.8523041,0.573231,0.286618,0.0415672\r\n3079.84,0.8520394,0.57322,0.286623,0.0415647\r\n3080.32,0.851755,0.573148,0.286655,0.0415647\r\n3080.7,0.85143195,0.573109,0.286681,0.0415648\r\n3084.42,0.85107795,0.57273,0.286934,0.0415654\r\n3088.15,0.85071145,0.57235,0.287187,0.0415659\r\n3090.99,0.8503426,0.572061,0.287381,0.0415664\r\n3091.51,0.84994965,0.572008,0.28742,0.0415664\r\n3094.79,0.8495885,0.571506,0.287671,0.0415669\r\n3095.61,0.84916985,0.571382,0.287733,0.0415463\r\n3098.97,0.84875455,0.570867,0.28799,0.0414613\r\n3099.32,0.8482719,0.570832,0.288017,0.0414523\r\n3105.96,0.84779665,0.570156,0.288524,0.0412843\r\n3106.77,0.8473242,0.570073,0.28858,0.0412637\r\n3109.73,0.84688,0.569771,0.288781,0.0411887\r\n3110.16,0.84642345,0.569728,0.28881,0.0411779\r\n3114.23,0.8459374,0.569106,0.289087,0.0410749\r\n3117.18,0.84547555,0.568654,0.289287,0.0410002\r\n3117.62,0.8450383,0.568587,0.289317,0.0409891\r\n3117.95,0.8446058,0.568553,0.28934,0.0409806\r\n3121.68,0.84415155,0.568174,0.289593,0.0408863\r\n3124.68,0.84367605,0.567868,0.289797,0.0408105\r\n3128.4,0.84320325,0.567489,0.29005,0.040811\r\n3128.81,0.842766,0.567447,0.290216,0.0408111\r\n3129.14,0.84234365,0.567398,0.290347,0.0408111\r\n3132.16,0.84191425,0.566935,0.291573,0.0408116\r\n3135.92,0.8414862,0.56636,0.293095,0.0408122\r\n3136.27,0.84108075,0.566307,0.293131,0.0408122\r\n3136.59,0.8406506,0.566241,0.293164,0.0408123\r\n3139.62,0.8402475,0.565622,0.293472,0.0408127\r\n3143.72,0.8398483,0.564786,0.293889,0.0408134\r\n3144.04,0.8394931,0.564753,0.293922,0.0408134\r\n3147.15,0.8391129,0.564437,0.294239,0.0408139\r\n3151.5,0.83873975,0.563994,0.294902,0.0408145\r\n3154.57,0.8383695,0.563681,0.295371,0.040815\r\n3154.64,0.838029,0.563674,0.295381,0.0408127\r\n3154.91,0.83769795,0.563646,0.295409,0.0408035\r\n3158.96,0.83735405,0.563027,0.295821,0.0406666\r\n3162.37,0.8370119,0.562506,0.296168,0.0405514\r\n3162.69,0.83670785,0.562473,0.296201,0.0405404\r\n3165.78,0.8364267,0.562159,0.296515,0.0404362\r\n3165.87,0.8361741,0.562149,0.296524,0.0404362\r\n3173.36,0.83592465,0.561387,0.297667,0.0404373\r\n3173.56,0.8356777,0.561366,0.297698,0.0404374\r\n3177.64,0.83544995,0.561089,0.298321,0.040438\r\n3180.72,0.8352508,0.56088,0.298791,0.0404385\r\n3180.85,0.83504935,0.560872,0.29881,0.0404353\r\n3184.76,0.8348459,0.560607,0.299605,0.0403362\r\n3188.35,0.83466845,0.559875,0.300333,0.0402455\r\n3192.21,0.8345323,0.559086,0.300596,0.0401476\r\n3192.58,0.83439425,0.559048,0.300621,0.0401383\r\n3195.67,0.83427335,0.558734,0.300831,0.0400602\r\n3199.57,0.83417925,0.558337,0.301096,0.0400608\r\n3203.41,0.83410015,0.557946,0.301357,0.0400614\r\n3203.79,0.83391675,0.557887,0.301383,0.0400615\r\n3210.61,0.833732,0.556844,0.301847,0.0400625\r\n3210.79,0.83356675,0.556817,0.301859,0.0400625\r\n3210.86,0.83341325,0.556805,0.30186,0.0400625\r\n3211.23,0.8332995,0.556781,0.30187,0.0400626\r\n3211.24,0.8330335,0.55678,0.30187,0.0400626\r\n3214.93,0.8328436,0.55653,0.301965,0.0400632\r\n3214.95,0.83250145,0.556528,0.301965,0.0400632\r\n3218.65,0.8320893,0.556278,0.30206,0.0400637\r\n3222.06,0.83167105,0.556046,0.302147,0.0400642\r\n3222.36,0.83112475,0.555986,0.302155,0.0400643\r\n3225.56,0.8305711,0.555332,0.302237,0.0400648\r\n3225.74,0.8300097,0.555295,0.302241,0.0400602\r\n3226.05,0.82949085,0.555232,0.302262,0.0400524\r\n3226.06,0.82901495,0.55523,0.302263,0.0400521\r\n3226.07,0.8283977,0.555227,0.302264,0.0400518\r\n3229.51,0.82777535,0.554525,0.302498,0.0399647\r\n3229.76,0.8270074,0.5545,0.302515,0.0399584\r\n3233.46,0.82632205,0.554124,0.302766,0.0398649\r\n3233.48,0.82557495,0.554121,0.302767,0.0398643\r\n3236.97,0.82486915,0.553766,0.303004,0.0397761\r\n3240.5,0.82416235,0.553406,0.303185,0.0396865\r\n3240.71,0.82345305,0.553385,0.303195,0.0396866\r\n3240.91,0.8227874,0.553354,0.303206,0.0396866\r\n3240.92,0.8221217,0.553352,0.303206,0.0396866\r\n3244.67,0.82168435,0.552779,0.303397,0.0396872\r\n3244.7,0.82124115,0.552775,0.303399,0.0396872\r\n3248.17,0.820956,0.552245,0.303576,0.0396877\r\n3248.46,0.82081695,0.552186,0.30359,0.0396877\r\n3248.48,0.8207495,0.552181,0.303592,0.0396877\r\n3251.71,0.8208183,0.55152,0.303757,0.0396882\r\n3251.92,0.82090095,0.551477,0.303767,0.0396883\r\n3252.24,0.82103435,0.551413,0.303789,0.0396883\r\n3252.25,0.82116775,0.551411,0.303789,0.0396883\r\n3252.26,0.8213392,0.551409,0.30379,0.0396883\r\n3252.28,0.82172,0.551404,0.303792,0.0396883\r\n3255.62,0.82211975,0.550724,0.304018,0.0396888\r\n3256.04,0.8227478,0.550695,0.304047,0.0396889\r\n3256.07,0.82337585,0.550693,0.304049,0.0396889\r\n3256.09,0.82410435,0.550692,0.30405,0.0396889\r\n3256.1,0.82493425,0.550691,0.304051,0.0396889\r\n3256.11,0.82584395,0.55069,0.304052,0.0396889\r\n3256.13,0.8268334,0.550689,0.304053,0.0396889\r\n3256.16,0.82786095,0.550687,0.304055,0.0396889\r\n3259.92,0.82892655,0.550432,0.304311,0.0396895\r\n3259.94,0.82995405,0.550431,0.304312,0.0396895\r\n3259.95,0.83094935,0.55043,0.304313,0.0396895\r\n3259.96,0.8320149,0.550429,0.304314,0.0396895\r\n3259.98,0.83308045,0.550428,0.304315,0.0396895\r\n3260.01,0.83414405,0.550426,0.304317,0.0396895\r\n3263.15,0.83521595,0.550213,0.30453,0.03969\r\n3263.77,0.8363576,0.550171,0.304546,0.0396901\r\n3263.79,0.83749925,0.55017,0.304547,0.0396901\r\n3263.8,0.8386409,0.550169,0.304547,0.0396901\r\n3263.81,0.83978255,0.550168,0.304547,0.0396901\r\n3263.84,0.84091395,0.550167,0.304548,0.0396901\r\n3266.66,0.8420398,0.549975,0.30462,0.0396905\r\n3266.82,0.84310535,0.549965,0.304624,0.0396905\r\n3267.59,0.8441709,0.549805,0.304644,0.0396907\r\n3267.62,0.8452745,0.549801,0.304645,0.0396907\r\n3267.64,0.8463781,0.549796,0.304645,0.0396907\r\n3267.65,0.84744365,0.549794,0.304645,0.0396907\r\n3271.4,0.8485092,0.549029,0.304741,0.0396912\r\n3271.42,0.8497865,0.549024,0.304742,0.0396912\r\n3274.27,0.85137575,0.548444,0.304815,0.0396917\r\n3278.1,0.85303115,0.547857,0.304913,0.0396923\r\n3279.13,0.85483875,0.5477,0.304939,0.0396924\r\n3279.19,0.8566918,0.547691,0.304941,0.0396924\r\n3281.61,0.85847445,0.547321,0.305003,0.0396928\r\n3281.72,0.86025655,0.547303,0.305006,0.0396899\r\n3282.98,0.86221635,0.547175,0.305038,0.0396581\r\n3283.04,0.86429035,0.547169,0.305039,0.0396566\r\n3286.83,0.86651655,0.546783,0.305136,0.0395606\r\n3286.89,0.868895,0.546777,0.305138,0.0395592\r\n3290.68,0.8713876,0.546391,0.305235,0.0394631\r\n3290.74,0.8739322,0.546385,0.305236,0.0394617\r\n3292.92,0.87648055,0.546163,0.305292,0.0394065\r\n3293.05,0.8790873,0.546135,0.305296,0.0394031\r\n3294.53,0.8818082,0.545834,0.305334,0.0393657\r\n3294.59,0.8845514,0.545822,0.305335,0.0393642\r\n3296.55,0.8873484,0.545421,0.305385,0.0393145\r\n3298.38,0.89029765,0.545048,0.305432,0.0393148\r\n3298.44,0.8932675,0.545036,0.305434,0.0393148\r\n3300.37,0.896043,0.544642,0.305483,0.0393151\r\n3302.23,0.89862075,0.544357,0.305531,0.0393154\r\n3306.03,0.90128455,0.543777,0.305628,0.039316\r\n3306.08,0.9038454,0.543768,0.305629,0.039316\r\n3307.83,0.90629385,0.543502,0.305674,0.0393163\r\n3308.01,0.90883045,0.543474,0.305679,0.0393163\r\n3309.88,0.91136925,0.543188,0.305742,0.0393166\r\n3311.5,0.9138048,0.54294,0.305798,0.0393168\r\n3313.66,0.91601265,0.542609,0.305871,0.0393171\r\n3315.28,0.91793935,0.542362,0.305926,0.0393174\r\n3319.22,0.91963285,0.541557,0.306061,0.039318\r\n3321.08,0.92105085,0.541177,0.306061,0.0393183\r\n3322.73,0.92225465,0.540841,0.306061,0.0393185\r\n3324.76,0.92336455,0.54053,0.306062,0.0393188\r\n3326.45,0.92429705,0.540273,0.306062,0.0393191\r\n3328.44,0.92485595,0.539967,0.306062,0.0393194\r\n3330.19,0.9251193,0.5397,0.306062,0.0393197\r\n3332.07,0.92548105,0.539413,0.306063,0.0393199\r\n3332.12,0.9253248,0.539404,0.306063,0.03932\r\n3334.17,0.9248288,0.539091,0.306063,0.0393203\r\n3335.69,0.92442955,0.538859,0.306141,0.0393205\r\n3335.75,0.92353595,0.53885,0.306143,0.0393205\r\n3337.65,0.9221854,0.53856,0.30624,0.0393208\r\n3339.31,0.9209378,0.538333,0.306325,0.0393211\r\n3339.37,0.9194684,0.538326,0.306328,0.0393211\r\n3341.39,0.9177373,0.53805,0.306432,0.0393214\r\n3342.99,0.9153702,0.537833,0.306513,0.0393216\r\n3346.56,0.91310635,0.537349,0.306695,0.0393222\r\n3346.62,0.91107015,0.537341,0.306698,0.0393222\r\n3346.67,0.9086562,0.537333,0.306701,0.0393222\r\n3348.83,0.906221,0.537039,0.306811,0.0393225\r\n3349.13,0.90364285,0.536994,0.306826,0.0393225\r\n3350.18,0.9013599,0.536833,0.306853,0.0393227\r\n3350.24,0.89859325,0.536825,0.306855,0.0393227\r\n3353.81,0.8961201,0.536279,0.306946,0.0393233\r\n3353.86,0.89357525,0.53627,0.306947,0.0393233\r\n3356.29,0.89122215,0.535899,0.30701,0.0393236\r\n3356.34,0.88855875,0.535891,0.307011,0.0393236\r\n3357.49,0.8860328,0.535716,0.30704,0.0393238\r\n3361.17,0.88347875,0.535153,0.307134,0.0393244\r\n3363.75,0.88069915,0.534759,0.307201,0.0393248\r\n3364.08,0.8781495,0.534707,0.307209,0.0393248\r\n3364.79,0.87605685,0.534599,0.307257,0.0393249\r\n3364.85,0.8732032,0.534591,0.307261,0.0393249\r\n3368.41,0.87075245,0.534045,0.307503,0.0393255\r\n3368.47,0.86818225,0.534037,0.307507,0.0393255\r\n3371.21,0.8661738,0.533618,0.307693,0.0393259\r\n3371.29,0.8638909,0.533605,0.307699,0.0393259\r\n3372.09,0.86125165,0.533482,0.307753,0.039326\r\n3375.31,0.8590569,0.532991,0.307972,0.0393265\r\n3375.77,0.8566229,0.53292,0.308004,0.0393266\r\n3378.66,0.85434005,0.532478,0.3082,0.039327\r\n3379.4,0.8520572,0.532378,0.30825,0.0393272\r\n3379.45,0.84977435,0.53237,0.308254,0.0393272\r\n3383.02,0.84749145,0.531886,0.308496,0.0393277\r\n3383.08,0.8451315,0.531878,0.3085,0.0393277\r\n3386.24,0.8427465,0.531448,0.308715,0.0393282\r\n3386.53,0.8404636,0.531409,0.308735,0.0393282\r\n3386.7,0.83820805,0.531386,0.308743,0.0393283\r\n3389.85,0.83619635,0.530957,0.308904,0.0393287\r\n3390.38,0.8341846,0.530876,0.308931,0.0393288\r\n3397.31,0.8322272,0.529817,0.309285,0.0393299\r\n3401.19,0.83007795,0.529025,0.309483,0.0393305\r\n3401.49,0.82849905,0.528964,0.309498,0.0393305\r\n3401.57,0.82676625,0.528946,0.309496,0.0393305\r\n3404.76,0.82540595,0.528296,0.309415,0.039331\r\n3412.21,0.82409945,0.527156,0.309226,0.0393321\r\n3416.13,0.8228805,0.526356,0.309127,0.0393327\r\n3416.43,0.8218293,0.526295,0.30912,0.0393328\r\n3416.57,0.8208274,0.526266,0.309115,0.0393328\r\n3419.67,0.82015915,0.525635,0.30901,0.0393333\r\n3423.39,0.8196025,0.524494,0.308885,0.0393338\r\n3427.63,0.81888365,0.524061,0.308741,0.0393345\r\n3431.08,0.818764,0.52371,0.308507,0.039335\r\n3434.58,0.81846,0.523354,0.30827,0.0394246\r\n3435.3,0.8185016,0.523206,0.308221,0.0394431\r\n3438.83,0.81858585,0.522485,0.307982,0.0395335\r\n3442.03,0.81870335,0.521833,0.307819,0.0396153\r\n3446.03,0.81899695,0.52122,0.307616,0.0397178\r\n3449.49,0.8193342,0.520692,0.30744,0.0397183\r\n3453.77,0.8197189,0.520037,0.307223,0.039719\r\n3456.94,0.8201388,0.519552,0.307062,0.0397195\r\n3460.98,0.82057645,0.519141,0.306856,0.0397201\r\n3465.23,0.82102275,0.518708,0.306641,0.0397207\r\n3468.14,0.8214343,0.518412,0.306493,0.0397212\r\n3468.71,0.82178115,0.518354,0.306464,0.0397213\r\n3475.93,0.8220928,0.517619,0.30622,0.0397224\r\n3479.33,0.82240275,0.517272,0.306105,0.0398383\r\n3479.91,0.82273155,0.517213,0.306085,0.039858\r\n3487.15,0.8230448,0.516477,0.305902,0.0401046\r\n3490.53,0.82334005,0.516133,0.305817,0.0401911\r\n3494.85,0.8236122,0.515692,0.305707,0.0403019\r\n3498.89,0.82386865,0.515281,0.305708,0.0404053\r\n3501.72,0.8240983,0.514993,0.305708,0.0404776\r\n3502.1,0.8243557,0.514967,0.305708,0.0404874\r\n3509.8,0.8246132,0.514445,0.305709,0.0406845\r\n3512.92,0.82487655,0.514233,0.30571,0.0407643\r\n3517.05,0.8251391,0.513601,0.30571,0.0408702\r\n3520.38,0.82540755,0.513093,0.305711,0.0408707\r\n3524.75,0.8256611,0.512648,0.305712,0.0408713\r\n3525.08,0.82591465,0.512613,0.305689,0.0408714\r\n3528.26,0.8261654,0.51229,0.305473,0.0408719\r\n3531.57,0.82641665,0.511953,0.305249,0.0409565\r\n3535.95,0.82668575,0.511656,0.304952,0.0410686\r\n3542.77,0.82695495,0.511194,0.30478,0.0412432\r\n3543.22,0.827233,0.511148,0.304768,0.0412547\r\n3550.89,0.82751315,0.510367,0.304574,0.0415161\r\n3553.96,0.82779345,0.510054,0.304496,0.0416208\r\n3554.43,0.8280555,0.51003,0.304484,0.0416369\r\n3555.01,0.8283813,0.51,0.30447,0.0416665\r\n3565.83,0.82871565,0.509451,0.304196,0.0422185\r\n3568.9,0.8290419,0.509295,0.303988,0.042375\r\n3569.39,0.8294076,0.509278,0.303954,0.0424002\r\n3577.03,0.8297747,0.50902,0.303436,0.0424014\r\n3577.48,0.8301295,0.509005,0.303425,0.0424014\r\n3580.1,0.83047475,0.508916,0.303359,0.0424018\r\n3580.6,0.83089445,0.508942,0.303346,0.0424019\r\n3591.97,0.83132475,0.509522,0.303058,0.0429821\r\n3595.06,0.83174005,0.50968,0.30298,0.0431397\r\n3595.56,0.83215915,0.509714,0.302967,0.0431652\r\n3596.21,0.83264805,0.509758,0.302951,0.0432093\r\n3606.28,0.83312835,0.510442,0.302696,0.043894\r\n3606.79,0.83359305,0.510493,0.302683,0.043928\r\n3606.92,0.8340969,0.510507,0.30268,0.0439313\r\n3614.94,0.83462845,0.511324,0.302136,0.0441368\r\n3617.51,0.8351375,0.511586,0.301961,0.0442026\r\n3618.12,0.83567625,0.511647,0.301921,0.044218\r\n3621.74,0.83625045,0.512016,0.301921,0.0443108\r\n3625,0.83687885,0.512348,0.301922,0.0445323\r\n3629.94,0.83747225,0.512851,0.301922,0.0448682\r\n3632.96,0.83804615,0.513159,0.301923,0.0450735\r\n3633.06,0.8386568,0.513169,0.301923,0.045077\r\n3636.23,0.8393107,0.513491,0.301923,0.0451848\r\n3643.72,0.8399674,0.514634,0.301924,0.04544\r\n3644.18,0.8406125,0.514727,0.301925,0.0454557\r\n3644.95,0.84128855,0.514883,0.301925,0.0454949\r\n3648.01,0.8419087,0.515506,0.301925,0.0456514\r\n3651.21,0.8425893,0.516157,0.302007,0.0458148\r\n3658.7,0.84327075,0.5173,0.302199,0.046197\r\n3659.14,0.8439562,0.517387,0.30221,0.046219\r\n3659.94,0.8445942,0.517552,0.302231,0.0462739\r\n3662.96,0.8452651,0.518165,0.302308,0.0464792\r\n3666.2,0.8459824,0.518823,0.302308,0.0466992\r\n3670.36,0.8466726,0.519668,0.302309,0.0469818\r\n3673.7,0.8473336,0.520346,0.30231,0.0472087\r\n3673.71,0.8480018,0.521488,0.30231,0.0472099\r\n3674.94,0.8486575,0.521985,0.30231,0.0472934\r\n3677.47,0.84928095,0.52301,0.30231,0.0474654\r\n3677.91,0.84988985,0.523099,0.30231,0.0474952\r\n3681.58,0.85049915,0.523845,0.302404,0.0477445\r\n3684.97,0.8511501,0.524533,0.302491,0.0479749\r\n3688.73,0.8517612,0.526056,0.302587,0.0482305\r\n3688.75,0.85229085,0.527578,0.302588,0.048232\r\n3689.94,0.852844,0.52806,0.302618,0.0483128\r\n3692.51,0.8533907,0.529101,0.302684,0.0484876\r\n3692.8,0.85390315,0.529145,0.302691,0.0485073\r\n3692.87,0.8544605,0.529155,0.302693,0.0485094\r\n3700,0.8549326,0.530244,0.302876,0.0487527\r\n3700.03,0.8554427,0.531766,0.302876,0.0487534\r\n3703.79,0.8559462,0.533288,0.302973,0.0488816\r\n3704.02,0.8564257,0.533383,0.302979,0.0488895\r\n3704.94,0.85690135,0.533756,0.303002,0.0489599\r\n3707.55,0.85733295,0.534811,0.303069,0.049159\r\n3707.82,0.85777065,0.534921,0.303076,0.0491799\r\n3711.31,0.85825585,0.536334,0.303313,0.0494464\r\n3715.07,0.8587746,0.537856,0.303568,0.0497338\r\n3718.82,0.85926655,0.538998,0.303823,0.0500207\r\n3718.98,0.8597542,0.539065,0.303835,0.0500333\r\n3719.04,0.86021145,0.539088,0.303839,0.0500372\r\n3719.95,0.86066005,0.539454,0.3039,0.0500987\r\n3722.58,0.86110865,0.540521,0.304079,0.0502777\r\n3726.34,0.86155725,0.542044,0.304335,0.0505332\r\n3726.36,0.8620352,0.543566,0.304336,0.0505347\r\n3730.12,0.86248475,0.545088,0.304592,0.0507903\r\n3730.21,0.86293165,0.545123,0.304597,0.0507961\r\n3730.26,0.86341915,0.545146,0.304601,0.0507999\r\n3733.88,0.8638254,0.546611,0.304786,0.0510458\r\n3733.9,0.86424325,0.548133,0.304787,0.0510474\r\n3734.94,0.86464025,0.548555,0.30484,0.0511181\r\n3737.66,0.8650652,0.549656,0.304979,0.0513029\r\n3741.42,0.8654797,0.551178,0.305171,0.0515584\r\n3741.43,0.8658612,0.551559,0.305171,0.0515588\r\n3741.45,0.86626985,0.5527,0.305172,0.0515606\r\n3745.21,0.8666344,0.554223,0.305364,0.0519434\r\n3745.22,0.8669513,0.555365,0.305365,0.0519451\r\n3745.23,0.8672208,0.555745,0.305365,0.0519457\r\n3745.25,0.86751875,0.557267,0.305367,0.051948\r\n3749.01,0.8678162,0.55879,0.30575,0.0523308\r\n3749.03,0.8681316,0.560312,0.305753,0.0523332\r\n3752.66,0.868418,0.561779,0.306122,0.0527021\r\n3752.79,0.86866165,0.561835,0.306135,0.0527114\r\n3752.82,0.86891205,0.563357,0.306138,0.0527129\r\n3753.67,0.8691449,0.563704,0.306225,0.0527713\r\n3756.45,0.86937795,0.564829,0.306508,0.0529599\r\n3756.58,0.86964465,0.564879,0.306512,0.0529684\r\n3760.33,0.8698702,0.566021,0.30664,0.0532236\r\n3760.35,0.8701278,0.567543,0.306641,0.0532251\r\n3763.88,0.8703756,0.568972,0.306761,0.0534648\r\n3764.11,0.8706015,0.569066,0.306769,0.0534886\r\n3764.13,0.87082915,0.570588,0.30677,0.0534909\r\n3767.67,0.8710588,0.572019,0.30689,0.0538505\r\n3767.89,0.87132275,0.572111,0.306913,0.0538737\r\n3771.65,0.871563,0.573633,0.307296,0.0542565\r\n3772.41,0.8718214,0.57394,0.307373,0.0543336\r\n3775.11,0.87208185,0.575032,0.307648,0.0546081\r\n3775.41,0.87234225,0.575156,0.307679,0.054629\r\n3775.44,0.87259555,0.576678,0.307681,0.0546305\r\n3778.89,0.8728507,0.578079,0.308033,0.0548656\r\n3779.2,0.87307655,0.578201,0.308064,0.054886\r\n3779.22,0.8733268,0.579723,0.308066,0.0548876\r\n3782.98,0.87359955,0.581246,0.308449,0.0551431\r\n3786.33,0.8738681,0.582602,0.30879,0.0553708\r\n3786.74,0.8741342,0.582768,0.308832,0.0554125\r\n3790.12,0.8744017,0.584139,0.309177,0.0557571\r\n3790.5,0.874635,0.584291,0.309202,0.0557954\r\n3790.51,0.8748936,0.585432,0.309203,0.0557971\r\n3794.27,0.87512005,0.586955,0.309459,0.0561799\r\n3794.3,0.87534455,0.588477,0.30946,0.0561822\r\n3794.32,0.87557265,0.589999,0.309462,0.0561845\r\n3794.88,0.8757616,0.590225,0.3095,0.0562413\r\n3797.56,0.87594715,0.59131,0.309682,0.0565141\r\n3798.08,0.87609825,0.591522,0.309717,0.0565674\r\n3798.1,0.8762342,0.593044,0.309719,0.0565697\r\n3801.34,0.87635055,0.594357,0.309939,0.0568998\r\n3801.86,0.87645675,0.594566,0.309992,0.0569525\r\n3805.04,0.8765613,0.595854,0.310316,0.0572763\r\n3805.62,0.87664235,0.596089,0.310375,0.0573157\r\n3805.64,0.8767118,0.597611,0.310377,0.0573172\r\n3809.4,0.87678105,0.599134,0.31076,0.0575728\r\n3809.43,0.8768168,0.600656,0.310762,0.0575743\r\n3812.57,0.8768283,0.60193,0.311083,0.0577881\r\n3813.19,0.8768292,0.602179,0.311124,0.0578299\r\n3816.26,0.8768054,0.603425,0.311333,0.058039\r\n3816.95,0.8767782,0.603701,0.31138,0.0581085\r\n3817.28,0.8766761,0.603904,0.311403,0.0581427\r\n3823.79,0.8765405,0.607848,0.311845,0.0588058\r\n3824.49,0.87637025,0.608269,0.311916,0.0588765\r\n3827.49,0.87619115,0.609485,0.312222,0.0591823\r\n3828.25,0.8759631,0.609791,0.312299,0.0592593\r\n3832.18,0.87566895,0.61059,0.312699,0.0596595\r\n3835.02,0.8753576,0.611168,0.312989,0.0599491\r\n3835.74,0.8750459,0.611314,0.313038,0.0600226\r\n3835.76,0.8746666,0.612456,0.313039,0.0600244\r\n3838.72,0.874268,0.613654,0.31324,0.0603255\r\n3839.52,0.87387555,0.613979,0.313294,0.0604072\r\n3839.6,0.8734281,0.614009,0.313299,0.0604147\r\n3843.28,0.87293525,0.615501,0.31355,0.06079\r\n3846.24,0.87243975,0.616701,0.313751,0.0610918\r\n3847.04,0.8718997,0.617024,0.313832,0.0611728\r\n3849.95,0.87135525,0.618201,0.314128,0.0614688\r\n3850.75,0.8708112,0.618528,0.314211,0.061551\r\n3850.8,0.87017855,0.618547,0.314215,0.0615557\r\n3854.56,0.86954705,0.620069,0.314598,0.0619385\r\n3854.58,0.8688423,0.621591,0.3146,0.0619408\r\n3857.47,0.8681944,0.622761,0.314895,0.062235\r\n3858.17,0.86755015,0.623045,0.314966,0.0623063\r\n3858.34,0.86686775,0.623114,0.314983,0.0623236\r\n3861.18,0.8661703,0.624261,0.315272,0.0626121\r\n3862.1,0.86542855,0.624637,0.315366,0.0627535\r\n3865.59,0.8647556,0.625697,0.315721,0.0632853\r\n3865.86,0.8640581,0.625779,0.315748,0.0633261\r\n3868.67,0.8633605,0.626917,0.316034,0.0637547\r\n3868.7,0.8627247,0.62693,0.316038,0.0637582\r\n3869.62,0.86210915,0.627301,0.3161,0.0638515\r\n3869.64,0.86143785,0.628823,0.316102,0.0638538\r\n3873.4,0.86077165,0.630346,0.316357,0.0642366\r\n3876.75,0.86014395,0.631703,0.316585,0.0645777\r\n3877.16,0.8594818,0.631869,0.316613,0.0646194\r\n3879.89,0.85886375,0.632977,0.316799,0.064898\r\n3879.92,0.85824115,0.632988,0.316801,0.0649009\r\n3880.92,0.8575556,0.633391,0.316952,0.0650023\r\n3884.68,0.8569198,0.634914,0.317526,0.0653851\r\n3887.41,0.85627625,0.636022,0.317943,0.0656637\r\n3887.9,0.85567915,0.63622,0.317993,0.0657135\r\n3888.44,0.85496125,0.636437,0.318048,0.0657679\r\n3891.12,0.85422385,0.636846,0.318321,0.0660413\r\n3891.58,0.853485,0.636916,0.318368,0.0661115\r\n3895.93,0.85272415,0.637579,0.31881,0.0667744\r\n3898.61,0.85205225,0.638667,0.319084,0.067184\r\n3898.64,0.8513666,0.638678,0.319087,0.0671869\r\n3899.69,0.8507237,0.639102,0.319246,0.0672934\r\n3899.71,0.8500297,0.640624,0.31925,0.0672957\r\n3902.74,0.8493385,0.641849,0.319711,0.0676039\r\n3903.47,0.8486068,0.642147,0.319823,0.0676786\r\n3906.13,0.8478759,0.642553,0.320229,0.0679496\r\n3909.84,0.8471792,0.643118,0.320607,0.0683272\r\n3910.96,0.84644445,0.643289,0.320721,0.0684413\r\n3913.9,0.84574875,0.644478,0.32102,0.0687402\r\n3914.72,0.84505335,0.644812,0.321104,0.0688241\r\n3914.74,0.8443509,0.646334,0.321106,0.0688265\r\n3917.36,0.84370205,0.647394,0.321373,0.0690929\r\n3918.5,0.8430763,0.647857,0.321547,0.0692093\r\n3921.07,0.84242415,0.648378,0.321938,0.0694705\r\n3924.85,0.8417891,0.649146,0.322515,0.0700475\r\n3926,0.8412693,0.64938,0.322749,0.0702227\r\n3928.56,0.8407624,0.650159,0.323269,0.0706132\r\n3929.75,0.84030575,0.650522,0.323511,0.0707954\r\n3932.35,0.8398932,0.651572,0.324038,0.071191\r\n3932.53,0.83948005,0.651646,0.324057,0.0712188\r\n3933.51,0.8390819,0.652045,0.324157,0.0713689\r\n3936.05,0.8386785,0.65256,0.324415,0.0717559\r\n3941.01,0.83832385,0.653568,0.32492,0.0722609\r\n3943.58,0.8379791,0.654607,0.325182,0.0725221\r\n3944.77,0.83767235,0.655091,0.325364,0.0726437\r\n3947.28,0.8374182,0.655473,0.325747,0.0728991\r\n3951.07,0.8371799,0.656051,0.326324,0.0732848\r\n3952.26,0.8369796,0.656233,0.326567,0.0734065\r\n3958.51,0.83678685,0.657186,0.327836,0.0740424\r\n3958.56,0.83659295,0.657195,0.327848,0.0740511\r\n3959.75,0.8364188,0.657376,0.328029,0.0742324\r\n3963.51,0.8362524,0.658899,0.328602,0.0748059\r\n3966,0.83610945,0.659067,0.328982,0.0751851\r\n3966.05,0.83600115,0.659071,0.32899,0.0751938\r\n3966.15,0.83589515,0.659078,0.329005,0.0752085\r\n3973.49,0.8358126,0.659577,0.330124,0.0763278\r\n3973.54,0.83573965,0.659581,0.330133,0.0763365\r\n3974.73,0.83568355,0.659661,0.330254,0.0765178\r\n3980.98,0.8356248,0.65998,0.33089,0.0774705\r\n3984.77,0.83556165,0.660174,0.331276,0.0778568\r\n3989.69,0.8354985,0.660425,0.332276,0.0783577\r\n3992.21,0.8354409,0.660425,0.332787,0.0786137\r\n3992.27,0.8353777,0.660425,0.332799,0.0786222\r\n3999.77,0.8353153,0.660426,0.333563,0.0796399\r\n4000.9,0.83525445,0.660426,0.333679,0.0797934\r\n4003.44,0.83519785,0.660233,0.333937,0.0801375\r\n4003.5,0.83513885,0.660228,0.333943,0.0801462\r\n4010.93,0.83508685,0.659661,0.335077,0.0812802\r\n4010.99,0.83503245,0.659657,0.335085,0.0812889\r\n4015.83,0.8349781,0.659287,0.335579,0.0820281\r\n4018.42,0.8349261,0.658891,0.335842,0.0824229\r\n4022.22,0.8348763,0.658311,0.336229,0.0830016\r\n4023.29,0.8348253,0.658147,0.336393,0.0831656\r\n4025.91,0.8347725,0.657746,0.336793,0.0835656\r\n4029.71,0.8347274,0.657165,0.337371,0.0839519\r\n4030.75,0.83467915,0.657006,0.337477,0.0840579\r\n4037.13,0.83463225,0.655702,0.338128,0.0847083\r\n4037.14,0.83459185,0.655701,0.338128,0.0847088\r\n4038.2,0.83455435,0.655485,0.338236,0.0848702\r\n4040.93,0.8345205,0.655067,0.338515,0.0852876\r\n4044.63,0.834489,0.654501,0.339079,0.0858515\r\n4045.66,0.83445615,0.654345,0.339235,0.0859904\r\n4048.43,0.8344313,0.65321,0.339657,0.0863662\r\n4049.37,0.8344037,0.652823,0.339753,0.0864942\r\n4055.87,0.83437695,0.65183,0.340415,0.0873753\r\n4056.83,0.83434875,0.651683,0.340513,0.087522\r\n4059.65,0.83432245,0.651106,0.340801,0.0879532\r\n4063.36,0.8342957,0.65035,0.341052,0.088518\r\n4064.28,0.83426955,0.650162,0.341115,0.0886586\r\n4067.99,0.83424325,0.64864,0.341367,0.0892251\r\n4070.85,0.83421885,0.648204,0.341561,0.0896607\r\n4070.88,0.8341932,0.648199,0.341563,0.0896636\r\n4074.5,0.8341672,0.647645,0.34181,0.0900325\r\n4075.45,0.83413685,0.6475,0.341874,0.0901292\r\n4079.16,0.8341021,0.645979,0.342127,0.0905074\r\n4082.07,0.83407055,0.644786,0.342324,0.090804\r\n4082.1,0.83403765,0.644776,0.342326,0.0908086\r\n4082.88,0.833999,0.644457,0.342405,0.0909669\r\n4086.59,0.8339567,0.642936,0.342784,0.0917216\r\n4089.57,0.8339099,0.64248,0.343087,0.0923272\r\n4093.33,0.8338624,0.641906,0.343469,0.0928998\r\n4094.05,0.8338112,0.641795,0.343543,0.09301\r\n4097.06,0.83375925,0.64056,0.34385,0.0934699\r\n4097.76,0.8336991,0.640274,0.343921,0.0935766\r\n4104.55,0.8336383,0.638887,0.344613,0.0946126\r\n4105.21,0.83357405,0.638753,0.344657,0.0946797\r\n4108.11,0.8335098,0.637564,0.344855,0.0949751\r\n4108.93,0.8334376,0.637231,0.34491,0.0950579\r\n4112.65,0.8333602,0.63609,0.345163,0.0954367\r\n4115.78,0.8332745,0.635452,0.345375,0.0957552\r\n4120.1,0.83318165,0.634569,0.345596,0.0964144\r\n4123.27,0.8330894,0.633269,0.345758,0.0968985\r\n4123.81,0.8329904,0.633048,0.345785,0.0970083\r\n4127.53,0.8328904,0.631526,0.345975,0.097763\r\n4130.73,0.8327953,0.631036,0.346139,0.0984148\r\n4130.77,0.83269065,0.63103,0.346141,0.0984217\r\n4134.98,0.832579,0.630386,0.346427,0.0988509\r\n4138.7,0.83246675,0.628864,0.34668,0.0992291\r\n4141.72,0.83235955,0.628402,0.346886,0.0995372\r\n4141.96,0.8322587,0.628366,0.346901,0.0995609\r\n4142,0.8321591,0.62836,0.346902,0.099565\r\n4146.15,0.8320646,0.627724,0.347009,0.100199\r\n4149.49,0.8319713,0.626359,0.347094,0.100708\r\n4149.87,0.83188955,0.626202,0.347104,0.100766\r\n4153.58,0.8318089,0.624681,0.347199,0.101332\r\n4156.91,0.83173315,0.623318,0.347284,0.10184\r\n4156.98,0.83165925,0.62329,0.347284,0.10185\r\n4157.3,0.83159385,0.623159,0.347284,0.101899\r\n4164.47,0.8315355,0.622062,0.347285,0.102993\r\n4164.75,0.83148695,0.622019,0.347285,0.103032\r\n4171.86,0.83144555,0.620569,0.347286,0.103995\r\n4172.2,0.83140535,0.620498,0.34731,0.104042\r\n4175.7,0.83137355,0.619425,0.347548,0.104517\r\n4175.92,0.83134905,0.619357,0.347563,0.104551\r\n4179.09,0.8313449,0.61871,0.347778,0.105034\r\n4183.08,0.83135085,0.617897,0.348049,0.105642\r\n4183.19,0.8313661,0.617873,0.348052,0.10566\r\n4183.38,0.8314071,0.617836,0.348057,0.105678\r\n4187.09,0.83148545,0.616315,0.348152,0.106056\r\n4194.42,0.8315644,0.614818,0.34834,0.106803\r\n4194.54,0.83166105,0.614794,0.348343,0.106821\r\n4198.03,0.8317589,0.613722,0.348432,0.107354\r\n4198.26,0.8318753,0.613653,0.348438,0.107388\r\n4201.91,0.831992,0.612157,0.348531,0.107945\r\n4201.98,0.83212555,0.612131,0.348533,0.107955\r\n4205.29,0.83229255,0.611454,0.348618,0.108461\r\n4209.4,0.83245975,0.610615,0.348723,0.109088\r\n4209.43,0.8326558,0.61061,0.348724,0.109091\r\n4212.99,0.832853,0.608046,0.348815,0.109453\r\n4213.12,0.83305025,0.607947,0.348822,0.109467\r\n4213.14,0.83330845,0.609089,0.348823,0.109469\r\n4220.63,0.8335942,0.607304,0.349205,0.110231\r\n4224.02,0.8338826,0.606496,0.349378,0.110749\r\n4224.31,0.8342042,0.606427,0.349393,0.110793\r\n4227.95,0.8345063,0.605312,0.349578,0.111347\r\n4228.03,0.8348087,0.605286,0.349581,0.11136\r\n4228.12,0.83514325,0.605249,0.349584,0.111374\r\n4231.75,0.8354929,0.603764,0.349708,0.111866\r\n4235.46,0.83583905,0.602243,0.349834,0.112369\r\n4239.16,0.8361864,0.601487,0.34996,0.112872\r\n4239.36,0.8365471,0.601448,0.349965,0.112898\r\n4242.76,0.83690835,0.600753,0.350052,0.113244\r\n4242.91,0.83729355,0.600722,0.350056,0.11326\r\n4246.63,0.8376808,0.599581,0.350151,0.113639\r\n4246.84,0.83809125,0.599495,0.350157,0.11366\r\n4250.34,0.83851065,0.598059,0.350247,0.114135\r\n4254.06,0.8389305,0.596538,0.350342,0.114639\r\n4254.12,0.83936295,0.596491,0.350343,0.114647\r\n4257.75,0.8397943,0.593494,0.350529,0.11514\r\n4257.76,0.8402257,0.593875,0.350529,0.115141\r\n4257.77,0.8405989,0.595016,0.35053,0.115143\r\n4258.07,0.84097555,0.594768,0.350545,0.115184\r\n4261.46,0.841387,0.591973,0.350718,0.115529\r\n4265.56,0.84179595,0.591347,0.350927,0.115946\r\n4268.92,0.84220555,0.590832,0.351099,0.116288\r\n4269.07,0.84264685,0.590769,0.351106,0.116304\r\n4272.64,0.843089,0.589311,0.351228,0.116667\r\n4276.35,0.84349845,0.587789,0.351354,0.117045\r\n4276.49,0.84387445,0.587731,0.351359,0.117059\r\n4276.79,0.84425285,0.58761,0.351369,0.117089\r\n4280.06,0.84460045,0.586268,0.351481,0.117534\r\n4280.29,0.84495075,0.586175,0.351489,0.117565\r\n4283.78,0.84526395,0.584746,0.351578,0.118038\r\n4288.02,0.8455789,0.584097,0.351687,0.118613\r\n4291.23,0.84585855,0.583606,0.351769,0.11894\r\n4294.95,0.84609615,0.582084,0.351864,0.119319\r\n4295.24,0.84633345,0.582039,0.351871,0.119349\r\n4295.51,0.84653805,0.581999,0.351878,0.119375\r\n4302.41,0.8467477,0.580944,0.352055,0.120078\r\n4306.4,0.8469541,0.580129,0.352157,0.120484\r\n4306.73,0.8471291,0.580061,0.352166,0.120519\r\n4309.86,0.84726995,0.579423,0.352245,0.120836\r\n4310.2,0.8473424,0.579283,0.352254,0.120871\r\n4313.57,0.84734375,0.577902,0.352483,0.121215\r\n4317.96,0.8473155,0.57723,0.352782,0.121662\r\n4321.03,0.8472511,0.576761,0.35299,0.121974\r\n4321.42,0.8471196,0.576681,0.353017,0.122014\r\n4325.03,0.8469472,0.575944,0.353109,0.122382\r\n4328.48,0.84676625,0.57524,0.353198,0.122733\r\n4329.19,0.84649855,0.575192,0.353216,0.122805\r\n4336.37,0.8461926,0.574705,0.3534,0.123537\r\n4339.68,0.84587935,0.574481,0.3534,0.123873\r\n4339.93,0.84555505,0.574455,0.3534,0.123899\r\n4340.42,0.84510655,0.574406,0.3534,0.123948\r\n4350.87,0.8446481,0.573341,0.353402,0.125013\r\n4351.32,0.8441856,0.57333,0.353402,0.125059\r\n4351.65,0.8436862,0.573321,0.353419,0.125092\r\n4354.83,0.84310455,0.573241,0.353581,0.125415\r\n4359.13,0.84240385,0.573132,0.353801,0.125854\r\n4365.81,0.84170345,0.572963,0.354142,0.126308\r\n4365.98,0.84102825,0.572957,0.35415,0.126319\r\n4366.28,0.8403094,0.572947,0.354165,0.12634\r\n4370.35,0.8395449,0.572809,0.354442,0.126616\r\n4377.02,0.8388152,0.572584,0.354895,0.127126\r\n4377.5,0.8380749,0.572551,0.354928,0.127163\r\n4380.88,0.8372872,0.572323,0.3551,0.127421\r\n4385.32,0.8364993,0.572021,0.355327,0.12776\r\n4388.22,0.8357023,0.571825,0.355475,0.128056\r\n4392.46,0.8349095,0.571393,0.355691,0.128488\r\n4392.8,0.83416515,0.571358,0.355715,0.128522\r\n4395.77,0.8334402,0.571056,0.355916,0.128674\r\n4399.41,0.83269635,0.570685,0.356164,0.12886\r\n4403.68,0.8319374,0.57025,0.356454,0.129078\r\n4407.76,0.83128405,0.569835,0.356662,0.129286\r\n4410.61,0.8305974,0.569545,0.356807,0.129479\r\n4418.16,0.82991555,0.569033,0.357193,0.129992\r\n4418.64,0.82927105,0.569,0.357217,0.130025\r\n4418.98,0.82870635,0.568977,0.357229,0.130049\r\n4421.81,0.8282725,0.568786,0.357325,0.13024\r\n4429.26,0.8278428,0.567265,0.357579,0.130747\r\n4429.86,0.82741735,0.567173,0.3576,0.130788\r\n4430.21,0.82705345,0.567119,0.357609,0.130811\r\n4436.71,0.82677685,0.566124,0.357775,0.131254\r\n4441.43,0.8265135,0.565644,0.357896,0.131574\r\n4444.81,0.8262959,0.5653,0.357982,0.131804\r\n4447.91,0.8261382,0.564984,0.357983,0.132014\r\n4451.81,0.82600955,0.564387,0.357984,0.13228\r\n4452.65,0.8259158,0.564259,0.357984,0.132337\r\n4455.36,0.8258036,0.563844,0.357984,0.132406\r\n4459.76,0.8256909,0.563396,0.357985,0.132519\r\n4466.56,0.82559145,0.562704,0.357986,0.132693\r\n4467.6,0.82549915,0.562598,0.357986,0.13272\r\n4470.47,0.8253671,0.562306,0.357986,0.132914\r\n4474.02,0.82523775,0.561944,0.357987,0.133156\r\n4474.71,0.8250767,0.561909,0.357987,0.133202\r\n4478.83,0.8248668,0.5617,0.358092,0.133482\r\n4481.62,0.8245437,0.561558,0.358164,0.133554\r\n4488.96,0.82420335,0.561186,0.358352,0.133742\r\n4489.66,0.8237982,0.561138,0.35837,0.13376\r\n4492.78,0.8233662,0.560926,0.35837,0.13384\r\n4493.78,0.8227699,0.560859,0.35837,0.133865\r\n4500.16,0.82215765,0.560426,0.358371,0.134082\r\n4500.2,0.82150325,0.560424,0.358371,0.134084\r\n4504.61,0.8208384,0.5602,0.358372,0.134234\r\n4505,0.8200945,0.56018,0.358382,0.134247\r\n4515.09,0.81931495,0.559667,0.35864,0.134762\r\n4518.85,0.8185756,0.559285,0.358737,0.134954\r\n4519.56,0.81786975,0.559212,0.358755,0.134991\r\n4519.96,0.8173167,0.559172,0.358755,0.135011\r\n4522.55,0.8168025,0.558908,0.358755,0.135011\r\n4522.64,0.8164839,0.55889,0.358755,0.135011\r\n4526.43,0.81635115,0.558116,0.358756,0.135012\r\n4526.48,0.8164555,0.558105,0.358756,0.135012\r\n4530.01,0.81658525,0.557387,0.358756,0.135012\r\n4530.28,0.8169755,0.557331,0.358757,0.135012\r\n4530.34,0.81745515,0.557319,0.358757,0.135012\r\n4531.17,0.8181958,0.55715,0.358757,0.135012\r\n4534.13,0.8191376,0.556545,0.358757,0.135088\r\n4534.19,0.82014355,0.556533,0.358757,0.13509\r\n4534.51,0.82132275,0.556467,0.358757,0.135098\r\n4537.46,0.8226917,0.555866,0.358858,0.135173\r\n4537.98,0.82425215,0.555705,0.358875,0.135187\r\n4538.04,0.8260015,0.555688,0.358877,0.135188\r\n4541.18,0.8277864,0.554725,0.358984,0.135269\r\n4541.83,0.8298336,0.554658,0.359007,0.135285\r\n4541.89,0.83209785,0.554652,0.359009,0.135287\r\n4541.94,0.8345159,0.554646,0.35901,0.135288\r\n4545.73,0.8369148,0.554261,0.359139,0.135385\r\n4545.74,0.8393736,0.55426,0.359139,0.135385\r\n4545.8,0.8418444,0.554254,0.359139,0.135387\r\n4546.12,0.8442989,0.554221,0.359139,0.135395\r\n4549.59,0.84677245,0.553868,0.35914,0.135484\r\n4549.65,0.84920605,0.553862,0.35914,0.135485\r\n4552.37,0.8516796,0.553585,0.35914,0.135555\r\n4553.44,0.8541531,0.553476,0.359141,0.135583\r\n4553.5,0.85677525,0.55347,0.359141,0.135584\r\n4557.29,0.85941855,0.553084,0.359141,0.135681\r\n4560.67,0.86189155,0.552739,0.359142,0.135768\r\n4561.07,0.86434885,0.552699,0.359142,0.135778\r\n4561.08,0.86684865,0.552698,0.359142,0.135778\r\n4561.14,0.86944375,0.552692,0.359142,0.135778\r\n4563.56,0.8719173,0.552445,0.359142,0.135778\r\n4564.93,0.8745986,0.552235,0.359142,0.135779\r\n4571.02,0.87733855,0.551305,0.359143,0.135779\r\n4572.55,0.8799644,0.550993,0.359143,0.13578\r\n4572.57,0.88247605,0.550989,0.359143,0.13578\r\n4572.59,0.88485935,0.550984,0.359143,0.13578\r\n4575.62,0.8872502,0.550365,0.359144,0.13578\r\n4576.02,0.8894574,0.550284,0.359144,0.13578\r\n4576.35,0.89172445,0.550217,0.359144,0.13578\r\n4576.37,0.8938936,0.550212,0.359144,0.13578\r\n4576.4,0.8959486,0.550207,0.359144,0.13578\r\n4576.42,0.89794305,0.550203,0.359144,0.13578\r\n4576.44,0.89996,0.550198,0.359144,0.13578\r\n4576.46,0.90182735,0.550193,0.359144,0.13578\r\n4578.47,0.90353985,0.549784,0.359144,0.135781\r\n4580.22,0.9051587,0.549516,0.359145,0.135781\r\n4580.25,0.90683365,0.549512,0.359145,0.135781\r\n4580.27,0.9085081,0.549509,0.359145,0.135781\r\n4584.01,0.9100684,0.548937,0.359145,0.135781\r\n4584.03,0.91139895,0.548934,0.359145,0.135781\r\n4585.93,0.9125597,0.548643,0.359145,0.135782\r\n4587.7,0.91343135,0.547916,0.359146,0.135782\r\n4587.73,0.9140923,0.547907,0.359146,0.135782\r\n4589.64,0.9146194,0.547122,0.359146,0.135782\r\n4590.57,0.9150456,0.546837,0.359146,0.135782\r\n4590.97,0.9153839,0.546715,0.359146,0.135782\r\n4591.39,0.9157902,0.546584,0.359146,0.135783\r\n4591.42,0.91602745,0.546577,0.359146,0.135783\r\n4593.36,0.91604665,0.545981,0.359147,0.135783\r\n4595.06,0.91606585,0.545721,0.359147,0.135783\r\n4595.09,0.916085,0.545717,0.359147,0.135783\r\n4595.11,0.91564755,0.545714,0.359147,0.135783\r\n4598.73,0.9152101,0.54516,0.359147,0.135784\r\n4598.76,0.9148081,0.545156,0.359147,0.135784\r\n4598.78,0.91402095,0.545153,0.359147,0.135784\r\n4600.82,0.9128986,0.54484,0.359148,0.135784\r\n4602.33,0.91177625,0.544686,0.359148,0.135784\r\n4602.36,0.91072995,0.544684,0.359148,0.135784\r\n4602.38,0.90968365,0.544682,0.359148,0.135784\r\n4602.4,0.90882165,0.544679,0.359148,0.135784\r\n4602.42,0.90813685,0.544677,0.359148,0.135784\r\n4602.45,0.9067099,0.544675,0.359148,0.135784\r\n4605.52,0.90531585,0.544362,0.359148,0.135785\r\n4605.92,0.90397085,0.544321,0.359149,0.135785\r\n4605.96,0.90272675,0.544317,0.359149,0.135785\r\n4605.98,0.90158535,0.544315,0.359149,0.135785\r\n4606,0.90044395,0.544313,0.359149,0.135785\r\n4606.02,0.89954105,0.54431,0.359149,0.135785\r\n4606.05,0.8980952,0.544308,0.359149,0.135785\r\n4609.58,0.89664935,0.543948,0.359149,0.135785\r\n4609.6,0.8952035,0.543946,0.359149,0.135785\r\n4609.62,0.89421425,0.543944,0.359149,0.135785\r\n4609.65,0.893225,0.543941,0.359149,0.135785\r\n4609.67,0.89170525,0.543939,0.359149,0.135785\r\n4612.01,0.8903047,0.5437,0.359149,0.135786\r\n4613.2,0.88923935,0.543579,0.35915,0.135786\r\n4613.22,0.888174,0.543577,0.35915,0.135786\r\n4613.25,0.88710865,0.543575,0.35915,0.135786\r\n4613.27,0.8860433,0.543572,0.35915,0.135786\r\n4613.29,0.88429305,0.54357,0.35915,0.135786\r\n4616.85,0.88254275,0.543208,0.35915,0.135786\r\n4616.87,0.8815346,0.543206,0.35915,0.135786\r\n4616.89,0.8806889,0.543204,0.35915,0.135786\r\n4616.92,0.87924405,0.543201,0.35915,0.135786\r\n4620.47,0.8777982,0.54284,0.359151,0.135787\r\n4620.49,0.8763523,0.542837,0.359151,0.135787\r\n4620.52,0.8749064,0.542835,0.359151,0.135787\r\n4620.54,0.87346055,0.542832,0.359151,0.135787\r\n4620.56,0.8726411,0.54283,0.359151,0.135787\r\n4620.86,0.8712912,0.542799,0.359151,0.135787\r\n4623.21,0.8696931,0.542561,0.359151,0.135787\r\n4624.09,0.86809505,0.542425,0.359151,0.135788\r\n4624.12,0.866497,0.542421,0.359151,0.135788\r\n4624.14,0.86542945,0.542418,0.359151,0.135788\r\n4624.16,0.8646685,0.542414,0.359151,0.135788\r\n4624.19,0.86322265,0.542411,0.359151,0.135788\r\n4627.74,0.8617768,0.541867,0.359152,0.135788\r\n4627.76,0.86033095,0.541864,0.359152,0.135788\r\n4627.79,0.8588851,0.54186,0.359152,0.135788\r\n4627.81,0.85755905,0.541857,0.359152,0.135788\r\n4630.66,0.8560371,0.54142,0.359152,0.135789\r\n4631.36,0.85451515,0.541349,0.359152,0.135789\r\n4631.39,0.8529932,0.541347,0.359152,0.135789\r\n4631.41,0.85215515,0.541344,0.359152,0.135789\r\n4631.43,0.8513181,0.541342,0.359152,0.135789\r\n4631.45,0.84979615,0.54134,0.359152,0.135789\r\n4635.01,0.8482742,0.540978,0.359153,0.135789\r\n4635.03,0.8467522,0.540975,0.359153,0.135789\r\n4635.06,0.8453648,0.540973,0.359153,0.135789\r\n4635.08,0.8443743,0.540971,0.359153,0.135789\r\n4635.41,0.8434877,0.540937,0.359153,0.135789\r\n4635.81,0.8420418,0.540896,0.359153,0.135789\r\n4638.63,0.84059595,0.540609,0.359154,0.13579\r\n4638.66,0.8391501,0.540606,0.359154,0.13579\r\n4638.68,0.83770425,0.540604,0.359154,0.13579\r\n4638.7,0.8369433,0.540602,0.359154,0.13579\r\n4638.72,0.8355695,0.540599,0.359154,0.13579\r\n4641.86,0.83404755,0.54028,0.359154,0.13579\r\n4642.28,0.8325256,0.540108,0.359154,0.13579\r\n4642.3,0.83156875,0.540099,0.359154,0.13579\r\n4642.32,0.8308078,0.540089,0.359154,0.13579\r\n4642.35,0.82941845,0.54008,0.359154,0.13579\r\n4645.57,0.8278965,0.538759,0.359155,0.135791\r\n4645.9,0.82637455,0.538725,0.359155,0.135791\r\n4645.92,0.8248526,0.538723,0.359155,0.135791\r\n4645.95,0.82401555,0.538721,0.359155,0.135791\r\n4645.97,0.8231785,0.538718,0.359155,0.135791\r\n4645.99,0.82165655,0.538716,0.359155,0.135791\r\n4649.55,0.8201346,0.538354,0.359155,0.135791\r\n4649.57,0.8187462,0.538352,0.359155,0.135791\r\n4649.59,0.8175021,0.538349,0.359155,0.135791\r\n4649.62,0.8166467,0.538347,0.359155,0.135791\r\n4650.36,0.81566465,0.538271,0.359155,0.135792\r\n4650.76,0.8142949,0.538231,0.359145,0.135792\r\n4653.22,0.81292515,0.53798,0.359083,0.135709\r\n4653.24,0.8115554,0.537978,0.359082,0.135708\r\n4653.26,0.81062345,0.537976,0.359082,0.135707\r\n4656.77,0.8096587,0.537619,0.358993,0.135589\r\n4660.48,0.8086124,0.536097,0.358899,0.135463\r\n4660.67,0.80749985,0.536068,0.358894,0.135456\r\n4661.96,0.80633665,0.53587,0.358862,0.135413\r\n4665.3,0.80574585,0.53536,0.358777,0.135413\r\n4667.94,0.8052133,0.534957,0.358643,0.135414\r\n4668.16,0.80467805,0.534911,0.358632,0.135414\r\n4675.39,0.80406945,0.533436,0.358265,0.135415\r\n4675.66,0.8034101,0.533352,0.358251,0.135415\r\n4676.91,0.8027191,0.532969,0.358187,0.135415\r\n4679.11,0.80264325,0.532295,0.358076,0.135415\r\n4679.42,0.80249755,0.532247,0.35806,0.135415\r\n4680.24,0.80232395,0.532122,0.358018,0.135416\r\n4686.56,0.8020769,0.531155,0.357804,0.135417\r\n4686.91,0.8019925,0.531013,0.357793,0.135417\r\n4690.28,0.80196655,0.529633,0.357679,0.135417\r\n4690.67,0.80226005,0.529553,0.357666,0.135417\r\n4691.44,0.8024817,0.529395,0.357639,0.135417\r\n4691.86,0.80268685,0.52931,0.357618,0.135417\r\n4697.73,0.8029954,0.528112,0.35732,0.135418\r\n4698.17,0.80346595,0.528068,0.357298,0.135418\r\n4705.66,0.80394745,0.527305,0.356917,0.135419\r\n4706.38,0.80442235,0.527231,0.356881,0.13542\r\n4706.81,0.80488795,0.527188,0.35687,0.13542\r\n4708.92,0.80536175,0.526972,0.356816,0.135366\r\n4712.64,0.80583775,0.525451,0.356722,0.135272\r\n4713.15,0.80625805,0.525373,0.356709,0.135259\r\n4720.09,0.80667875,0.524311,0.356534,0.135083\r\n4721.32,0.80707515,0.52406,0.356502,0.135052\r\n4721.75,0.8074419,0.523973,0.356481,0.135041\r\n4727.55,0.80780375,0.52279,0.356186,0.134895\r\n4728.11,0.80817695,0.522703,0.356158,0.13488\r\n4735,0.80850515,0.521649,0.355808,0.134706\r\n4736.26,0.8088318,0.521457,0.355744,0.134674\r\n4736.69,0.80910495,0.521391,0.355729,0.134663\r\n4742.46,0.8093717,0.520509,0.355534,0.134664\r\n4743.07,0.809653,0.520467,0.355514,0.134664\r\n4747.46,0.80993225,0.520169,0.355365,0.134665\r\n4747.9,0.8101654,0.52014,0.355365,0.134665\r\n4750.55,0.81040205,0.51996,0.355365,0.134598\r\n4753.66,0.8106039,0.519749,0.355366,0.134519\r\n4762.41,0.81080135,0.518858,0.355367,0.134298\r\n4762.84,0.81100065,0.518814,0.355378,0.134287\r\n4764.85,0.8111855,0.51861,0.35543,0.134236\r\n4765.51,0.81137785,0.518565,0.355447,0.134219\r\n4776.05,0.81156855,0.51785,0.355716,0.133952\r\n4776.73,0.8117448,0.517781,0.355734,0.133935\r\n4777.37,0.81192005,0.517716,0.35575,0.133919\r\n4777.79,0.8121424,0.517674,0.355761,0.133908\r\n4787.25,0.8123536,0.51671,0.356003,0.13391\r\n4787.96,0.8125855,0.516662,0.356021,0.13391\r\n4792.32,0.81280195,0.516366,0.356133,0.133911\r\n4792.73,0.813029,0.516338,0.356143,0.133911\r\n4795.45,0.8132702,0.516154,0.356213,0.133911\r\n4798.44,0.8135247,0.515951,0.35629,0.133911\r\n4803.94,0.8137876,0.51511,0.35643,0.133912\r\n4805.9,0.81403205,0.514811,0.35648,0.133863\r\n4806.68,0.81427835,0.514653,0.3565,0.133843\r\n4807.27,0.8145524,0.514531,0.356516,0.133828\r\n4813.35,0.8148494,0.51329,0.356723,0.133674\r\n4817.91,0.815127,0.512981,0.356878,0.133559\r\n4818.49,0.81540555,0.512941,0.356898,0.133544\r\n4818.89,0.8157078,0.512914,0.356908,0.133534\r\n4824.55,0.81601265,0.51253,0.357053,0.133391\r\n4825.39,0.8163321,0.512359,0.357075,0.133369\r\n4832,0.816656,0.511009,0.357244,0.133202\r\n4832.88,0.8169804,0.51095,0.357266,0.13318\r\n4833.44,0.8173053,0.510912,0.357281,0.133166\r\n4833.83,0.8176456,0.510886,0.357307,0.133156\r\n4843.2,0.8179885,0.51025,0.357944,0.132839\r\n4844.12,0.8183134,0.51011,0.358006,0.132808\r\n4844.66,0.81863895,0.510027,0.358043,0.13279\r\n4845.03,0.8189936,0.50997,0.358053,0.132777\r\n4850.66,0.81934025,0.50911,0.358197,0.132635\r\n4851.61,0.81973505,0.508964,0.358221,0.132611\r\n4855.37,0.82014085,0.508389,0.358317,0.132515\r\n4858.12,0.82055415,0.507969,0.358388,0.132446\r\n4859.62,0.82096615,0.508071,0.358426,0.132408\r\n4859.98,0.8213589,0.508096,0.358444,0.132399\r\n4862.86,0.82178655,0.508292,0.358592,0.132399\r\n4869.34,0.8222216,0.508732,0.358922,0.1324\r\n4870.36,0.8226868,0.508801,0.358974,0.1324\r\n4874.58,0.82312585,0.509088,0.359189,0.132401\r\n4874.92,0.82358285,0.509112,0.359213,0.132401\r\n4877.85,0.82401705,0.50931,0.359412,0.132302\r\n4880.56,0.8244931,0.509495,0.359596,0.13221\r\n4885.34,0.82496885,0.510467,0.359921,0.132049\r\n4885.8,0.82544445,0.510559,0.359952,0.132033\r\n4886.13,0.82587125,0.510626,0.359961,0.132022\r\n4888.06,0.8263282,0.511018,0.36001,0.131924\r\n4892.83,0.82683595,0.511747,0.360132,0.131682\r\n4895.55,0.8273621,0.512161,0.360202,0.131544\r\n4896.59,0.8278876,0.51232,0.360229,0.131491\r\n4900.75,0.828409,0.512955,0.360335,0.131279\r\n4901.07,0.8288743,0.513002,0.360356,0.131263\r\n4903.04,0.82932935,0.513303,0.36049,0.131214\r\n4904.09,0.82982205,0.513464,0.360562,0.131187\r\n4910.53,0.83032005,0.514446,0.361,0.131024\r\n4911.58,0.830804,0.514589,0.361071,0.130997\r\n4911.97,0.83129035,0.514642,0.361098,0.130987\r\n4915.34,0.8317731,0.515099,0.36127,0.130902\r\n4916.01,0.8322821,0.515189,0.361303,0.130885\r\n4921.76,0.8327994,0.51597,0.361597,0.130886\r\n4922.84,0.8333352,0.516297,0.361652,0.130886\r\n4925.52,0.83385975,0.517112,0.361789,0.130887\r\n4926.59,0.83433915,0.517547,0.361844,0.130887\r\n4926.93,0.834818,0.517686,0.361861,0.130887\r\n4927.22,0.83531525,0.517801,0.361904,0.130887\r\n4929.28,0.83584665,0.518635,0.362218,0.130835\r\n4934.09,0.83634845,0.519612,0.362953,0.130713\r\n4934.42,0.83684285,0.519681,0.363004,0.130705\r\n4936.77,0.83733785,0.520158,0.363163,0.130645\r\n4937.85,0.83781105,0.520593,0.363236,0.130618\r\n4940.53,0.8382935,0.52168,0.363419,0.13055\r\n4942.16,0.83877215,0.522175,0.363529,0.130509\r\n4944.29,0.8392481,0.522823,0.363674,0.130437\r\n4945.34,0.8396781,0.523036,0.363746,0.130401\r\n4945.65,0.8401624,0.523099,0.363766,0.130391\r\n4951.78,0.84064945,0.524346,0.364391,0.130183\r\n4952.84,0.8410723,0.524772,0.364499,0.130148\r\n4953.37,0.84150505,0.524986,0.364552,0.13013\r\n4955.54,0.8418895,0.525868,0.364774,0.130075\r\n4956.87,0.8422815,0.526407,0.36491,0.130041\r\n4959.3,0.84264055,0.527391,0.365403,0.12998\r\n4960.33,0.84303855,0.527703,0.365611,0.129954\r\n4963.06,0.84345395,0.528533,0.366166,0.129885\r\n4964.09,0.8438722,0.52895,0.366375,0.129858\r\n4964.37,0.8443191,0.529065,0.366433,0.129851\r\n4966.82,0.84473795,0.530056,0.366682,0.129789\r\n4967.85,0.84515465,0.530473,0.366787,0.129763\r\n4968.31,0.84554685,0.530659,0.366834,0.129752\r\n4970.58,0.8459175,0.531578,0.367065,0.129694\r\n4970.6,0.84626795,0.533101,0.367067,0.129694\r\n4970.62,0.8466442,0.534623,0.367069,0.129693\r\n4974.38,0.8470044,0.536145,0.367452,0.129598\r\n4974.4,0.84735685,0.537667,0.367455,0.129597\r\n4974.43,0.84770635,0.539189,0.367457,0.129597\r\n4974.45,0.8480026,0.540711,0.367459,0.129596\r\n4975.34,0.84829205,0.541073,0.36755,0.129574\r\n4975.6,0.8486172,0.541177,0.367576,0.129567\r\n4978.21,0.84892605,0.542234,0.367974,0.129501\r\n4978.23,0.8492379,0.543756,0.367978,0.1295\r\n4979.1,0.8495718,0.543932,0.36811,0.129479\r\n4983.09,0.84989955,0.544743,0.368719,0.129377\r\n4983.25,0.8501972,0.544776,0.368743,0.129373\r\n4985.73,0.8504742,0.545279,0.369121,0.12929\r\n4985.75,0.850754,0.546801,0.369125,0.129289\r\n4986.59,0.85104335,0.547142,0.369253,0.12926\r\n4989.51,0.8513289,0.548324,0.369698,0.129162\r\n4990.35,0.85161155,0.548665,0.369827,0.129133\r\n4990.58,0.85189165,0.548757,0.369862,0.129126\r\n4993.27,0.8521805,0.549847,0.370408,0.129035\r\n4994.45,0.85248875,0.550326,0.370649,0.128995\r\n4997.03,0.85276505,0.551369,0.371172,0.128864\r\n4997.84,0.8530435,0.551617,0.371337,0.128822\r\n4998.08,0.8533492,0.551688,0.371385,0.128811\r\n5000.78,0.85365495,0.552512,0.371798,0.128673\r\n5000.81,0.8539898,0.554034,0.371801,0.128672\r\n5004.57,0.85432095,0.555556,0.372375,0.128481\r\n5004.59,0.85461265,0.557078,0.372378,0.12848\r\n5004.61,0.8549114,0.5586,0.372382,0.128479\r\n5005.34,0.8552016,0.558895,0.372493,0.128442\r\n5005.57,0.85550805,0.558987,0.372527,0.12843\r\n5008.37,0.8558133,0.560123,0.372908,0.128288\r\n5008.4,0.8560938,0.561645,0.372911,0.128286\r\n5008.42,0.8563879,0.563167,0.372914,0.128285\r\n5009.1,0.8566757,0.563444,0.373007,0.128251\r\n5009.39,0.85692565,0.563561,0.373046,0.128236\r\n5012.18,0.8571587,0.56469,0.373424,0.128142\r\n5012.2,0.8573902,0.566212,0.373427,0.128141\r\n5012.22,0.85762985,0.567734,0.37343,0.12814\r\n5015.98,0.8578607,0.569257,0.37394,0.128013\r\n5016.01,0.85807205,0.570779,0.373943,0.128012\r\n5016.03,0.85828145,0.572301,0.373946,0.128011\r\n5016.59,0.85849275,0.572529,0.374023,0.127992\r\n5016.8,0.85873705,0.572614,0.374051,0.127985\r\n5019.79,0.85898155,0.573823,0.374507,0.127884\r\n5019.81,0.8591883,0.575345,0.37451,0.127884\r\n5019.83,0.85940535,0.576868,0.374514,0.127883\r\n5020.35,0.8596271,0.577077,0.374593,0.127865\r\n5020.6,0.85990215,0.577176,0.37463,0.127857\r\n5023.59,0.86017305,0.57839,0.375087,0.127857\r\n5023.62,0.86039715,0.579912,0.375091,0.127857\r\n5024.11,0.8606222,0.580113,0.375166,0.127858\r\n5024.29,0.8608703,0.580186,0.375194,0.127858\r\n5027.38,0.8611048,0.581435,0.375508,0.127858\r\n5027.4,0.8613657,0.582957,0.37551,0.127858\r\n5031.16,0.86160555,0.58448,0.375893,0.127859\r\n5031.18,0.86184845,0.586002,0.375895,0.127859\r\n5031.6,0.86212505,0.586172,0.375938,0.127859\r\n5034.94,0.86237325,0.587524,0.376278,0.127859\r\n5034.96,0.8626215,0.589046,0.37628,0.127859\r\n5034.99,0.862875,0.590568,0.376283,0.127859\r\n5035.52,0.86312445,0.590785,0.376337,0.127859\r\n5035.54,0.8634022,0.590794,0.376341,0.127859\r\n5038.75,0.86361965,0.592091,0.376829,0.127642\r\n5038.77,0.86384,0.593613,0.376833,0.127641\r\n5039.1,0.86408605,0.593747,0.376883,0.127618\r\n5042.53,0.8643253,0.595136,0.377406,0.127386\r\n5043.01,0.86458455,0.595283,0.37748,0.127353\r\n5046.28,0.86478545,0.596278,0.377813,0.127131\r\n5046.59,0.8649867,0.596402,0.377844,0.12711\r\n5046.74,0.86519475,0.596465,0.37786,0.1271\r\n5050.04,0.8654015,0.597801,0.378196,0.127016\r\n5050.06,0.8656039,0.599323,0.378198,0.127016\r\n5053.82,0.8658082,0.600845,0.378581,0.126921\r\n5054.24,0.86600085,0.601013,0.378623,0.12691\r\n5057.58,0.8661945,0.602368,0.378964,0.126826\r\n5057.82,0.8664029,0.602462,0.378987,0.12682\r\n5061.34,0.86657755,0.603891,0.379347,0.12673\r\n5061.37,0.86675355,0.605413,0.379349,0.12673\r\n5061.68,0.8669468,0.605542,0.379381,0.126722\r\n5065.13,0.8671348,0.606935,0.379732,0.126605\r\n5065.15,0.86732265,0.608457,0.379734,0.126605\r\n5065.17,0.86747805,0.609979,0.379736,0.126604\r\n5065.19,0.8676334,0.611502,0.379739,0.126603\r\n5065.22,0.86778665,0.613024,0.379741,0.126602\r\n5065.47,0.86793165,0.613125,0.379766,0.126594\r\n5068.98,0.86807305,0.614546,0.380005,0.126475\r\n5069,0.8681896,0.616068,0.380007,0.126474\r\n5069.02,0.8683039,0.61759,0.380008,0.126474\r\n5069.05,0.868427,0.619112,0.38001,0.126473\r\n5072.8,0.86853355,0.620635,0.380265,0.126346\r\n5072.89,0.86864925,0.62067,0.380271,0.126343\r\n5076.52,0.868745,0.622142,0.380518,0.126251\r\n5076.56,0.8688361,0.622158,0.380521,0.12625\r\n5076.69,0.8688375,0.622208,0.380529,0.126247\r\n5080.24,0.86883705,0.623646,0.38077,0.126157\r\n5080.32,0.8688185,0.62368,0.380776,0.126155\r\n5080.34,0.86877105,0.624822,0.380777,0.126154\r\n5084.1,0.8686945,0.626345,0.381033,0.126059\r\n5087.7,0.86859765,0.6278,0.381277,0.125968\r\n5087.83,0.8685001,0.627856,0.381287,0.125965\r\n5087.86,0.8684014,0.627867,0.381288,0.125964\r\n5087.91,0.86823095,0.627888,0.381292,0.125962\r\n5091.41,0.86805605,0.629305,0.381826,0.125844\r\n5091.62,0.8678081,0.62939,0.381858,0.125837\r\n5095.12,0.867537,0.630809,0.382392,0.125718\r\n5095.38,0.86726535,0.630913,0.382431,0.125709\r\n5095.4,0.86692325,0.630922,0.382435,0.125709\r\n5098.84,0.8665795,0.632313,0.382697,0.125593\r\n5099.04,0.8662254,0.632394,0.382712,0.125586\r\n5099.14,0.8658423,0.632435,0.38272,0.125581\r\n5102.9,0.86542395,0.633958,0.383008,0.12539\r\n5106.29,0.86499905,0.634991,0.383267,0.125217\r\n5106.65,0.864508,0.6351,0.383295,0.125199\r\n5110.01,0.8640823,0.636459,0.383551,0.125028\r\n5110.37,0.8636563,0.636604,0.383578,0.12501\r\n5110.41,0.86316265,0.636623,0.383583,0.125008\r\n5113.72,0.86269265,0.637963,0.38392,0.12484\r\n5113.97,0.86224605,0.638065,0.383946,0.124827\r\n5114.17,0.8617354,0.638145,0.383966,0.12482\r\n5117.44,0.8612168,0.639467,0.384298,0.12471\r\n5117.85,0.8606976,0.639636,0.384341,0.124696\r\n5117.93,0.8601841,0.639668,0.384357,0.124693\r\n5121.15,0.85966385,0.640972,0.385011,0.124584\r\n5121.69,0.8591504,0.641191,0.385121,0.124566\r\n5124.87,0.8586358,0.642476,0.385766,0.124459\r\n5125.18,0.85811815,0.642603,0.385829,0.124448\r\n5125.35,0.85766875,0.642672,0.385864,0.124444\r\n5125.45,0.8571568,0.642713,0.385879,0.124441\r\n5128.58,0.8566196,0.64398,0.386357,0.124362\r\n5129.21,0.855985,0.644236,0.386453,0.124346\r\n5132.27,0.8554525,0.645167,0.38692,0.124269\r\n5132.29,0.85490765,0.645174,0.386923,0.124268\r\n5132.84,0.854427,0.64534,0.387007,0.124254\r\n5132.97,0.85384475,0.645378,0.387019,0.124251\r\n5135.99,0.8532332,0.646603,0.387327,0.124175\r\n5136.72,0.8525675,0.646901,0.387402,0.124156\r\n5139.68,0.851983,0.647502,0.387703,0.124081\r\n5139.71,0.8513855,0.647506,0.387706,0.124081\r\n5140.12,0.8507198,0.647591,0.387748,0.12407\r\n5143.4,0.8500349,0.648256,0.388081,0.123959\r\n5144.07,0.84934535,0.648393,0.38815,0.123937\r\n5144.22,0.84860355,0.648424,0.388165,0.123931\r\n5147.09,0.8479489,0.649585,0.388457,0.123834\r\n5147.11,0.8473234,0.649594,0.38846,0.123834\r\n5147.98,0.846588,0.649946,0.388548,0.123804\r\n5150.8,0.84584535,0.650377,0.388836,0.123709\r\n5151.33,0.84503945,0.650457,0.388889,0.123691\r\n5154.52,0.8442897,0.650943,0.389214,0.12361\r\n5155.3,0.84356415,0.651062,0.389293,0.123591\r\n5155.47,0.8429264,0.651089,0.38932,0.123586\r\n5159.23,0.8421846,0.652612,0.389893,0.123491\r\n5161.97,0.84144575,0.65372,0.390311,0.123422\r\n5162.79,0.8407072,0.654051,0.390436,0.123401\r\n5162.99,0.8400225,0.654134,0.390477,0.123396\r\n5165.68,0.83935905,0.655224,0.391024,0.123328\r\n5166.27,0.83881015,0.655461,0.391143,0.123313\r\n5166.75,0.8381445,0.655657,0.391241,0.12328\r\n5169.4,0.83747785,0.656463,0.39178,0.1231\r\n5170.28,0.83694175,0.656732,0.391959,0.123041\r\n5170.51,0.83637985,0.656799,0.391982,0.123026\r\n5173.12,0.8357482,0.65733,0.392247,0.122849\r\n5176.83,0.83522825,0.658084,0.392626,0.122597\r\n5177.47,0.8346267,0.658214,0.39269,0.122554\r\n5178,0.8340252,0.658322,0.392745,0.12254\r\n5181.51,0.83351175,0.659035,0.393102,0.122451\r\n5184.28,0.83299035,0.659598,0.393525,0.122381\r\n5185.5,0.83249595,0.659846,0.39371,0.12235\r\n5188,0.8320027,0.660227,0.394092,0.122287\r\n5189,0.8314881,0.66038,0.394245,0.122262\r\n5192.41,0.8310079,0.6609,0.394592,0.122175\r\n5192.99,0.8305135,0.660988,0.394651,0.122156\r\n5199.19,0.8300279,0.661934,0.395282,0.121946\r\n5200.23,0.8295446,0.662093,0.395388,0.121911\r\n5200.48,0.82909245,0.662131,0.395405,0.121903\r\n5203.61,0.8286261,0.662609,0.395618,0.121797\r\n5206.65,0.82816265,0.663072,0.395824,0.121643\r\n5207.97,0.82772805,0.663274,0.395914,0.121575\r\n5211.45,0.82729795,0.663805,0.396151,0.121398\r\n5215.46,0.8268643,0.664416,0.396423,0.121195\r\n5217.85,0.82648325,0.664659,0.396585,0.121074\r\n5218.55,0.826173,0.664731,0.396633,0.121038\r\n5222.67,0.8258642,0.665151,0.396914,0.120898\r\n5226.69,0.825557,0.66556,0.397323,0.120763\r\n5229.76,0.82528175,0.66556,0.397635,0.120659\r\n5232.79,0.82503475,0.665561,0.397943,0.120582\r\n5233.9,0.8248065,0.665561,0.398057,0.120554\r\n5237.9,0.82461665,0.665561,0.398159,0.120453\r\n5244.7,0.82443365,0.664638,0.398333,0.120281\r\n5247.74,0.82427235,0.664225,0.398411,0.120178\r\n5248.86,0.8241147,0.664073,0.39844,0.12014\r\n5249.09,0.8239906,0.664041,0.398456,0.120132\r\n5255.9,0.8238736,0.663347,0.398919,0.119902\r\n5256.55,0.82375405,0.663281,0.398963,0.119858\r\n5260.08,0.82365515,0.662922,0.399202,0.119619\r\n5262.68,0.82357385,0.662657,0.399335,0.119443\r\n5267.1,0.82349625,0.662207,0.399561,0.119143\r\n5267.75,0.8234212,0.662141,0.399594,0.119126\r\n5275.04,0.82335955,0.661026,0.399966,0.118942\r\n5275.2,0.8233029,0.661001,0.39997,0.118938\r\n5277.62,0.82323965,0.660507,0.400032,0.118876\r\n5282.04,0.82318065,0.659604,0.400145,0.118764\r\n5282.65,0.82311625,0.65948,0.400161,0.118723\r\n5288.82,0.82305575,0.658536,0.400319,0.118305\r\n5289.99,0.8229991,0.658358,0.400349,0.118226\r\n5290.11,0.8229425,0.658339,0.400352,0.118218\r\n5293.24,0.8228859,0.65786,0.400432,0.118005\r\n5297.57,0.8228293,0.657199,0.400542,0.117896\r\n5303.77,0.82277265,0.655932,0.400701,0.117739\r\n5304.94,0.82271745,0.655693,0.400731,0.117709\r\n5305.02,0.82266255,0.655678,0.400736,0.117707\r\n5308.19,0.8226163,0.655194,0.400952,0.117627\r\n5312.47,0.8225709,0.654538,0.401243,0.117482\r\n5316.17,0.82253,0.653973,0.401494,0.117357\r\n5318.72,0.8224924,0.653583,0.401559,0.117271\r\n5319.39,0.8224604,0.65348,0.401577,0.117248\r\n5319.93,0.82242925,0.653397,0.40159,0.117221\r\n5323.65,0.8224074,0.651876,0.401686,0.117032\r\n5331.1,0.82238575,0.650355,0.401876,0.116653\r\n5331.12,0.82236715,0.650351,0.401877,0.116652\r\n5333.67,0.8223569,0.649962,0.401942,0.116523\r\n5334.33,0.82235305,0.649861,0.401959,0.116489\r\n5338.55,0.8223686,0.649214,0.402067,0.116346\r\n5345.53,0.82238475,0.648147,0.402246,0.11611\r\n5346.01,0.822401,0.648074,0.402258,0.116086\r\n5346.07,0.82242045,0.648048,0.40226,0.116083\r\n5348.62,0.82243845,0.647004,0.402433,0.115954\r\n5349.72,0.8224517,0.646552,0.402508,0.115897\r\n5353.44,0.82246025,0.645031,0.40276,0.115709\r\n5357.15,0.8224686,0.643509,0.403013,0.11552\r\n5357.29,0.82247295,0.64348,0.403022,0.115513\r\n5360.47,0.82247335,0.642833,0.403104,0.115352\r\n5363.56,0.82247375,0.642201,0.403183,0.115247\r\n5364.6,0.8224741,0.641989,0.40321,0.115212\r\n5368.32,0.8224745,0.640848,0.403305,0.115086\r\n5371.67,0.8224749,0.640164,0.403391,0.114973\r\n5372.25,0.8224753,0.640046,0.403405,0.114953\r\n5375.77,0.82247565,0.639327,0.403585,0.114834\r\n5378.51,0.82247265,0.638487,0.403725,0.114742\r\n5379.49,0.8224582,0.638186,0.403775,0.114708\r\n5382.88,0.8224291,0.637495,0.403948,0.114594\r\n5386.95,0.8223937,0.636665,0.404155,0.114491\r\n5387.21,0.8223409,0.636625,0.404169,0.114484\r\n5389.71,0.82227555,0.636242,0.404339,0.114421\r\n5394.4,0.82220145,0.635524,0.404658,0.114302\r\n5397.82,0.82212335,0.634124,0.40489,0.114216\r\n5398.12,0.82204585,0.634003,0.40491,0.114186\r\n5398.43,0.82195725,0.633939,0.404931,0.114154\r\n5404.65,0.821871,0.63267,0.40509,0.113521\r\n5405.57,0.8217762,0.632482,0.405114,0.113427\r\n5409.01,0.8216848,0.62964,0.405202,0.113076\r\n5409.26,0.8215972,0.629438,0.405208,0.11307\r\n5409.28,0.82150025,0.63096,0.405209,0.113069\r\n5412.97,0.82140225,0.627917,0.405304,0.112976\r\n5413.38,0.82128845,0.627854,0.405314,0.112965\r\n5419.58,0.82117465,0.626906,0.405473,0.112808\r\n5420.43,0.82106085,0.626776,0.405494,0.112787\r\n5423.96,0.82094705,0.625332,0.405585,0.112698\r\n5424.14,0.8208366,0.625255,0.40559,0.112691\r\n5427.86,0.8207376,0.623733,0.405685,0.112566\r\n5428.34,0.82065245,0.623562,0.405697,0.11255\r\n5434.53,0.82057145,0.621344,0.406118,0.11234\r\n5435.16,0.82049955,0.621118,0.406161,0.112319\r\n5435.29,0.8204397,0.621071,0.40617,0.11231\r\n5435.32,0.8204073,0.622593,0.406171,0.112308\r\n5439.01,0.82037845,0.619549,0.406422,0.112058\r\n5439.56,0.820382,0.619437,0.40646,0.112021\r\n5445.76,0.8204044,0.618171,0.406618,0.1116\r\n5446.36,0.82042965,0.618048,0.406634,0.11156\r\n5446.46,0.8204982,0.618028,0.406636,0.111557\r\n5453.24,0.82057075,0.616991,0.40681,0.111385\r\n5453.91,0.82064645,0.616888,0.406827,0.111368\r\n5454.51,0.82074735,0.616643,0.406842,0.111353\r\n5457.63,0.820868,0.615366,0.406922,0.111274\r\n5461.3,0.82100465,0.613861,0.407016,0.111181\r\n5461.34,0.8211572,0.613845,0.407017,0.11118\r\n5464.47,0.82131415,0.612563,0.407097,0.111074\r\n5465.06,0.8214993,0.612323,0.407112,0.111054\r\n5468.75,0.8216846,0.60928,0.407207,0.11093\r\n5468.77,0.8218752,0.610802,0.407207,0.110929\r\n5469.47,0.8220848,0.61023,0.407225,0.110905\r\n5471.96,0.8222962,0.608171,0.407289,0.110821\r\n5472.46,0.82250745,0.607758,0.407302,0.110804\r\n5472.51,0.8227892,0.607751,0.407303,0.110803\r\n5479.46,0.82305565,0.606688,0.407481,0.110449\r\n5479.92,0.82334765,0.606618,0.407493,0.110426\r\n5483.63,0.82361405,0.605096,0.407588,0.110237\r\n5484.42,0.8238967,0.604775,0.407608,0.110197\r\n5486.95,0.8241829,0.603738,0.407673,0.110069\r\n5487.35,0.8244355,0.603575,0.407683,0.110049\r\n5487.44,0.82472145,0.603535,0.407685,0.110044\r\n5491.06,0.82503875,0.602053,0.407778,0.109921\r\n5494.45,0.8253453,0.601015,0.407865,0.109807\r\n5494.78,0.82568515,0.600912,0.407873,0.109796\r\n5498.2,0.82602775,0.599512,0.407961,0.10968\r\n5498.5,0.826356,0.599391,0.407968,0.10967\r\n5498.65,0.82668715,0.59936,0.407972,0.109665\r\n5499.37,0.82705435,0.599212,0.407991,0.109628\r\n5505.7,0.8274265,0.597921,0.407992,0.109307\r\n5505.95,0.8278644,0.59787,0.407992,0.109294\r\n5509.46,0.8282848,0.596432,0.407992,0.109116\r\n5509.66,0.82872855,0.596349,0.407992,0.109106\r\n5513.38,0.82917335,0.595208,0.407993,0.108917\r\n5513.59,0.82955315,0.595166,0.407993,0.108906\r\n5514.32,0.8299495,0.595016,0.407993,0.108881\r\n5516.95,0.83035695,0.59448,0.407815,0.108793\r\n5520.83,0.8307951,0.593687,0.407551,0.108661\r\n5524.44,0.8312166,0.593134,0.407307,0.108539\r\n5524.79,0.8316396,0.593081,0.407283,0.108527\r\n5525.52,0.83208275,0.59297,0.407234,0.108503\r\n5528.29,0.83251695,0.592546,0.407164,0.108409\r\n5531.94,0.83291745,0.591053,0.407071,0.108286\r\n5532,0.8333506,0.591025,0.40707,0.108284\r\n5535.72,0.8337346,0.589503,0.406976,0.108158\r\n5536,0.8341505,0.589461,0.406969,0.108149\r\n5539.43,0.8345727,0.588935,0.406882,0.107916\r\n5540.46,0.83500825,0.588778,0.406855,0.107846\r\n5543.17,0.83540785,0.588363,0.406787,0.107662\r\n5546.89,0.8358026,0.586841,0.406693,0.10741\r\n5546.92,0.8361284,0.586838,0.406692,0.107408\r\n5547.2,0.8365077,0.58681,0.406685,0.107389\r\n5554.41,0.83686115,0.586075,0.406502,0.107023\r\n5555.4,0.83721985,0.585974,0.406477,0.106972\r\n5558.08,0.8375833,0.585701,0.406296,0.106836\r\n5562.13,0.8379354,0.585082,0.406021,0.10663\r\n5565.54,0.83825075,0.584561,0.40579,0.106515\r\n5566.6,0.83853645,0.584344,0.405718,0.106479\r\n5569.37,0.8388011,0.583778,0.40553,0.106386\r\n5572.99,0.83905845,0.58304,0.405285,0.106263\r\n5573.34,0.83927195,0.583017,0.405261,0.106252\r\n5577.8,0.83944355,0.582714,0.404959,0.106025\r\n5580.57,0.83958675,0.582526,0.404818,0.105884\r\n5584.19,0.8396629,0.582281,0.404634,0.1057\r\n5588.02,0.8397311,0.581694,0.404439,0.105506\r\n5588.27,0.8396953,0.581656,0.404427,0.105493\r\n5591.65,0.8396498,0.58114,0.404255,0.105379\r\n5591.74,0.83957335,0.581128,0.404251,0.105376\r\n5592.74,0.8394028,0.580992,0.4042,0.105342\r\n5599.19,0.83922975,0.580115,0.403762,0.105124\r\n5599.48,0.83902895,0.580076,0.403743,0.105114\r\n5602.83,0.83876465,0.57962,0.403515,0.104944\r\n5603.94,0.8384771,0.579545,0.40344,0.104888\r\n5606.65,0.8381067,0.579362,0.403165,0.10475\r\n5610.36,0.83769785,0.57911,0.402787,0.104561\r\n5614.03,0.83727735,0.578861,0.402412,0.104375\r\n5614.41,0.8368487,0.578802,0.402374,0.104355\r\n5615.13,0.83639245,0.578693,0.402301,0.104331\r\n5617.82,0.8359171,0.578282,0.402119,0.10424\r\n5621.49,0.83540495,0.57772,0.401869,0.104116\r\n5625.27,0.83490835,0.577336,0.401613,0.103988\r\n5625.62,0.8344113,0.5773,0.40159,0.103977\r\n5626.33,0.83388765,0.577227,0.401541,0.103952\r\n5628.98,0.833365,0.576957,0.401136,0.103863\r\n5632.68,0.832839,0.57658,0.40057,0.103738\r\n5633.79,0.8323618,0.576506,0.400401,0.1037\r\n5636.44,0.83187855,0.576326,0.400221,0.103611\r\n5636.83,0.8313374,0.5763,0.400195,0.103598\r\n5640.15,0.8308335,0.576074,0.399969,0.103429\r\n5643.88,0.83032345,0.575821,0.399716,0.103239\r\n5644.99,0.829819,0.575746,0.399642,0.103183\r\n5647.61,0.8292806,0.575568,0.399375,0.10305\r\n5651.76,0.82872925,0.575287,0.398952,0.102839\r\n5655.06,0.82825375,0.575063,0.398616,0.102727\r\n5655.08,0.82779795,0.575062,0.398613,0.102727\r\n5656.18,0.82729665,0.574987,0.398502,0.10269\r\n5662.52,0.8267993,0.574558,0.397856,0.102475\r\n5662.97,0.82631125,0.574527,0.397811,0.10246\r\n5666.28,0.825857,0.574302,0.397473,0.102292\r\n5667.37,0.8254198,0.574228,0.397362,0.102236\r\n5673.72,0.82498695,0.573798,0.396067,0.101914\r\n5674.83,0.82456195,0.573723,0.395841,0.101858\r\n5677.48,0.82419015,0.573543,0.395435,0.101723\r\n5677.9,0.82383555,0.573532,0.39537,0.101701\r\n5682.28,0.8234732,0.573421,0.3947,0.101404\r\n5688.65,0.82313115,0.57326,0.394052,0.100972\r\n5689.1,0.82279705,0.573249,0.394006,0.100942\r\n5692.42,0.8225311,0.573165,0.393668,0.10083\r\n5693.48,0.82229375,0.573093,0.393561,0.100794\r\n5700.31,0.8220648,0.57263,0.393097,0.100563\r\n5703.6,0.8218559,0.572407,0.392874,0.10048\r\n5703.62,0.82168945,0.572405,0.392873,0.100479\r\n5704.68,0.8215567,0.572334,0.392801,0.100453\r\n5714.82,0.82142415,0.571646,0.391768,0.100196\r\n5715.25,0.8213,0.571614,0.391725,0.100185\r\n5715.87,0.82122435,0.571566,0.391661,0.100143\r\n5718.55,0.8211421,0.571362,0.391252,0.0999613\r\n5723.33,0.82106585,0.570997,0.390521,0.0996374\r\n5726.45,0.8209877,0.570759,0.390203,0.0994256\r\n5729.75,0.82090675,0.570507,0.389867,0.099342\r\n5734.52,0.82081495,0.570021,0.389381,0.0992213\r\n5737.22,0.82071085,0.569747,0.389198,0.0991531\r\n5740.95,0.8206056,0.569367,0.388945,0.0990586\r\n5741.39,0.8204894,0.569322,0.388915,0.0990474\r\n5745.72,0.8203757,0.568881,0.388622,0.0987539\r\n5748.41,0.820243,0.568607,0.388348,0.0985715\r\n5752.14,0.82010335,0.568227,0.387968,0.0983185\r\n5752.59,0.81991355,0.568181,0.387922,0.0982881\r\n5755.86,0.81970215,0.567848,0.387589,0.0980663\r\n5756.91,0.81943625,0.567741,0.387482,0.097995\r\n5759.57,0.81916975,0.56747,0.387211,0.0978145\r\n5759.6,0.8188277,0.567467,0.387208,0.0978126\r\n5763.29,0.81847535,0.566904,0.386833,0.0975627\r\n5763.79,0.818111,0.566827,0.386782,0.0975287\r\n5764.38,0.81769285,0.566737,0.386722,0.0975089\r\n5767,0.8172723,0.566335,0.386522,0.0974201\r\n5767.06,0.81671,0.566327,0.386517,0.0974182\r\n5770.69,0.81622905,0.565771,0.38624,0.0972953\r\n5770.72,0.81560155,0.565767,0.386238,0.0972945\r\n5774.41,0.8149961,0.565203,0.385957,0.0971697\r\n5774.52,0.8143946,0.565186,0.385948,0.0971661\r\n5774.99,0.813748,0.565113,0.385912,0.0971499\r\n5778.12,0.8130794,0.564635,0.385673,0.096991\r\n5779.31,0.81238105,0.564454,0.385583,0.0969309\r\n5781.84,0.8116943,0.564067,0.385325,0.0968023\r\n5781.97,0.8109716,0.564046,0.385311,0.0967954\r\n5785.56,0.81026695,0.563559,0.384946,0.0966134\r\n5789.93,0.8096139,0.562965,0.384501,0.0963912\r\n5790.5,0.80898885,0.562887,0.384443,0.0963719\r\n5793.04,0.8084944,0.562543,0.384271,0.0962862\r\n5793.06,0.80807665,0.56254,0.384269,0.0962854\r\n5793.08,0.8078105,0.562536,0.384268,0.0962846\r\n5793.1,0.80755575,0.562533,0.384266,0.0962839\r\n5793.16,0.807388,0.562526,0.384263,0.0962819\r\n5796.86,0.80733115,0.56139,0.384011,0.0961568\r\n5796.88,0.80729565,0.561385,0.38401,0.0961562\r\n5796.89,0.8074861,0.561383,0.38401,0.096156\r\n5796.91,0.80767655,0.561379,0.384008,0.0961552\r\n5796.93,0.80809525,0.561374,0.384007,0.0961545\r\n5796.96,0.8085923,0.56137,0.384005,0.0961537\r\n5796.98,0.80917525,0.561365,0.384004,0.0961529\r\n5800.74,0.8098984,0.560597,0.383749,0.0960258\r\n5800.76,0.8107219,0.560593,0.383747,0.096025\r\n5800.78,0.8116733,0.560588,0.383746,0.0960243\r\n5800.81,0.8127029,0.560584,0.383744,0.0960235\r\n5800.83,0.81386355,0.560579,0.383743,0.0960227\r\n5800.85,0.8150892,0.560574,0.383741,0.096022\r\n5800.87,0.81631865,0.56057,0.38374,0.0960212\r\n5801.14,0.8175531,0.560516,0.383722,0.0960123\r\n5801.7,0.81876475,0.560401,0.383684,0.0959837\r\n5804.33,0.8199064,0.559864,0.383505,0.09585\r\n5804.63,0.82104805,0.559818,0.383485,0.0958347\r\n5804.66,0.82226465,0.559814,0.383483,0.0958335\r\n5804.68,0.82348235,0.559811,0.383482,0.0958323\r\n5804.7,0.82471905,0.559807,0.38348,0.0958312\r\n5804.73,0.8260129,0.559804,0.383479,0.09583\r\n5808.48,0.82730675,0.559229,0.383224,0.095639\r\n5808.51,0.8286006,0.559225,0.383222,0.0956379\r\n5808.53,0.82989445,0.559222,0.383221,0.0956367\r\n5808.55,0.83117775,0.559218,0.383219,0.0956356\r\n5811.79,0.8323955,0.558723,0.383,0.0954712\r\n5812.31,0.83361325,0.558616,0.382964,0.0954446\r\n5812.34,0.834831,0.558612,0.382963,0.0954434\r\n5812.36,0.83604875,0.558607,0.382961,0.0954423\r\n5812.38,0.8372009,0.558603,0.382959,0.0954411\r\n5812.9,0.8383412,0.558497,0.382924,0.0954148\r\n5816.07,0.83940675,0.557849,0.382709,0.0952536\r\n5816.14,0.84054315,0.557835,0.382705,0.0952513\r\n5816.16,0.8417442,0.55783,0.382703,0.0952505\r\n5816.19,0.84296805,0.557826,0.382701,0.0952497\r\n5816.21,0.8442619,0.557821,0.3827,0.095249\r\n5816.23,0.84555575,0.557816,0.382698,0.0952482\r\n5816.25,0.846834,0.557812,0.382697,0.0952474\r\n5819.24,0.8480518,0.557202,0.382494,0.0951465\r\n5820.01,0.8492696,0.557084,0.382442,0.0951203\r\n5820.04,0.85048735,0.557081,0.38244,0.0951196\r\n5820.06,0.8517051,0.557077,0.382439,0.0951188\r\n5820.08,0.8529229,0.557074,0.382437,0.095118\r\n5823.84,0.8541407,0.556499,0.382182,0.0949909\r\n5823.86,0.85536905,0.556495,0.382181,0.0949902\r\n5823.89,0.8566629,0.556492,0.382179,0.0949894\r\n5823.91,0.85788445,0.556488,0.382178,0.0949886\r\n5824.1,0.85908255,0.556459,0.382165,0.0949823\r\n5826.7,0.8602163,0.556062,0.382033,0.0948944\r\n5827.28,0.8613475,0.555943,0.382003,0.0948748\r\n5827.67,0.86249055,0.555863,0.381984,0.0948615\r\n5827.69,0.8637083,0.555859,0.381982,0.0948607\r\n5827.72,0.86492605,0.555854,0.381981,0.09486\r\n5827.74,0.8661438,0.555849,0.38198,0.0948592\r\n5831.5,0.86736155,0.555082,0.381789,0.0947321\r\n5831.52,0.86857935,0.555077,0.381788,0.0947313\r\n5831.54,0.86979715,0.555073,0.381787,0.0947306\r\n5831.57,0.87103055,0.555068,0.381786,0.0947298\r\n5831.59,0.8723001,0.555063,0.381784,0.094729\r\n5834.15,0.87351785,0.554541,0.381654,0.0946425\r\n5835.35,0.8747356,0.554357,0.381593,0.0946019\r\n5835.37,0.87595335,0.554354,0.381592,0.0946011\r\n5835.39,0.8771711,0.55435,0.381591,0.0946004\r\n5835.42,0.8783748,0.554347,0.38159,0.0945996\r\n5838.48,0.8795136,0.553878,0.381434,0.094496\r\n5839.03,0.88057915,0.553794,0.381406,0.0944679\r\n5839.18,0.881717,0.553772,0.381401,0.0944607\r\n5839.2,0.8828783,0.553769,0.381401,0.0944596\r\n5839.22,0.88410395,0.553765,0.3814,0.0944584\r\n5839.24,0.88536945,0.553762,0.381399,0.0944572\r\n5841.61,0.8865872,0.553401,0.381319,0.0943373\r\n5843,0.88780495,0.553187,0.381272,0.0942663\r\n5843.03,0.8890227,0.553183,0.381271,0.0942651\r\n5843.05,0.89024045,0.55318,0.38127,0.0942639\r\n5843.07,0.89145825,0.553176,0.38127,0.0942628\r\n5846.83,0.892676,0.552601,0.381143,0.0940718\r\n5846.86,0.89389375,0.552598,0.381142,0.0940706\r\n5846.88,0.8951115,0.552594,0.381141,0.0940695\r\n5846.9,0.89635355,0.552591,0.38114,0.0940683\r\n5846.92,0.8976474,0.552587,0.38114,0.0940672\r\n5846.95,0.898908,0.552584,0.381139,0.094066\r\n5849.06,0.90011635,0.55226,0.381067,0.0939585\r\n5850.24,0.901258,0.55202,0.381027,0.0938987\r\n5850.71,0.9024137,0.551925,0.381016,0.093875\r\n5850.73,0.9036343,0.55192,0.381015,0.0938739\r\n5850.75,0.90492815,0.551915,0.381014,0.0938727\r\n5850.77,0.906222,0.551911,0.381014,0.0938716\r\n5850.8,0.90749285,0.551906,0.381013,0.0938704\r\n5853.42,0.90871065,0.551371,0.380947,0.0937372\r\n5854.56,0.90992025,0.551139,0.380918,0.0936987\r\n5856.51,0.91113805,0.550739,0.380869,0.0936326\r\n5858.32,0.9123088,0.550001,0.380823,0.0935716\r\n5860.23,0.9134315,0.549218,0.380775,0.093507\r\n5862.07,0.9144911,0.548936,0.380728,0.0934447\r\n5864.62,0.91547735,0.548545,0.380663,0.0933584\r\n5865.18,0.91639075,0.54846,0.380649,0.0933301\r\n5865.81,0.9171116,0.548363,0.380617,0.093298\r\n5867.68,0.9176445,0.548077,0.380522,0.0932029\r\n5869.48,0.9181774,0.547341,0.380431,0.0931116\r\n5869.5,0.9187103,0.547332,0.38043,0.0931105\r\n5869.53,0.91892175,0.547322,0.380428,0.0931093\r\n5871.4,0.91885525,0.546556,0.380333,0.0930143\r\n5873.1,0.9188554,0.545857,0.380247,0.0929276\r\n5873.13,0.91885555,0.545847,0.380246,0.0929264\r\n5873.15,0.9188557,0.545838,0.380244,0.0929253\r\n5873.17,0.9188559,0.545829,0.380243,0.0929241\r\n5873.2,0.9184517,0.545819,0.380242,0.092923\r\n5875.11,0.9177139,0.545034,0.380145,0.0928256\r\n5876.75,0.91702915,0.544363,0.380061,0.0927424\r\n5876.77,0.9163809,0.544354,0.38006,0.0927412\r\n5876.8,0.9157722,0.544345,0.380059,0.0927401\r\n5876.82,0.9147906,0.544335,0.380058,0.0927389\r\n5878.83,0.9136553,0.543513,0.379956,0.0926369\r\n5879.56,0.9124115,0.543401,0.379919,0.0925996\r\n5880.12,0.9111211,0.543315,0.37989,0.0925618\r\n5880.37,0.90990355,0.543276,0.379882,0.0925445\r\n5880.4,0.90887855,0.543273,0.379881,0.0925429\r\n5880.42,0.9080415,0.543269,0.37988,0.0925414\r\n5880.44,0.90720445,0.543266,0.379879,0.0925399\r\n5880.46,0.9056825,0.543262,0.379879,0.0925383\r\n5884.02,0.90451525,0.542719,0.379759,0.0922973\r\n5884.04,0.9036782,0.542715,0.379758,0.0922957\r\n5884.07,0.90284115,0.542712,0.379757,0.0922942\r\n5884.09,0.9015023,0.542708,0.379756,0.0922927\r\n5886.28,0.89982815,0.542372,0.379682,0.0921438\r\n5887.62,0.89815395,0.541825,0.379637,0.0920532\r\n5887.64,0.89688415,0.541815,0.379636,0.0920516\r\n5887.67,0.89597095,0.541806,0.379635,0.0920501\r\n5887.69,0.89505775,0.541797,0.379634,0.0920485\r\n5887.71,0.8941446,0.541788,0.379634,0.092047\r\n5887.73,0.8926118,0.541778,0.379633,0.0920455\r\n5890,0.8912399,0.540851,0.379556,0.091892\r\n5890.76,0.889883,0.540734,0.379531,0.0918403\r\n5891.22,0.8886977,0.540664,0.379515,0.0918247\r\n5891.24,0.8876323,0.54066,0.379514,0.091824\r\n5891.27,0.88656695,0.540657,0.379514,0.0918232\r\n5891.29,0.8855016,0.540653,0.379513,0.0918224\r\n5891.31,0.8843982,0.54065,0.379512,0.0918216\r\n5891.32,0.88325675,0.540648,0.379512,0.0918213\r\n5891.33,0.88196305,0.540646,0.379511,0.0918209\r\n5894.84,0.8806694,0.54011,0.379333,0.0917022\r\n5894.87,0.87937575,0.540106,0.379332,0.0917015\r\n5894.89,0.8780821,0.540103,0.37933,0.0917007\r\n5894.91,0.87729025,0.540099,0.379329,0.0916999\r\n5894.93,0.87632605,0.540096,0.379328,0.0916992\r\n5897.45,0.8751085,0.53971,0.3792,0.091614\r\n5898.49,0.87389095,0.539499,0.379148,0.091579\r\n5898.51,0.8726734,0.539494,0.379146,0.0915782\r\n5898.54,0.87145585,0.53949,0.379145,0.0915774\r\n5898.56,0.8697301,0.539485,0.379144,0.0915767\r\n5901.97,0.86852335,0.53879,0.378971,0.0914615\r\n5902.16,0.86757565,0.53875,0.378961,0.0914517\r\n5902.18,0.8668147,0.538746,0.37896,0.0914505\r\n5902.2,0.86563505,0.538741,0.378959,0.0914494\r\n5904.91,0.8642653,0.538189,0.378822,0.0913121\r\n5905.8,0.8628955,0.538098,0.378776,0.0912665\r\n5905.83,0.86152575,0.538096,0.378775,0.0912653\r\n5905.85,0.86019405,0.538093,0.378774,0.0912641\r\n5905.87,0.85875805,0.538091,0.378773,0.091263\r\n5906.26,0.85754055,0.538052,0.378753,0.0912433\r\n5909.43,0.856323,0.537729,0.378538,0.0910824\r\n5909.45,0.85510545,0.537727,0.378537,0.0910812\r\n5909.47,0.8538879,0.537724,0.378535,0.0910801\r\n5909.5,0.85267035,0.537722,0.378533,0.0910789\r\n5909.52,0.8513318,0.53772,0.378532,0.0910778\r\n5912.37,0.85003815,0.53743,0.378339,0.090933\r\n5913.05,0.8487445,0.53729,0.378292,0.0908983\r\n5913.07,0.84745085,0.537286,0.378291,0.0908972\r\n5913.1,0.8461572,0.537281,0.378289,0.090896\r\n5913.12,0.8453717,0.537276,0.378288,0.0908949\r\n5913.14,0.84400195,0.537272,0.378286,0.0908937\r\n5916.7,0.8426322,0.536546,0.378045,0.0907131\r\n5916.72,0.84126245,0.536541,0.378044,0.090712\r\n5916.74,0.8403114,0.536537,0.378042,0.0907108\r\n5916.76,0.839451,0.536532,0.378041,0.0907097\r\n5916.9,0.83841905,0.536504,0.378031,0.0907027\r\n5917.46,0.8369067,0.53639,0.377994,0.0906839\r\n5919.82,0.8352325,0.535909,0.377934,0.0906041\r\n5920.32,0.83370065,0.535806,0.377921,0.0905872\r\n5920.34,0.83278745,0.535802,0.377921,0.0905864\r\n5920.37,0.83187425,0.535797,0.37792,0.0905856\r\n5920.39,0.8309611,0.535793,0.377919,0.0905849\r\n5920.41,0.8292869,0.535788,0.377919,0.0905841\r\n5923.94,0.82761275,0.535067,0.377829,0.0904647\r\n5923.97,0.82656725,0.535062,0.377829,0.0904639\r\n5923.99,0.8257302,0.535058,0.377828,0.0904631\r\n5924.01,0.82489315,0.535053,0.377828,0.0904624\r\n5924.03,0.82356955,0.535048,0.377827,0.0904616\r\n5927.27,0.8221237,0.534388,0.377745,0.0903522\r\n5927.63,0.82067785,0.534276,0.377736,0.0903399\r\n5927.66,0.8198045,0.534269,0.377735,0.0903391\r\n5928.11,0.818622,0.534131,0.377724,0.0903239\r\n5930.99,0.8172494,0.533247,0.377651,0.0902509\r\n5932.4,0.81561455,0.532669,0.377615,0.0902152\r\n5934.7,0.8139828,0.531725,0.377498,0.0901569\r\n5934.95,0.81241115,0.531623,0.377486,0.0901506\r\n5938.42,0.8112411,0.530204,0.37731,0.0900629\r\n5938.68,0.8101901,0.530164,0.377296,0.0900562\r\n5943.05,0.80908455,0.529496,0.377074,0.0899457\r\n5945.88,0.80790515,0.529063,0.376931,0.089754\r\n5946.17,0.80667325,0.529003,0.376916,0.089734\r\n5947.34,0.80541775,0.528765,0.376857,0.0896549\r\n5949.93,0.80491595,0.528235,0.376769,0.089479\r\n5953.33,0.8043454,0.527542,0.376654,0.0892488\r\n5953.69,0.8037044,0.527487,0.376642,0.0892241\r\n5954.25,0.8030309,0.527401,0.376623,0.0891863\r\n5958.54,0.8023041,0.526745,0.376478,0.0890412\r\n5960.78,0.80199185,0.526402,0.376364,0.0889654\r\n5961.19,0.80180185,0.526361,0.376343,0.0889517\r\n5964.95,0.80153965,0.525978,0.376152,0.0888246\r\n5965.45,0.8013635,0.525926,0.376127,0.0888075\r\n5971.98,0.8014239,0.525262,0.375795,0.0884761\r\n5972.44,0.80160625,0.525192,0.375772,0.0884527\r\n5973.48,0.80202015,0.525032,0.375719,0.0883998\r\n5979.43,0.8024581,0.524122,0.375315,0.0880973\r\n5979.93,0.80293605,0.524054,0.375282,0.0880722\r\n5980.39,0.80346025,0.523992,0.37525,0.0880488\r\n5984.68,0.80397905,0.523409,0.37496,0.0879038\r\n5987.43,0.80449255,0.523036,0.37496,0.0878109\r\n5990.62,0.8050088,0.522601,0.374961,0.0877028\r\n5991.59,0.80553445,0.522453,0.374961,0.08767\r\n5998.08,0.80601045,0.521461,0.374962,0.0872303\r\n5998.65,0.80642515,0.521373,0.374962,0.0871914\r\n5999.63,0.8068566,0.521224,0.374962,0.0871254\r\n6002.79,0.8073033,0.52074,0.374882,0.0869106\r\n6005.54,0.80771105,0.52032,0.374812,0.0868412\r\n6006.14,0.8081389,0.520279,0.374797,0.0868258\r\n6014.57,0.80857365,0.519708,0.374584,0.0866126\r\n6016.73,0.80893555,0.519561,0.374529,0.0865577\r\n6017.37,0.8092954,0.519464,0.374513,0.0865416\r\n6017.74,0.8096384,0.519408,0.374503,0.0865324\r\n6024.19,0.8100003,0.518421,0.37434,0.0860947\r\n6028.6,0.81035685,0.517822,0.374228,0.0857958\r\n6028.94,0.81067235,0.517776,0.37422,0.085773\r\n6029.51,0.81102885,0.517698,0.374205,0.0857585\r\n6035.38,0.8113861,0.5169,0.374057,0.0856099\r\n6036.09,0.8117341,0.516792,0.374039,0.0855919\r\n6042.84,0.8120595,0.51576,0.373868,0.0854212\r\n6043.88,0.8123716,0.515689,0.373842,0.0853948\r\n6044.45,0.81269325,0.51565,0.373827,0.0853559\r\n6047.32,0.81303315,0.515456,0.37373,0.0851615\r\n6054.04,0.81337605,0.515001,0.373503,0.084706\r\n6054.81,0.81371285,0.514922,0.373477,0.0846536\r\n6055.08,0.81402845,0.514895,0.373468,0.0846355\r\n6055.66,0.8143719,0.514836,0.373448,0.0846159\r\n6065.23,0.8147148,0.513861,0.37345,0.0842922\r\n6066.04,0.8150167,0.513779,0.37345,0.0842649\r\n6066.28,0.81534055,0.513754,0.37345,0.0842567\r\n6070.6,0.81568345,0.513313,0.373451,0.0841472\r\n6073.53,0.81603925,0.513016,0.373451,0.0840732\r\n6076.42,0.8163866,0.512721,0.373452,0.0839999\r\n6081.22,0.81672505,0.511987,0.373452,0.0838784\r\n6083.88,0.81706535,0.51158,0.373453,0.0836983\r\n6084.76,0.8174094,0.511491,0.373453,0.0836389\r\n6085.55,0.81777675,0.51141,0.373453,0.0835849\r\n6092.25,0.81814,0.510728,0.373454,0.0831307\r\n6092.42,0.8184891,0.510711,0.373454,0.0831191\r\n6095.07,0.8188685,0.510441,0.373454,0.0829394\r\n6099.74,0.81925075,0.510322,0.373455,0.0826228\r\n6100.5,0.81964225,0.510303,0.373455,0.0825715\r\n6103.62,0.82001005,0.510224,0.373456,0.0823598\r\n6107.23,0.8204002,0.510133,0.373456,0.0822683\r\n6110.02,0.8208369,0.510062,0.373457,0.0821979\r\n6114.73,0.8212731,0.509823,0.373457,0.0820786\r\n6115.45,0.82167115,0.509786,0.373457,0.0820604\r\n6118.56,0.8220868,0.509628,0.373379,0.0819815\r\n6124.95,0.82250395,0.509304,0.373217,0.0815484\r\n6125.95,0.82292685,0.509338,0.373192,0.0814806\r\n6129.76,0.8233338,0.509468,0.373095,0.0812222\r\n6130.39,0.8237571,0.509489,0.373079,0.0812063\r\n6133.45,0.8241767,0.509593,0.37308,0.0811289\r\n6136.17,0.8246192,0.509686,0.37308,0.0810601\r\n6140.94,0.8250849,0.510656,0.373081,0.0809393\r\n6143.67,0.8255531,0.511209,0.373081,0.0808703\r\n6144.71,0.8259596,0.511368,0.373081,0.0808439\r\n6145.34,0.8263963,0.511464,0.373081,0.0808011\r\n6148.44,0.82683355,0.511937,0.372766,0.080591\r\n6151.16,0.82726385,0.512352,0.372489,0.0804066\r\n6154.92,0.8276959,0.513874,0.372106,0.0801517\r\n6155.91,0.82810435,0.514275,0.372005,0.0800846\r\n6155.93,0.82849745,0.514284,0.372003,0.0800838\r\n6156.53,0.8289057,0.514529,0.371942,0.0800635\r\n6158.68,0.82928655,0.515397,0.371723,0.079991\r\n6159.69,0.8297411,0.515603,0.37162,0.0799567\r\n6163.45,0.8302402,0.516367,0.371237,0.0798296\r\n6166.17,0.8307366,0.51692,0.37096,0.0797375\r\n6167.11,0.8312316,0.517206,0.370864,0.0797058\r\n6167.21,0.83169185,0.517235,0.370854,0.0797034\r\n6167.72,0.8321625,0.517393,0.370802,0.0796903\r\n6169.93,0.83260225,0.518062,0.370578,0.0796345\r\n6169.95,0.8330499,0.519584,0.370575,0.079634\r\n6173.71,0.8334687,0.521107,0.370192,0.0795388\r\n6174.7,0.833936,0.521258,0.370092,0.0795137\r\n6178.46,0.8343974,0.521832,0.369709,0.0794186\r\n6178.92,0.83487555,0.521902,0.369662,0.0794069\r\n6181.2,0.83533085,0.52225,0.369313,0.0793492\r\n6182.05,0.83579495,0.522595,0.369183,0.0793276\r\n6184.96,0.8362405,0.523772,0.368738,0.0791305\r\n6185.95,0.83668705,0.524176,0.368586,0.079063\r\n6186.38,0.83718085,0.524346,0.368522,0.0790345\r\n6188.72,0.8376826,0.525295,0.368163,0.0788756\r\n6189.71,0.8382126,0.525447,0.368011,0.0788081\r\n6193.25,0.83872655,0.525987,0.36747,0.0785682\r\n6193.47,0.8391671,0.52602,0.367436,0.0785626\r\n6193.83,0.83957065,0.526075,0.367381,0.0785535\r\n6196.21,0.83998385,0.526438,0.367139,0.0784934\r\n6199.97,0.84039525,0.52796,0.366756,0.0783982\r\n6199.99,0.8408101,0.529482,0.366754,0.0783976\r\n6200.96,0.84126465,0.529876,0.366655,0.078373\r\n6203.75,0.8417387,0.531005,0.366371,0.0783024\r\n6204.72,0.8421902,0.531203,0.366272,0.0782778\r\n6205.03,0.8426983,0.531264,0.366241,0.0782702\r\n6208.19,0.8431361,0.531908,0.365757,0.07819\r\n6208.48,0.8435973,0.531967,0.365712,0.0781802\r\n6211.25,0.8440356,0.532528,0.36529,0.0780867\r\n6211.27,0.84447755,0.53405,0.365286,0.078086\r\n6212.48,0.84491585,0.534541,0.365101,0.078045\r\n6215.03,0.84534425,0.535573,0.364581,0.0779589\r\n6215.05,0.84576435,0.537095,0.364576,0.0779581\r\n6215.08,0.8461462,0.538617,0.364571,0.0779573\r\n6215.98,0.84655125,0.538892,0.364387,0.0779267\r\n6218.83,0.8468933,0.539759,0.363805,0.0778304\r\n6219.4,0.847236,0.539875,0.363689,0.0778112\r\n6219.73,0.84757995,0.539943,0.363621,0.0777941\r\n6219.93,0.84797785,0.539984,0.36358,0.077784\r\n6223.49,0.8484045,0.540707,0.363217,0.0776031\r\n6226.33,0.8488314,0.541283,0.362929,0.0774592\r\n6226.35,0.8492692,0.542805,0.362927,0.0774581\r\n6227.25,0.84967955,0.543171,0.362834,0.0774121\r\n6230.11,0.85007045,0.544327,0.362544,0.0772671\r\n6230.13,0.85046535,0.545849,0.362541,0.0772659\r\n6231.13,0.8508242,0.546252,0.36244,0.0772154\r\n6233.89,0.8511817,0.547372,0.362017,0.0770749\r\n6234.34,0.8515153,0.547462,0.361949,0.0770524\r\n6234.75,0.8519248,0.547546,0.361886,0.0770243\r\n6238.51,0.85232255,0.54831,0.361311,0.0767693\r\n6238.58,0.8527158,0.548325,0.3613,0.0767643\r\n6241.39,0.853109,0.548895,0.360727,0.0765742\r\n6241.41,0.85350215,0.550417,0.360723,0.0765727\r\n6241.43,0.85388625,0.551939,0.360718,0.0765711\r\n6241.46,0.8542557,0.553461,0.360713,0.0765696\r\n6245.22,0.85461665,0.554984,0.359946,0.0763147\r\n6245.24,0.8549727,0.556506,0.359941,0.0763131\r\n6245.26,0.8553249,0.558028,0.359937,0.0763116\r\n6245.28,0.8556069,0.55955,0.359932,0.07631\r\n6245.54,0.8558351,0.559652,0.359881,0.0762931\r\n6246,0.8560635,0.559841,0.359786,0.0762773\r\n6246.04,0.85633455,0.559854,0.359779,0.0762762\r\n6249.04,0.8565911,0.561073,0.359319,0.0761744\r\n6249.76,0.85690905,0.561363,0.359209,0.0761502\r\n6252.8,0.85721725,0.562596,0.358744,0.0760473\r\n6252.82,0.8575109,0.563737,0.358741,0.0760468\r\n6253.49,0.85780065,0.564009,0.358638,0.0760241\r\n6253.52,0.85810955,0.56402,0.358635,0.0760231\r\n6256.58,0.8583435,0.56526,0.358324,0.0759197\r\n6256.74,0.85860435,0.565324,0.358308,0.0759143\r\n6260.34,0.858844,0.566782,0.357941,0.0757314\r\n6260.36,0.85908365,0.568304,0.357939,0.0757302\r\n6260.39,0.8593279,0.569827,0.357936,0.0757291\r\n6261.01,0.8596038,0.57008,0.357873,0.0756973\r\n6264.15,0.85985125,0.571349,0.357553,0.0755381\r\n6264.17,0.8601038,0.572871,0.357551,0.0755369\r\n6264.69,0.860395,0.572976,0.357498,0.0755106\r\n6268.51,0.86074995,0.573753,0.356718,0.0753164\r\n6271.67,0.86110325,0.574394,0.356074,0.0751561\r\n6271.68,0.86146225,0.574399,0.356072,0.0751555\r\n6272.14,0.86182325,0.574586,0.355977,0.0751399\r\n6272.27,0.8621474,0.574639,0.355957,0.0751355\r\n6275.42,0.8624572,0.575917,0.355475,0.0750288\r\n6275.45,0.862737,0.577439,0.355471,0.075028\r\n6279.59,0.8630177,0.578282,0.354837,0.0748878\r\n6279.76,0.8633166,0.578315,0.354812,0.0748823\r\n6282.88,0.8636155,0.57895,0.354334,0.0747767\r\n6282.94,0.86389125,0.578962,0.354325,0.0747751\r\n6282.96,0.864204,0.580104,0.354322,0.0747747\r\n6286.72,0.86449265,0.581627,0.353747,0.0746795\r\n6287.05,0.86478315,0.58176,0.353697,0.0746712\r\n6287.26,0.8650981,0.581843,0.353655,0.074666\r\n6290.48,0.86540845,0.583149,0.352997,0.0745843\r\n6290.5,0.86571565,0.584671,0.352992,0.0745838\r\n6294.26,0.8660228,0.586194,0.352224,0.0744886\r\n6294.29,0.86632485,0.587716,0.35222,0.074488\r\n6294.31,0.86658955,0.589238,0.352215,0.0744874\r\n6294.5,0.8667922,0.589316,0.352176,0.0744825\r\n6294.75,0.86702585,0.589415,0.352101,0.0744763\r\n6297.82,0.8672527,0.590662,0.351157,0.0743985\r\n6298.07,0.86747715,0.590761,0.351082,0.0743819\r\n6298.09,0.8676788,0.592283,0.351075,0.0743803\r\n6298.22,0.86791715,0.592336,0.351035,0.0743715\r\n6301.85,0.8681241,0.593805,0.350294,0.0741254\r\n6301.87,0.86833005,0.595328,0.350289,0.0741239\r\n6301.9,0.86850445,0.59685,0.350285,0.0741223\r\n6301.92,0.8686785,0.598372,0.35028,0.0741208\r\n6301.94,0.8688554,0.599894,0.350275,0.0741192\r\n6302.24,0.86902025,0.600016,0.350214,0.0740988\r\n6305.67,0.869182,0.601405,0.349514,0.0738663\r\n6305.7,0.86934185,0.601416,0.349508,0.0738643\r\n6305.72,0.8694773,0.602938,0.349503,0.0738628\r\n6305.75,0.8696127,0.604461,0.349499,0.0738613\r\n6305.77,0.86974425,0.605983,0.349494,0.0738597\r\n6309.02,0.8698795,0.6073,0.34883,0.0736391\r\n6309.53,0.87001475,0.607505,0.348727,0.073622\r\n6309.55,0.8701499,0.609027,0.348722,0.0736212\r\n6309.73,0.87030905,0.609101,0.348685,0.0736151\r\n6313.12,0.8704384,0.610474,0.347993,0.0735005\r\n6313.31,0.87059215,0.61055,0.347935,0.0734941\r\n6316.84,0.8707474,0.61198,0.346852,0.0733747\r\n6317.07,0.87090155,0.612073,0.346805,0.073367\r\n6317.09,0.87101985,0.613595,0.346801,0.0733663\r\n6317.22,0.87115315,0.613648,0.346774,0.0733618\r\n6320.23,0.8712894,0.614864,0.346161,0.0732603\r\n6320.85,0.87142555,0.615117,0.346033,0.0732445\r\n6320.88,0.8715789,0.616639,0.346029,0.0732439\r\n6324.29,0.8717309,0.618024,0.345331,0.0731574\r\n6324.64,0.87187395,0.618162,0.345191,0.0731487\r\n6328.01,0.872019,0.618677,0.343809,0.0730633\r\n6328.45,0.8721639,0.618744,0.343742,0.0730521\r\n6332.13,0.87230865,0.619305,0.34318,0.0729591\r\n6335.17,0.87245325,0.620537,0.342714,0.0728821\r\n6335.47,0.87257305,0.620657,0.342669,0.072862\r\n6335.89,0.872689,0.620827,0.342497,0.0728335\r\n6339.18,0.8728048,0.622161,0.341148,0.0726102\r\n6339.65,0.8729192,0.62235,0.341052,0.0725786\r\n6343.4,0.8730077,0.62387,0.340286,0.072324\r\n6343.41,0.8730721,0.623873,0.340285,0.0723237\r\n6346.37,0.8731076,0.625073,0.33968,0.0721227\r\n6346.63,0.87313725,0.625179,0.339627,0.0721161\r\n6347.17,0.87316655,0.625395,0.339408,0.0721026\r\n6347.19,0.8731707,0.626917,0.339398,0.072102\r\n6350.35,0.8731557,0.628196,0.338105,0.0720221\r\n6350.86,0.873137,0.628403,0.338027,0.0720091\r\n6350.95,0.8731181,0.62844,0.338013,0.0720068\r\n6350.97,0.8730626,0.629962,0.338009,0.0720063\r\n6354.73,0.8730053,0.631485,0.337434,0.0719111\r\n6354.75,0.8729153,0.633007,0.337431,0.0719105\r\n6357.8,0.87281945,0.633934,0.336965,0.0718333\r\n6358.51,0.8727093,0.634149,0.336676,0.0718155\r\n6361.31,0.87259805,0.635285,0.335527,0.0717445\r\n6361.52,0.8724841,0.635368,0.335443,0.0717376\r\n6362.05,0.87235895,0.635584,0.335334,0.0717195\r\n6362.27,0.8720394,0.635672,0.33529,0.0717122\r\n6366.03,0.8717187,0.637194,0.334522,0.0715851\r\n6366.05,0.8713968,0.638716,0.334518,0.0715843\r\n6366.07,0.8709253,0.640238,0.334513,0.0715835\r\n6368.97,0.8704318,0.641411,0.333922,0.0714857\r\n6369.83,0.8698015,0.641761,0.33379,0.0714564\r\n6372.52,0.8691456,0.642848,0.333379,0.0713657\r\n6373.09,0.8684796,0.643081,0.333292,0.0713267\r\n6373.59,0.86777975,0.643284,0.333215,0.0712928\r\n6376.42,0.867076,0.644431,0.332782,0.0711007\r\n6376.81,0.86636855,0.644585,0.332626,0.0710749\r\n6377.35,0.8656399,0.644806,0.332402,0.0710379\r\n6380.14,0.86492295,0.645935,0.33126,0.0708489\r\n6381.11,0.8641862,0.646329,0.331062,0.070783\r\n6383.72,0.86346075,0.647384,0.33053,0.0706063\r\n6384.26,0.86272645,0.647605,0.330419,0.0705925\r\n6384.87,0.86195075,0.647852,0.330294,0.0705771\r\n6387.59,0.8611682,0.648266,0.329739,0.0705083\r\n6387.98,0.8603543,0.648325,0.329621,0.0704985\r\n6391.31,0.85954085,0.648834,0.328598,0.0704141\r\n6392.36,0.85892155,0.648994,0.328384,0.0703875\r\n6392.38,0.85827235,0.650516,0.328379,0.0703869\r\n6395.43,0.85760965,0.651749,0.327758,0.0703099\r\n6396.14,0.8570451,0.652039,0.327612,0.0702917\r\n6398.66,0.8565231,0.653057,0.327098,0.0702281\r\n6398.76,0.8561321,0.653099,0.327077,0.0702246\r\n6399.14,0.8557552,0.653253,0.327019,0.0702118\r\n6399.9,0.8553281,0.653562,0.326903,0.070186\r\n6402.86,0.8549466,0.654757,0.326451,0.0700862\r\n6403.66,0.85453375,0.655084,0.326328,0.0700589\r\n6406.22,0.8541196,0.655861,0.325937,0.0699726\r\n6406.58,0.8537095,0.65597,0.32579,0.0699604\r\n6407.42,0.85325675,0.656226,0.325445,0.069932\r\n6409.86,0.8528225,0.657217,0.324443,0.0698493\r\n6409.93,0.8523851,0.657245,0.324416,0.0698458\r\n6410.29,0.85190495,0.65739,0.324269,0.0698276\r\n6411.18,0.8513269,0.657749,0.323905,0.0697825\r\n6413.65,0.8507373,0.658749,0.322894,0.0696571\r\n6413.96,0.85025785,0.658875,0.32283,0.0696413\r\n6413.98,0.84986525,0.658885,0.322826,0.0696401\r\n6414,0.84937485,0.658894,0.322821,0.0696389\r\n6414.94,0.848633,0.659272,0.32263,0.0695916\r\n6417.63,0.84798195,0.659818,0.322081,0.0694549\r\n6417.65,0.8474585,0.659823,0.322077,0.0694537\r\n6417.67,0.8467241,0.659828,0.322072,0.0694526\r\n6421.1,0.84598225,0.660523,0.321373,0.0692786\r\n6421.34,0.84523365,0.660573,0.321298,0.0692662\r\n6422.43,0.84449705,0.660795,0.320963,0.0692107\r\n6424.8,0.8437725,0.661515,0.320237,0.0690905\r\n6424.82,0.8430822,0.66152,0.320232,0.06909\r\n6425.06,0.8423742,0.661593,0.320183,0.0690819\r\n6426.19,0.84163055,0.661937,0.319952,0.0690436\r\n6428.77,0.8409129,0.662983,0.319425,0.0689563\r\n6429.95,0.8401491,0.66346,0.319185,0.0689165\r\n6432.27,0.83938825,0.6644,0.318711,0.0688381\r\n6432.48,0.83867325,0.664487,0.318667,0.0688307\r\n6433.71,0.83809425,0.664982,0.318417,0.0687894\r\n6436,0.8375063,0.665449,0.317948,0.0687117\r\n6439.72,0.83684055,0.666204,0.31719,0.0686177\r\n6439.94,0.83608585,0.666249,0.317122,0.0686121\r\n6441.2,0.8354063,0.666506,0.316735,0.0685801\r\n6443.44,0.8349937,0.667411,0.316049,0.0685235\r\n6444.96,0.83448265,0.668028,0.315425,0.0684849\r\n6447.15,0.83389305,0.668694,0.314528,0.0684295\r\n6447.39,0.8335064,0.668767,0.314479,0.0684235\r\n6448.72,0.83311795,0.66917,0.314208,0.0683899\r\n6450.95,0.83274015,0.670073,0.313753,0.0683335\r\n6452.48,0.8323945,0.670693,0.313441,0.0682297\r\n6454.6,0.83204735,0.671554,0.313007,0.0680855\r\n6454.85,0.83169095,0.671653,0.312969,0.0680689\r\n6456.24,0.83131935,0.672216,0.312757,0.0679748\r\n6460,0.8309796,0.673738,0.312182,0.0677199\r\n6462.06,0.83066305,0.674053,0.311866,0.0675799\r\n6462.15,0.8303925,0.674066,0.311831,0.0675741\r\n6462.3,0.8301146,0.67409,0.311769,0.0675703\r\n6465.77,0.8298402,0.67462,0.310345,0.0674823\r\n6467.49,0.8295766,0.674881,0.30982,0.0674389\r\n6469.49,0.82932785,0.675289,0.309204,0.0673881\r\n6473.5,0.8290789,0.676102,0.30866,0.0672868\r\n6474.98,0.82884015,0.676404,0.308458,0.0672492\r\n6477.09,0.8286197,0.676725,0.308172,0.0671959\r\n6480.68,0.82841255,0.677273,0.307684,0.0670744\r\n6482.47,0.8282253,0.677547,0.30695,0.0670138\r\n6484.4,0.8280358,0.677743,0.306162,0.0669488\r\n6488.12,0.82785615,0.678121,0.305021,0.0668231\r\n6488.29,0.8276934,0.678139,0.304985,0.0668171\r\n6488.44,0.82754235,0.678154,0.304955,0.0668096\r\n6493.7,0.82740765,0.67869,0.303881,0.0665423\r\n6495.57,0.82727485,0.67888,0.3035,0.0664475\r\n6503.02,0.8271562,0.679639,0.30236,0.0660687\r\n6503.23,0.82707595,0.67966,0.302318,0.0660583\r\n6503.39,0.82701935,0.679676,0.302285,0.0660529\r\n6504.93,0.8269731,0.679833,0.301971,0.0660008\r\n6510.48,0.82693475,0.679457,0.300839,0.0658134\r\n6514.19,0.82690855,0.679205,0.299317,0.0656878\r\n6514.43,0.8268885,0.679189,0.29928,0.0656795\r\n6514.61,0.8268772,0.679177,0.299254,0.0656752\r\n6516.13,0.8268865,0.679074,0.299021,0.0656366\r\n6521.65,0.826905,0.6787,0.298177,0.0654969\r\n6527.33,0.8269284,0.678315,0.297017,0.0653531\r\n6529.1,0.8269567,0.677954,0.296656,0.0653083\r\n6529.38,0.8269875,0.677897,0.296627,0.0653013\r\n6529.56,0.82702745,0.67786,0.296609,0.0652889\r\n6534.78,0.82707435,0.676794,0.296077,0.0649349\r\n6536.56,0.82712665,0.676612,0.295896,0.0648142\r\n6540.58,0.82718125,0.676204,0.295076,0.0645419\r\n6542.24,0.8272381,0.676034,0.294736,0.0644856\r\n6544.01,0.82729555,0.675763,0.294375,0.0644258\r\n6544.51,0.82735955,0.675687,0.294298,0.0644089\r\n6549.7,0.82742585,0.674893,0.293505,0.0642335\r\n6551.47,0.82749255,0.674532,0.293235,0.0641737\r\n6551.78,0.8275635,0.674468,0.293171,0.0641631\r\n6557.15,0.82762725,0.673372,0.292075,0.0640272\r\n6558.92,0.82768535,0.67283,0.291714,0.0639824\r\n6559.47,0.8277431,0.672662,0.29163,0.0639686\r\n6560.87,0.8278006,0.672231,0.291415,0.063933\r\n6566.38,0.82785615,0.671389,0.290573,0.0637937\r\n6566.72,0.82790465,0.671336,0.290443,0.0637849\r\n6568.33,0.8279459,0.671091,0.289841,0.0637307\r\n6574.41,0.82798315,0.669849,0.287557,0.0635249\r\n6575.78,0.82801785,0.66957,0.287045,0.0634788\r\n6577.52,0.8280518,0.668855,0.286389,0.0634197\r\n6577.93,0.82807705,0.668689,0.286307,0.0634061\r\n6579.49,0.8280931,0.668049,0.285988,0.0633266\r\n6584.98,0.8281015,0.66721,0.284868,0.0630481\r\n6586.95,0.8281047,0.666908,0.284566,0.0629478\r\n6589.36,0.82810045,0.666417,0.284199,0.0628256\r\n6592.43,0.82809485,0.665789,0.283728,0.0626693\r\n6592.86,0.82808005,0.665701,0.283662,0.0626473\r\n6594.4,0.82805665,0.665387,0.283427,0.0625954\r\n6598.11,0.82802985,0.663866,0.282859,0.0624698\r\n6599.89,0.8279951,0.663594,0.282588,0.0624098\r\n6600.56,0.82795365,0.663492,0.282519,0.0623871\r\n6604.07,0.8279096,0.662955,0.282162,0.0622685\r\n6605.57,0.8278585,0.662725,0.282009,0.0622305\r\n6611.08,0.8278049,0.6616,0.281448,0.062091\r\n6613.02,0.82774905,0.661204,0.280653,0.0620419\r\n6614.8,0.8276923,0.66066,0.279926,0.061997\r\n6615.5,0.82763235,0.660444,0.279637,0.0619791\r\n6616.74,0.8275697,0.660063,0.279129,0.0619477\r\n6618.51,0.82750795,0.659702,0.278405,0.0619029\r\n6619.01,0.82743855,0.6596,0.278251,0.0618903\r\n6622.23,0.82736615,0.658943,0.277264,0.0617814\r\n6624.19,0.82728675,0.658542,0.276863,0.0617151\r\n6629.68,0.8272085,0.657422,0.275743,0.0615295\r\n6630.22,0.82713275,0.657313,0.275688,0.0615114\r\n6630.44,0.8270581,0.657268,0.275666,0.0615058\r\n6631.64,0.82695885,0.657022,0.275543,0.0614753\r\n6639.1,0.8268562,0.655881,0.274784,0.0612865\r\n6640.88,0.82675725,0.65561,0.274603,0.0612416\r\n6641.64,0.82665235,0.655493,0.274486,0.0612223\r\n6645.16,0.826545,0.654955,0.273948,0.0611332\r\n6646.56,0.8264376,0.654741,0.273734,0.0610859\r\n6648.33,0.8263272,0.654014,0.273462,0.0610259\r\n6650.27,0.82620975,0.653219,0.273166,0.0609603\r\n6655.79,0.8260933,0.652375,0.272322,0.0607738\r\n6656.36,0.8259786,0.652288,0.272234,0.0607544\r\n6656.57,0.82586285,0.652255,0.272202,0.0607437\r\n6657.73,0.8257409,0.652079,0.272025,0.060685\r\n6663.25,0.82562015,0.650953,0.271182,0.0604048\r\n6665.18,0.8254963,0.650558,0.270985,0.0603065\r\n6667.78,0.82537615,0.65016,0.27072,0.0601745\r\n6671.3,0.8252593,0.649622,0.270361,0.0599956\r\n6672.64,0.8251541,0.649417,0.270225,0.0599504\r\n6674.44,0.8250431,0.649049,0.270042,0.0598895\r\n6680.09,0.82493035,0.647897,0.269467,0.0596985\r\n6681.9,0.82482095,0.647153,0.269282,0.0596372\r\n6682.51,0.8247366,0.646906,0.269159,0.0596168\r\n6682.72,0.824661,0.646818,0.269115,0.0596113\r\n6683.8,0.82460215,0.646375,0.268894,0.059584\r\n6689.35,0.8245571,0.645526,0.267761,0.0594435\r\n6691.26,0.82452505,0.645235,0.267567,0.0593952\r\n6694.95,0.82449755,0.642191,0.267191,0.0593018\r\n6694.97,0.82448125,0.643713,0.267189,0.0593012\r\n6697.45,0.8244796,0.641673,0.266937,0.0592386\r\n6697.68,0.82448275,0.641481,0.266913,0.0592307\r\n6698.66,0.8244928,0.64067,0.266813,0.0591974\r\n6700.55,0.82452365,0.639995,0.266621,0.0591338\r\n6706.1,0.8245639,0.638007,0.265772,0.0589461\r\n6706.12,0.8246138,0.639149,0.265769,0.0589455\r\n6708,0.82467035,0.636998,0.265481,0.0588817\r\n6708.65,0.8247322,0.636259,0.265348,0.0588598\r\n6708.9,0.82480025,0.635974,0.265297,0.0588471\r\n6709.79,0.8248707,0.634963,0.265116,0.058802\r\n6709.81,0.8249671,0.636486,0.265111,0.0588008\r\n6713.5,0.82507575,0.633442,0.264358,0.0586133\r\n6715.46,0.82520895,0.633044,0.26396,0.0585142\r\n6720.13,0.8253452,0.632089,0.263245,0.0582767\r\n6720.96,0.82548535,0.631921,0.263119,0.0582348\r\n6722.91,0.82561365,0.631119,0.262819,0.0581354\r\n6723.59,0.82574075,0.630842,0.262716,0.058101\r\n6724.67,0.82587105,0.630399,0.262551,0.0580736\r\n6728.38,0.82600805,0.628878,0.261982,0.0579796\r\n6730.37,0.8261421,0.628269,0.261679,0.0579294\r\n6731.35,0.82627935,0.627967,0.261276,0.0579045\r\n6732.1,0.82642335,0.627737,0.260968,0.0578855\r\n6734.08,0.82658355,0.627333,0.260157,0.0578354\r\n6738.53,0.82673015,0.626425,0.259704,0.0577228\r\n6739.55,0.8268921,0.626216,0.2596,0.0576881\r\n6742.58,0.82705,0.624976,0.259292,0.0575858\r\n6743.27,0.82721245,0.624695,0.259222,0.0575626\r\n6745.28,0.8273892,0.623975,0.259018,0.0574947\r\n6749.74,0.82756475,0.622378,0.258107,0.0573439\r\n6750.7,0.82774025,0.622032,0.25791,0.0573195\r\n6750.73,0.8279038,0.623554,0.257905,0.0573189\r\n6752.73,0.82806105,0.621063,0.257497,0.0572682\r\n6753.8,0.8282005,0.619725,0.257387,0.057241\r\n6754.39,0.82833735,0.618988,0.257327,0.057226\r\n6754.42,0.82850505,0.620511,0.257325,0.0572254\r\n6758.11,0.8286916,0.617467,0.256949,0.057132\r\n6760.19,0.8288858,0.617042,0.256737,0.0570793\r\n6761.3,0.82910165,0.616816,0.25651,0.0570512\r\n6764.68,0.8293197,0.616126,0.255821,0.0569657\r\n6765.56,0.82955555,0.615946,0.255641,0.0569359\r\n6767.64,0.82979935,0.61423,0.255216,0.0568655\r\n6768.8,0.83003995,0.613278,0.255039,0.0568265\r\n6769.25,0.8302583,0.612902,0.25497,0.0568111\r\n6769.27,0.83052125,0.614425,0.254966,0.0568103\r\n6772.55,0.83077735,0.612065,0.254465,0.0566996\r\n6772.97,0.83107415,0.611761,0.254401,0.0566853\r\n6775.1,0.83138005,0.611327,0.254075,0.0566134\r\n6775.88,0.83167945,0.611167,0.253915,0.0565869\r\n6776.31,0.8320173,0.61108,0.253828,0.0565761\r\n6780.42,0.83237675,0.610241,0.252989,0.056472\r\n6782.55,0.8327421,0.609806,0.252554,0.0564181\r\n6783.81,0.8331348,0.60955,0.252362,0.0563863\r\n6787.87,0.8335378,0.60872,0.25174,0.0562834\r\n6790.01,0.83394685,0.608393,0.251414,0.0562294\r\n6790.83,0.834322,0.608268,0.251289,0.0562086\r\n6791.3,0.83471695,0.608196,0.251217,0.0561927\r\n6795.33,0.83512225,0.607579,0.2506,0.0560564\r\n6797.46,0.8355068,0.606705,0.250274,0.0559842\r\n6798.79,0.8358843,0.60616,0.250002,0.0559393\r\n6799.04,0.83626345,0.606058,0.249951,0.0559308\r\n6802.03,0.8366528,0.605601,0.249341,0.0558298\r\n6804.91,0.83704575,0.60516,0.248753,0.0556833\r\n6806.28,0.8374405,0.60495,0.248332,0.0556137\r\n6806.5,0.8378071,0.604917,0.248266,0.0556027\r\n6808.63,0.83821745,0.604482,0.247612,0.0554944\r\n6813.78,0.83858735,0.603431,0.246561,0.0552329\r\n6813.95,0.8389738,0.603396,0.246526,0.0552242\r\n6816.09,0.839365,0.60307,0.246091,0.0551159\r\n6816.97,0.8397231,0.602935,0.24573,0.0550711\r\n6817.54,0.84007105,0.602848,0.245497,0.0550712\r\n6819.8,0.84041445,0.602502,0.244569,0.0550715\r\n6821.41,0.84075665,0.602256,0.244323,0.0550718\r\n6827.26,0.84108725,0.601062,0.243429,0.0550726\r\n6828.18,0.84141455,0.600874,0.243241,0.0550728\r\n6828.76,0.84173795,0.600755,0.243121,0.055043\r\n6828.86,0.84200775,0.600735,0.243101,0.0550381\r\n6832.58,0.8422487,0.599594,0.242342,0.0548491\r\n6834.71,0.8424583,0.59916,0.241908,0.0547411\r\n6839.96,0.84266565,0.598087,0.241104,0.0544741\r\n6840.03,0.84283935,0.598073,0.241094,0.0544706\r\n6842.16,0.84298625,0.597747,0.240767,0.0543623\r\n6843.11,0.8431004,0.597602,0.240622,0.054314\r\n6847.49,0.8432015,0.596933,0.239953,0.0541661\r\n6849.62,0.8432731,0.596715,0.239627,0.054094\r\n6851.16,0.8432682,0.596559,0.239314,0.0540421\r\n6854.32,0.84323965,0.596237,0.238668,0.0539352\r\n6857.07,0.8431669,0.595957,0.238106,0.0538655\r\n6858.61,0.84308005,0.5958,0.237475,0.0538265\r\n6858.68,0.84296305,0.595793,0.237447,0.0538248\r\n6860.79,0.8428181,0.594931,0.236585,0.0537715\r\n6862.4,0.84262755,0.594271,0.236338,0.0537308\r\n6866.06,0.8423905,0.593898,0.235777,0.0536379\r\n6868.24,0.842141,0.593676,0.235444,0.0535827\r\n6869.26,0.84185595,0.593572,0.235375,0.053557\r\n6873.52,0.8415705,0.593138,0.235086,0.0534491\r\n6873.59,0.841238,0.593131,0.235082,0.0534474\r\n6879.44,0.84089705,0.592236,0.234685,0.0532992\r\n6880.97,0.8405728,0.592002,0.234607,0.0532605\r\n6881.05,0.84020045,0.591991,0.234603,0.0532586\r\n6884.2,0.83983145,0.59167,0.234443,0.0531787\r\n6884.69,0.83939125,0.59162,0.234418,0.0531788\r\n6892.14,0.8389726,0.590861,0.23404,0.0531799\r\n6892.24,0.8385485,0.590851,0.234035,0.0531799\r\n6894.38,0.83812425,0.590524,0.233926,0.0531803\r\n6895.41,0.8376922,0.590366,0.233856,0.0531804\r\n6899.6,0.8372802,0.589726,0.233572,0.0529678\r\n6899.7,0.83681995,0.589711,0.233566,0.0529629\r\n6905.58,0.83634535,0.588811,0.233167,0.0526641\r\n6907.05,0.83589015,0.588586,0.233067,0.0525893\r\n6907.15,0.83538595,0.58857,0.23306,0.0525841\r\n6910.35,0.83491065,0.588245,0.232843,0.0524217\r\n6910.77,0.83440605,0.588203,0.232815,0.0524076\r\n6916.78,0.83389815,0.58759,0.232407,0.0522044\r\n6918.22,0.8334219,0.587443,0.232309,0.0521555\r\n6918.35,0.8329218,0.58743,0.232301,0.0521513\r\n6921.56,0.8324499,0.586995,0.232083,0.0520429\r\n6925.68,0.8319702,0.586434,0.231804,0.0519385\r\n6927.98,0.8314757,0.586122,0.231648,0.0518803\r\n6929.54,0.83099295,0.58591,0.231569,0.0518408\r\n6933.13,0.83051705,0.584808,0.231386,0.0517498\r\n6933.26,0.8301007,0.584769,0.23138,0.0517467\r\n6936.5,0.8296636,0.584273,0.231215,0.0516646\r\n6940.71,0.82923165,0.583629,0.231001,0.0515579\r\n6942.91,0.82880045,0.583179,0.230889,0.0515022\r\n6944.32,0.8283719,0.582891,0.230794,0.0514665\r\n6948.16,0.8279193,0.582108,0.230533,0.0513693\r\n6951.44,0.82752405,0.581439,0.230311,0.0512864\r\n6951.78,0.82713595,0.581369,0.230288,0.0512748\r\n6954.11,0.8267449,0.580893,0.23013,0.051196\r\n6955.61,0.82639545,0.580587,0.230028,0.0511452\r\n6962.64,0.82605335,0.579512,0.229552,0.0509076\r\n6962.98,0.82575715,0.57946,0.229529,0.0508991\r\n6963.07,0.82546915,0.579446,0.229523,0.0508968\r\n6965.31,0.8251755,0.578989,0.229371,0.05084\r\n6970.52,0.82489885,0.577925,0.229017,0.0507081\r\n6976.51,0.8246523,0.57701,0.228611,0.0505565\r\n6977.59,0.8244288,0.576845,0.228538,0.0505293\r\n6977.92,0.8242212,0.576794,0.228516,0.0505294\r\n6977.98,0.8240503,0.576785,0.228512,0.0505294\r\n6981.69,0.8238801,0.575264,0.22826,0.0505299\r\n6987.71,0.82372645,0.574343,0.227852,0.0505309\r\n6988.8,0.82359425,0.574177,0.227815,0.050531\r\n6989.15,0.82347325,0.574123,0.227803,0.0505221\r\n6992.87,0.8233468,0.573364,0.227678,0.0504279\r\n6996.6,0.82324185,0.572602,0.227551,0.0503335\r\n6998.92,0.8231456,0.572248,0.227473,0.0502749\r\n7003.74,0.82305095,0.571511,0.227351,0.0501528\r\n7004.06,0.82296415,0.571462,0.227343,0.050142\r\n7004.07,0.8227302,0.57146,0.227343,0.0501416\r\n7007.76,0.8225962,0.570959,0.227249,0.0500168\r\n7007.78,0.8223112,0.570956,0.227249,0.0500161\r\n7011.47,0.82192805,0.570454,0.227155,0.0498913\r\n7013.86,0.82150295,0.57013,0.227095,0.0498107\r\n7014.94,0.8210753,0.569982,0.227067,0.049774\r\n7015.17,0.8207313,0.569952,0.227062,0.0497627\r\n7015.19,0.82038635,0.569949,0.227061,0.0497615\r\n7015.25,0.819892,0.569941,0.22706,0.0497587\r\n7018.88,0.8193406,0.569386,0.226968,0.049574\r\n7022.6,0.8187871,0.568817,0.226874,0.049385\r\n7022.7,0.81815955,0.568801,0.226871,0.0493798\r\n7026.31,0.81748105,0.568249,0.22678,0.0491964\r\n7028.8,0.8167804,0.567869,0.226717,0.0490701\r\n7029.88,0.81607675,0.567703,0.226744,0.0490152\r\n7030.03,0.81538575,0.567681,0.226748,0.0490152\r\n7030.16,0.81470265,0.567661,0.226752,0.0490152\r\n7037.61,0.81403595,0.56614,0.226942,0.0490164\r\n7043.75,0.8133703,0.5652,0.2271,0.0490173\r\n7044.83,0.8127047,0.565036,0.2271,0.0490175\r\n7044.98,0.81219765,0.565013,0.2271,0.0490125\r\n7045.07,0.81214095,0.564999,0.2271,0.0490094\r\n7052.61,0.8123125,0.564231,0.227101,0.0487544\r\n7052.63,0.8126584,0.564229,0.227101,0.0487537\r\n7052.65,0.81309585,0.564227,0.227101,0.0487529\r\n7054.96,0.8135641,0.563991,0.227101,0.0486748\r\n7056.03,0.81396085,0.563883,0.227129,0.0486387\r\n7056.26,0.81436305,0.563859,0.227134,0.0486329\r\n7056.41,0.81499115,0.563836,0.227138,0.048629\r\n7056.44,0.81575245,0.563832,0.227139,0.0486284\r\n7056.46,0.81659195,0.563829,0.22714,0.0486279\r\n7056.48,0.81758155,0.563825,0.22714,0.0486273\r\n7060.24,0.81869815,0.56325,0.227236,0.0485321\r\n7060.27,0.819913,0.563247,0.227237,0.0485315\r\n7060.29,0.82120695,0.563244,0.227237,0.048531\r\n7060.31,0.8225699,0.56324,0.227238,0.0485304\r\n7063.72,0.82393995,0.562719,0.227325,0.0484441\r\n7064.07,0.82538605,0.562683,0.227334,0.0484352\r\n7064.09,0.82690825,0.562681,0.227335,0.0484346\r\n7064.12,0.8285066,0.562678,0.227335,0.0484341\r\n7067.88,0.8301747,0.562296,0.227432,0.0483389\r\n7067.9,0.83139245,0.562293,0.227432,0.0483383\r\n7067.92,0.8326102,0.562291,0.227433,0.0483377\r\n7067.94,0.8337918,0.562289,0.227433,0.0483372\r\n7069.92,0.8349481,0.562088,0.227484,0.0482872\r\n7070.98,0.8360975,0.56198,0.22752,0.0482604\r\n7071.7,0.8373184,0.561906,0.227545,0.048242\r\n7071.73,0.8386123,0.561903,0.227546,0.0482414\r\n7071.75,0.83989415,0.561901,0.227546,0.0482408\r\n7074.91,0.84111195,0.561579,0.227654,0.0481608\r\n7075.51,0.84232975,0.561488,0.227675,0.0481457\r\n7075.53,0.84354755,0.561484,0.227675,0.0481451\r\n7075.55,0.84476535,0.561481,0.227676,0.0481445\r\n7075.58,0.84598315,0.561477,0.227677,0.0481439\r\n7079.34,0.84720095,0.560902,0.227805,0.0480488\r\n7079.36,0.8484259,0.560899,0.227806,0.0480482\r\n7079.38,0.8497198,0.560895,0.227807,0.0480476\r\n7079.4,0.8509726,0.560892,0.227807,0.048047\r\n7081.13,0.8521743,0.560627,0.227866,0.0480033\r\n7082.37,0.85331595,0.560439,0.227929,0.047972\r\n7083.16,0.8544576,0.560276,0.22797,0.0479519\r\n7083.19,0.85559925,0.560272,0.227971,0.0479513\r\n7083.21,0.85674095,0.560267,0.227972,0.0479507\r\n7083.23,0.8579188,0.560262,0.227973,0.0479501\r\n7083.26,0.8591513,0.560258,0.227974,0.0479496\r\n7083.28,0.86042245,0.560253,0.227976,0.047949\r\n7085.92,0.8616402,0.559714,0.22811,0.0478822\r\n7087.04,0.86285795,0.559486,0.228167,0.0478443\r\n7087.06,0.86408775,0.559481,0.228169,0.0478435\r\n7087.08,0.8653816,0.559476,0.22817,0.0478428\r\n7087.11,0.86667545,0.559472,0.228171,0.047842\r\n7087.13,0.8682358,0.559467,0.228172,0.0478412\r\n7089.82,0.8701937,0.558918,0.228309,0.0477503\r\n7093.54,0.87224865,0.557777,0.228499,0.0476245\r\n7094.9,0.8743036,0.557499,0.228569,0.0475786\r\n7094.92,0.8763585,0.557495,0.22857,0.0475778\r\n7094.94,0.87836055,0.55749,0.228571,0.047577\r\n7096.09,0.88034845,0.557255,0.22863,0.0475382\r\n7097.12,0.88234335,0.557045,0.228665,0.0475034\r\n7098.7,0.8843983,0.556723,0.228719,0.0475036\r\n7098.73,0.8866117,0.556718,0.228719,0.0475036\r\n7100.99,0.88931355,0.556256,0.228796,0.047504\r\n7106.44,0.8919569,0.555423,0.228982,0.0475048\r\n7107.31,0.8945472,0.555289,0.229012,0.0475049\r\n7108.45,0.89709685,0.555115,0.229089,0.0475051\r\n7110.2,0.8996692,0.554758,0.229208,0.0475054\r\n7110.22,0.9022563,0.554753,0.22921,0.0475054\r\n7112.07,0.904806,0.554376,0.229335,0.0475056\r\n7113.98,0.9073184,0.553986,0.229465,0.0474573\r\n7115.9,0.90979195,0.553595,0.229596,0.0474087\r\n7117.74,0.91220545,0.553218,0.229721,0.0473621\r\n7118.53,0.91433645,0.553057,0.229775,0.0473421\r\n7121.5,0.91584935,0.552451,0.229926,0.0472669\r\n7123.35,0.9170482,0.552074,0.23002,0.0472201\r\n7125.15,0.91824705,0.551521,0.230112,0.0471745\r\n7125.18,0.9194459,0.551514,0.230114,0.0471739\r\n7125.2,0.9206976,0.551507,0.230115,0.0471733\r\n7125.22,0.9216216,0.5515,0.230116,0.0471728\r\n7127.01,0.9225039,0.55095,0.230207,0.0471274\r\n7127.07,0.9230178,0.550933,0.23021,0.0471255\r\n7128.78,0.92337325,0.550584,0.230297,0.0470678\r\n7128.8,0.92324025,0.55058,0.230298,0.047067\r\n7128.82,0.9231657,0.550575,0.2303,0.0470663\r\n7128.84,0.9231442,0.55057,0.230301,0.0470655\r\n7128.87,0.92255455,0.550566,0.230302,0.0470647\r\n7132.44,0.92196495,0.549835,0.230484,0.0469438\r\n7132.47,0.921414,0.549831,0.230486,0.046943\r\n7132.49,0.9207192,0.549826,0.230487,0.0469422\r\n7133.49,0.9198775,0.549622,0.230538,0.0469084\r\n7134.52,0.91861025,0.549412,0.230573,0.0468736\r\n7138.22,0.9169321,0.548846,0.230699,0.0467486\r\n7141.98,0.9150487,0.548271,0.230827,0.0466534\r\n7143.36,0.91338585,0.547989,0.230874,0.0466184\r\n7143.38,0.91194,0.547984,0.230875,0.0466179\r\n7143.4,0.9102529,0.54798,0.230876,0.0466173\r\n7144.71,0.90819825,0.547714,0.23092,0.0465843\r\n7147.01,0.90614365,0.547245,0.231037,0.0465261\r\n7147.03,0.904472,0.54724,0.231038,0.0465256\r\n7147.05,0.90288615,0.547236,0.23104,0.046525\r\n7147.07,0.90118925,0.547231,0.231041,0.0465244\r\n7149.43,0.89921075,0.54675,0.231161,0.0464648\r\n7150.63,0.89723225,0.546259,0.231222,0.0464344\r\n7150.65,0.89525375,0.546249,0.231223,0.0464338\r\n7150.67,0.89327525,0.54624,0.231225,0.0464333\r\n7150.7,0.89190555,0.546231,0.231226,0.0464327\r\n7150.72,0.89004475,0.546221,0.231227,0.0464321\r\n7153.14,0.8881049,0.545229,0.23135,0.0463708\r\n7153.16,0.8860793,0.545226,0.231351,0.0463703\r\n7154.27,0.8842379,0.545056,0.231408,0.0463421\r\n7154.3,0.88286085,0.545052,0.231409,0.0463415\r\n7154.32,0.88195475,0.545049,0.231411,0.046341\r\n7154.34,0.8806611,0.545045,0.231412,0.0463404\r\n7157.92,0.87936745,0.544498,0.231594,0.0462498\r\n7157.94,0.8780738,0.544495,0.231595,0.0462493\r\n7157.97,0.8770214,0.544491,0.231597,0.0462487\r\n7157.99,0.87608425,0.544488,0.231598,0.0462481\r\n7159.67,0.874973,0.544231,0.231683,0.0462056\r\n7160.6,0.87367935,0.544088,0.231715,0.046182\r\n7161.61,0.8723857,0.543882,0.23175,0.0461564\r\n7161.63,0.8705222,0.543877,0.23175,0.0461558\r\n7168.05,0.86885645,0.542567,0.231969,0.0459934\r\n7168.1,0.86701105,0.54256,0.231971,0.0459921\r\n7168.78,0.86516565,0.542455,0.231994,0.0459922\r\n7168.81,0.86332025,0.542452,0.231995,0.0459922\r\n7168.83,0.86109305,0.542448,0.231996,0.0459922\r\n7170.88,0.85905385,0.542134,0.232066,0.0459925\r\n7172.41,0.8570937,0.541901,0.232118,0.0459928\r\n7172.43,0.85540055,0.541898,0.232118,0.0459928\r\n7172.45,0.8537074,0.541894,0.232119,0.0459928\r\n7172.48,0.85201425,0.541891,0.23212,0.0459928\r\n7172.5,0.84966155,0.541887,0.232121,0.0459928\r\n7175.51,0.8477401,0.541427,0.232223,0.0459932\r\n7176.03,0.84581865,0.54132,0.232241,0.0459933\r\n7176.05,0.8438972,0.541316,0.232242,0.0459933\r\n7176.08,0.84197575,0.541311,0.232243,0.0459933\r\n7176.1,0.8403066,0.541306,0.232243,0.0459933\r\n7176.12,0.83818895,0.541302,0.232244,0.0459933\r\n7179.31,0.83611525,0.54065,0.232353,0.0459938\r\n7179.68,0.83404155,0.540576,0.232365,0.0459846\r\n7179.7,0.8330171,0.540571,0.232366,0.045984\r\n7179.72,0.8320765,0.540567,0.232367,0.0459834\r\n7179.74,0.83088885,0.540562,0.232368,0.0459829\r\n7182.1,0.8294972,0.540081,0.232448,0.0459233\r\n7182.96,0.82797525,0.539906,0.232492,0.0459015\r\n7183.32,0.8268351,0.53985,0.23251,0.0458923\r\n7183.35,0.825998,0.539847,0.232511,0.0458917\r\n7183.37,0.8251609,0.539843,0.232513,0.0458911\r\n7183.39,0.82379115,0.53984,0.232514,0.0458906\r\n7186.99,0.8224214,0.539289,0.232697,0.0457994\r\n7187.01,0.82105165,0.539286,0.232699,0.0457989\r\n7187.04,0.8200551,0.539282,0.2327,0.0457983\r\n7190.42,0.81914195,0.538766,0.232872,0.0457128\r\n7190.71,0.81815275,0.538721,0.232887,0.0457054\r\n7194.25,0.81708745,0.538178,0.233068,0.0456156\r\n7194.44,0.81598585,0.53815,0.233078,0.0456108\r\n7197.06,0.8148205,0.53775,0.233211,0.0455446\r\n7197.87,0.81420655,0.537625,0.233211,0.045524\r\n7198.2,0.81372735,0.537559,0.233211,0.0455158\r\n7205.32,0.8131758,0.536104,0.233212,0.0453354\r\n7205.69,0.81260035,0.535991,0.233212,0.045326\r\n7208.27,0.81196445,0.535201,0.233213,0.0452608\r\n7209.04,0.81168225,0.534963,0.233233,0.0452412\r\n7209.2,0.81153315,0.5349,0.233237,0.0452373\r\n7209.45,0.81151435,0.534796,0.233243,0.0452287\r\n7213.21,0.8114956,0.533255,0.233339,0.0451016\r\n7216.97,0.81144965,0.531715,0.233436,0.0449745\r\n7220.19,0.81133085,0.530399,0.233518,0.0448658\r\n7220.4,0.81167355,0.530366,0.233523,0.0448585\r\n7220.73,0.8119655,0.530316,0.233532,0.0448503\r\n7223.22,0.81222625,0.529934,0.233596,0.0447871\r\n7227.64,0.81270315,0.529258,0.233709,0.0446752\r\n7228.22,0.8132331,0.52914,0.233724,0.0446605\r\n7235.09,0.81376425,0.527737,0.233899,0.0444866\r\n7235.34,0.81430985,0.527699,0.233906,0.0444803\r\n7238.18,0.81482215,0.527266,0.233978,0.0444807\r\n7239.45,0.8153457,0.527071,0.234022,0.0444809\r\n7242.55,0.8158978,0.526597,0.234127,0.0444814\r\n7246.94,0.8163962,0.525701,0.234277,0.044482\r\n7249.39,0.8168955,0.5252,0.234361,0.0444824\r\n7250,0.8173448,0.525076,0.234376,0.0444825\r\n7250.29,0.81781005,0.525032,0.234384,0.0444825\r\n7254.43,0.8182953,0.524398,0.23449,0.0443426\r\n7257.46,0.81880615,0.523936,0.234567,0.0442403\r\n7261.5,0.81924415,0.523111,0.23467,0.0441037\r\n7261.92,0.8196224,0.523024,0.234681,0.0441038\r\n7264.35,0.8199556,0.52253,0.234743,0.0441042\r\n7264.91,0.8203107,0.522415,0.234763,0.0441042\r\n7268.62,0.8206861,0.520893,0.234889,0.0441048\r\n7272.34,0.82104165,0.519752,0.235016,0.0441054\r\n7273.16,0.8213605,0.519586,0.235044,0.0441055\r\n7275.56,0.82167645,0.519095,0.235126,0.0441059\r\n7276.44,0.8219688,0.518915,0.235193,0.044106\r\n7279.79,0.82227545,0.518231,0.235449,0.0440212\r\n7284.38,0.82258955,0.51753,0.2358,0.0439051\r\n7287.25,0.82292215,0.517091,0.236019,0.0438324\r\n7290.53,0.8232377,0.516757,0.236269,0.0437495\r\n7291.39,0.82352345,0.51667,0.236299,0.0437278\r\n7291.87,0.8238573,0.516621,0.236315,0.0437155\r\n7298.44,0.82419585,0.515951,0.236539,0.0435491\r\n7299.37,0.8245443,0.51592,0.236571,0.0435258\r\n7301.74,0.8248845,0.51584,0.236652,0.0434656\r\n7306.33,0.8252185,0.515685,0.236963,0.0433495\r\n7309.65,0.8255265,0.515572,0.237189,0.0432372\r\n7310.59,0.82584935,0.515524,0.237253,0.0432053\r\n7312.97,0.82619065,0.515404,0.237414,0.0431251\r\n7317.53,0.8265323,0.515172,0.237648,0.0429707\r\n7318.08,0.8268984,0.515144,0.237676,0.0429708\r\n7324.59,0.8272468,0.514813,0.238007,0.0429718\r\n7325.58,0.82761355,0.514746,0.238058,0.0429719\r\n7327.92,0.8280184,0.514587,0.238178,0.0429723\r\n7332.48,0.82842625,0.514278,0.238333,0.042973\r\n7333.07,0.8288434,0.514238,0.238353,0.0429731\r\n7335.79,0.8292706,0.514054,0.238446,0.0429735\r\n7339.14,0.82968675,0.513884,0.23856,0.042974\r\n7340.56,0.83014765,0.513811,0.238657,0.0429742\r\n7347.43,0.8306084,0.513463,0.239123,0.0429753\r\n7348.06,0.8311005,0.513431,0.239166,0.0429593\r\n7350.36,0.83153125,0.513314,0.239323,0.042901\r\n7350.72,0.83196925,0.513295,0.239335,0.0428919\r\n7351.81,0.8324713,0.513296,0.239372,0.0428642\r\n7359.31,0.83299595,0.513297,0.239628,0.0426745\r\n7361.58,0.8335197,0.513297,0.239705,0.042617\r\n7362.37,0.83405285,0.513297,0.239765,0.042597\r\n7363.07,0.8345877,0.513297,0.239819,0.0425734\r\n7365.67,0.835125,0.513298,0.240018,0.0424855\r\n7370.56,0.83568875,0.513174,0.240391,0.0423201\r\n7373.58,0.83623305,0.513098,0.240622,0.0422182\r\n7376.54,0.8367852,0.513022,0.240849,0.0422187\r\n7378.06,0.83733295,0.512984,0.2409,0.0422189\r\n7380.61,0.83787205,0.512919,0.240987,0.0422193\r\n7385.55,0.83844995,0.513422,0.241156,0.04222\r\n7387.76,0.8390226,0.513647,0.241231,0.0422204\r\n7388.52,0.83958555,0.513725,0.241251,0.0422205\r\n7389.31,0.8401889,0.513805,0.241271,0.0422006\r\n7391.84,0.8407474,0.514063,0.241335,0.0421365\r\n7393.07,0.84136305,0.514188,0.241367,0.0421054\r\n7399.32,0.84195625,0.514825,0.241527,0.0419471\r\n7400.56,0.8425767,0.514951,0.241559,0.0419157\r\n7402.71,0.84319195,0.51517,0.241614,0.0418612\r\n7403.47,0.84374405,0.515246,0.241665,0.0418422\r\n7404.32,0.84431355,0.515333,0.241723,0.0418424\r\n7410.55,0.84487985,0.515968,0.242147,0.0418433\r\n7411.82,0.8454749,0.516161,0.242232,0.0418435\r\n7413.94,0.84608335,0.516484,0.242377,0.0418438\r\n7415.57,0.84667955,0.516734,0.242418,0.0418441\r\n7418.04,0.8472489,0.517111,0.242482,0.0418445\r\n7418.41,0.8478354,0.517167,0.242491,0.0418445\r\n7423.07,0.84844395,0.517878,0.24261,0.0417266\r\n7425.53,0.8490527,0.518254,0.242673,0.0416642\r\n7426.83,0.8496448,0.518776,0.242706,0.0416316\r\n7428.89,0.8501962,0.519613,0.242759,0.0415793\r\n7429.29,0.8507734,0.519776,0.242759,0.0415691\r\n7433.36,0.8513445,0.520601,0.24276,0.0414663\r\n7434.32,0.8518832,0.520798,0.24276,0.0414664\r\n7436.79,0.85242225,0.521299,0.242761,0.0414668\r\n7440.1,0.85290115,0.52264,0.242761,0.0414673\r\n7440.55,0.85336765,0.522822,0.242792,0.0414674\r\n7440.57,0.8538109,0.524344,0.242793,0.0414674\r\n7441.81,0.8543189,0.524533,0.242877,0.0414676\r\n7445.57,0.8548392,0.525107,0.243133,0.0414681\r\n7448.06,0.85529855,0.525487,0.243302,0.0414685\r\n7448.3,0.8557758,0.525584,0.243319,0.0414685\r\n7451.32,0.8562152,0.526807,0.243524,0.041469\r\n7451.82,0.85663405,0.527009,0.243558,0.0414691\r\n7453.07,0.85707875,0.527514,0.243642,0.0414693\r\n7455.58,0.8575201,0.528532,0.243813,0.0414696\r\n7455.61,0.85793895,0.530054,0.243815,0.0414697\r\n7456.83,0.8583408,0.530303,0.243898,0.0414698\r\n7459.51,0.85874615,0.530848,0.24408,0.0414702\r\n7462.55,0.85913475,0.531464,0.244287,0.0413935\r\n7463.1,0.8595285,0.531577,0.244343,0.0413794\r\n7464.32,0.8599324,0.531948,0.244467,0.0413486\r\n7466.86,0.86035125,0.532719,0.244726,0.0412844\r\n7468.08,0.8608026,0.533215,0.24485,0.0412534\r\n7470.62,0.8612535,0.534242,0.245108,0.0411892\r\n7471.84,0.8617206,0.534737,0.245233,0.0411582\r\n7473.77,0.86219365,0.535521,0.24543,0.0411093\r\n7474.38,0.86265805,0.535765,0.245491,0.041094\r\n7474.46,0.8630959,0.535781,0.245499,0.041092\r\n7479.34,0.86354705,0.536773,0.245996,0.0409685\r\n7481.87,0.8640143,0.537288,0.246255,0.0409042\r\n7483.09,0.86448955,0.537781,0.246378,0.0408734\r\n7485,0.8649725,0.538555,0.246573,0.0408251\r\n7485.63,0.8654431,0.538811,0.246669,0.0408091\r\n7485.65,0.86588715,0.539952,0.246672,0.0408086\r\n7486.85,0.8663566,0.540438,0.246855,0.0407783\r\n7489.4,0.8668014,0.54147,0.247244,0.0407138\r\n7489.41,0.86725705,0.541475,0.247245,0.0407138\r\n7492.49,0.86769645,0.542723,0.247716,0.0407142\r\n7493.17,0.86814355,0.542997,0.247785,0.0407143\r\n7494.35,0.8686011,0.543474,0.247905,0.0407145\r\n7496.93,0.8690359,0.54452,0.248167,0.0407149\r\n7500.69,0.86944615,0.546043,0.24855,0.0407155\r\n7500.71,0.8698136,0.547565,0.248553,0.0407155\r\n7501.84,0.8701944,0.548021,0.248667,0.0407157\r\n7503.72,0.87056825,0.548783,0.248859,0.0407159\r\n7504.35,0.8709385,0.549037,0.248923,0.040716\r\n7504.47,0.87133075,0.549087,0.248935,0.0407129\r\n7505.6,0.87172675,0.549544,0.24905,0.0406844\r\n7508.23,0.87210695,0.55061,0.249318,0.0406177\r\n7509.36,0.8724954,0.551066,0.249433,0.0405892\r\n7511.99,0.8728752,0.552133,0.249701,0.0405225\r\n7514.95,0.87326155,0.552584,0.250002,0.0404477\r\n7516.85,0.87368465,0.552875,0.250196,0.0403994\r\n7519.29,0.8740863,0.553246,0.250444,0.0403378\r\n7519.48,0.87447925,0.553275,0.250464,0.0403378\r\n7520.61,0.8748988,0.553732,0.250579,0.040338\r\n7523.24,0.87531685,0.554798,0.250847,0.0403384\r\n7526.18,0.8757476,0.555394,0.251146,0.0403388\r\n7528.1,0.8762149,0.555786,0.251277,0.0403391\r\n7530.5,0.87666735,0.556273,0.251439,0.0403395\r\n7530.74,0.87711395,0.556321,0.251456,0.0403395\r\n7531.86,0.87758045,0.556777,0.251532,0.0403397\r\n7534.5,0.8780605,0.557844,0.251711,0.0403401\r\n7537.4,0.87851735,0.558286,0.251908,0.0403405\r\n7539.35,0.87901485,0.558585,0.252041,0.0403408\r\n7541.99,0.8795326,0.558987,0.25222,0.0403412\r\n7543.11,0.88005125,0.559443,0.252297,0.0403414\r\n7545.45,0.8805196,0.560387,0.252455,0.0403418\r\n7545.75,0.88099435,0.560509,0.252476,0.0403341\r\n7548.62,0.8814513,0.561383,0.252671,0.0402614\r\n7549.5,0.88188945,0.561651,0.252701,0.0402391\r\n7550.61,0.8823285,0.562101,0.252739,0.040211\r\n7553.26,0.88275915,0.563174,0.252829,0.0401439\r\n7557.02,0.8831871,0.564697,0.252957,0.0400488\r\n7557.04,0.88360595,0.566219,0.252958,0.0400482\r\n7558.1,0.8840157,0.566647,0.252994,0.0400214\r\n7559.84,0.88440125,0.56735,0.253053,0.0399775\r\n7560.39,0.8847714,0.567573,0.253109,0.0399635\r\n7560.8,0.8850933,0.567741,0.253152,0.0399636\r\n7560.83,0.88544855,0.569263,0.253154,0.0399636\r\n7564.59,0.88579125,0.570786,0.253537,0.0399642\r\n7565.6,0.88615555,0.570992,0.25364,0.0399643\r\n7571.06,0.8865055,0.572102,0.254197,0.0399652\r\n7572.08,0.8868407,0.572309,0.254231,0.0399653\r\n7572.11,0.8871301,0.573831,0.254232,0.0399653\r\n7573.09,0.88741955,0.57423,0.254266,0.0399655\r\n7575.34,0.88769065,0.57514,0.254342,0.0399658\r\n7575.86,0.88799685,0.575354,0.25436,0.0399659\r\n7579.62,0.88828365,0.576496,0.254488,0.0399665\r\n7580.59,0.88857445,0.576888,0.254521,0.0399666\r\n7582.28,0.88886235,0.577574,0.254579,0.0399669\r\n7583.38,0.8891587,0.578019,0.254635,0.039967\r\n7587.14,0.88943355,0.579541,0.254827,0.0399676\r\n7588.08,0.8897157,0.579922,0.254874,0.0399677\r\n7590.28,0.8899919,0.580815,0.254987,0.0399681\r\n7590.9,0.8902505,0.581064,0.255018,0.0399682\r\n7590.92,0.8905036,0.582586,0.25502,0.0399682\r\n7590.94,0.89076525,0.584108,0.255021,0.0399682\r\n7594.7,0.89102675,0.585631,0.255213,0.0399688\r\n7594.73,0.8912501,0.587153,0.255214,0.0399688\r\n7594.75,0.8914717,0.588675,0.255215,0.0399688\r\n7597.24,0.8916559,0.589684,0.255342,0.0399691\r\n7598.51,0.89183245,0.590198,0.255471,0.0399693\r\n7598.53,0.8920114,0.59172,0.255474,0.0399693\r\n7599.3,0.89215995,0.592031,0.255552,0.0399695\r\n7602.29,0.8922855,0.593242,0.255856,0.0399699\r\n7602.31,0.89238335,0.594764,0.255859,0.0399699\r\n7605.23,0.89243665,0.595946,0.256156,0.0399704\r\n7606.07,0.89247995,0.596287,0.256241,0.0399705\r\n7606.1,0.89251015,0.597809,0.256244,0.0399705\r\n7606.12,0.8925271,0.599331,0.256246,0.0399705\r\n7606.76,0.8924979,0.599589,0.256311,0.0399706\r\n7608.47,0.89244715,0.600283,0.256485,0.0399708\r\n7609.88,0.89238865,0.600854,0.2567,0.0399711\r\n7609.9,0.8922897,0.602376,0.256704,0.0399711\r\n7613.66,0.8921851,0.603899,0.257277,0.0399716\r\n7614.21,0.89205355,0.60412,0.257361,0.0399717\r\n7615.96,0.89188685,0.604829,0.257628,0.039972\r\n7617.42,0.89171975,0.605421,0.257777,0.0399722\r\n7617.44,0.8915451,0.606563,0.257779,0.0399722\r\n7617.93,0.891339,0.606761,0.257828,0.0399723\r\n7620.18,0.8911182,0.607673,0.258058,0.0399726\r\n7621.2,0.8908589,0.608085,0.258161,0.0399728\r\n7624.96,0.8905968,0.609608,0.258544,0.0399734\r\n7624.98,0.8903535,0.61113,0.258547,0.0399734\r\n7625.38,0.8900733,0.611291,0.258587,0.0399734\r\n7627.19,0.88978365,0.612024,0.258771,0.0399737\r\n7628.74,0.8894932,0.612653,0.258929,0.0399739\r\n7629.09,0.8890625,0.612796,0.258965,0.039974\r\n7632.5,0.888631,0.614175,0.259312,0.0399745\r\n7632.52,0.88820345,0.615697,0.259315,0.0399745\r\n7632.54,0.8877835,0.61722,0.259317,0.0399745\r\n7632.78,0.88745405,0.617316,0.259341,0.0399745\r\n7632.81,0.8870292,0.617326,0.259343,0.0399746\r\n7635.13,0.88659415,0.618265,0.25958,0.0399749\r\n7636.3,0.8861638,0.618742,0.2597,0.0399351\r\n7636.33,0.88575935,0.620264,0.259702,0.0399343\r\n7636.35,0.88537635,0.621786,0.259704,0.0399336\r\n7636.37,0.8849885,0.623308,0.259707,0.0399328\r\n7636.5,0.8845689,0.623359,0.259719,0.0399286\r\n7638.41,0.88413705,0.624135,0.259915,0.0398638\r\n7640.13,0.88371515,0.624831,0.260089,0.0398057\r\n7640.15,0.88333115,0.626353,0.260092,0.0398049\r\n7640.18,0.88294665,0.627875,0.260094,0.0398042\r\n7640.21,0.8824902,0.627889,0.260098,0.039803\r\n7643.93,0.8820706,0.629393,0.260476,0.0396774\r\n7643.94,0.88168245,0.629398,0.260477,0.039677\r\n7643.96,0.88127175,0.63092,0.260479,0.0396763\r\n7645.9,0.8809949,0.631705,0.260677,0.0396107\r\n7646.33,0.88069885,0.631881,0.260721,0.0395961\r\n7647.65,0.88040225,0.632413,0.260854,0.0395963\r\n7647.72,0.880115,0.632443,0.260862,0.0395963\r\n7647.74,0.8797512,0.633965,0.260864,0.0395963\r\n7647.77,0.8794826,0.635487,0.260867,0.0395963\r\n7647.79,0.87926205,0.637009,0.260869,0.0395963\r\n7647.81,0.8789697,0.638531,0.260871,0.0395963\r\n7651.36,0.87866965,0.639968,0.261233,0.0395969\r\n7651.57,0.8783696,0.640054,0.261254,0.0395969\r\n7651.59,0.87807375,0.641576,0.261256,0.0395969\r\n7651.62,0.87781625,0.643098,0.261259,0.0395969\r\n7651.64,0.87745325,0.64462,0.261261,0.0395969\r\n7655.05,0.8771668,0.646002,0.261609,0.0395974\r\n7655.07,0.8768675,0.646011,0.261611,0.0395974\r\n7655.4,0.87656795,0.646142,0.261644,0.0395975\r\n7655.42,0.8762741,0.647664,0.261646,0.0395975\r\n7657.13,0.8759129,0.648356,0.26182,0.0395977\r\n7658.77,0.87554355,0.649019,0.261931,0.039598\r\n7659.18,0.8752035,0.649187,0.26196,0.039598\r\n7659.2,0.8748275,0.650709,0.261961,0.0395981\r\n7661.28,0.8744471,0.65155,0.262102,0.0395984\r\n7662.48,0.8740583,0.652036,0.262184,0.0395986\r\n7662.96,0.8736695,0.652232,0.262217,0.0395986\r\n7662.99,0.87321545,0.653754,0.262218,0.0395986\r\n7666.19,0.87275055,0.655053,0.262436,0.0395991\r\n7666.75,0.8722857,0.655277,0.262474,0.0395992\r\n7666.77,0.87186115,0.656799,0.262475,0.0395992\r\n7668.35,0.87141335,0.657439,0.262583,0.0395994\r\n7669.91,0.8709538,0.65807,0.262623,0.0395997\r\n7670.53,0.8704318,0.658321,0.262638,0.0395998\r\n7673.62,0.8698972,0.659263,0.262718,0.0396002\r\n7674.28,0.86946345,0.659463,0.262735,0.0396003\r\n7676.23,0.8689309,0.660251,0.262784,0.0396006\r\n7677.34,0.86840085,0.6607,0.262813,0.0396008\r\n7678.04,0.86782555,0.660986,0.262831,0.0396009\r\n7681.06,0.86730535,0.662207,0.262908,0.0396014\r\n7681.8,0.8668219,0.662509,0.262927,0.0396015\r\n7683.3,0.8663169,0.663117,0.262965,0.0396017\r\n7684.77,0.86579615,0.663711,0.263016,0.0396019\r\n7685.56,0.8652581,0.664031,0.263042,0.0396021\r\n7688.48,0.86472745,0.665215,0.263142,0.0396025\r\n7689.32,0.86416875,0.665554,0.263171,0.0396026\r\n7691.17,0.86358955,0.666305,0.263234,0.0396029\r\n7692.2,0.863058,0.66672,0.263269,0.0396031\r\n7693.08,0.8625373,0.667077,0.263299,0.0396032\r\n7693.1,0.86198805,0.668599,0.263299,0.0396032\r\n7694.52,0.86144265,0.669172,0.263348,0.0396034\r\n7695.91,0.8609097,0.669737,0.263383,0.0396036\r\n7696.86,0.86038895,0.670121,0.263408,0.0396038\r\n7696.89,0.85993115,0.671643,0.263408,0.0396038\r\n7696.91,0.8594864,0.673165,0.263409,0.0396038\r\n7696.93,0.85902625,0.674688,0.263409,0.0396038\r\n7699.63,0.85856705,0.675779,0.263478,0.0396042\r\n7700.69,0.8581182,0.67621,0.263506,0.0396044\r\n7700.71,0.85766175,0.677732,0.263506,0.0396044\r\n7703.34,0.85719745,0.678532,0.263574,0.0396048\r\n7704.47,0.85673005,0.678874,0.263602,0.0396049\r\n7706.12,0.8562736,0.679545,0.263645,0.0396052\r\n7707.06,0.8558244,0.679922,0.263669,0.0396053\r\n7708.23,0.85542555,0.680397,0.263699,0.0396055\r\n7709.47,0.8550246,0.680901,0.263731,0.0396057\r\n7711.99,0.8546423,0.68192,0.263795,0.0396061\r\n7714.51,0.8542557,0.682942,0.26386,0.0396065\r\n7715.75,0.85383635,0.683442,0.263891,0.0396067\r\n7718.23,0.85341315,0.684447,0.263955,0.039607\r\n7719.51,0.8530136,0.684965,0.263987,0.0396072\r\n7721.07,0.85263125,0.685204,0.264027,0.0396075\r\n7724.43,0.85225965,0.685715,0.264113,0.039608\r\n7727,0.8518803,0.686108,0.264201,0.0396084\r\n7729.43,0.851495,0.686601,0.264284,0.0396087\r\n7734.49,0.8511087,0.687631,0.264456,0.0396095\r\n7735.64,0.85077715,0.687806,0.264496,0.0396097\r\n7736.02,0.8504598,0.687863,0.264508,0.0396097\r\n7741.98,0.85013995,0.688774,0.264712,0.0396107\r\n7744.37,0.8498739,0.688935,0.264793,0.039611\r\n7746.86,0.849631,0.689105,0.264878,0.0396114\r\n7750.97,0.849422,0.689384,0.265087,0.039612\r\n7753.21,0.84923215,0.689536,0.265202,0.0396124\r\n7759.32,0.84905445,0.689848,0.265513,0.0396133\r\n7761.82,0.8488915,0.689976,0.265641,0.0396137\r\n7765.91,0.8487507,0.690185,0.265781,0.0396143\r\n7768.17,0.8486354,0.6903,0.265857,0.0396146\r\n7773.03,0.848547,0.690424,0.266023,0.0396154\r\n7774.27,0.84850945,0.690456,0.266086,0.0396156\r\n7780.86,0.8484762,0.690625,0.266423,0.0396166\r\n7783.12,0.84844835,0.690682,0.266538,0.0396744\r\n7787.99,0.84843185,0.690683,0.266787,0.0397992\r\n7789.22,0.8484241,0.690683,0.266828,0.0398305\r\n7794.33,0.8484246,0.690684,0.267003,0.0399614\r\n7795.81,0.84843155,0.690533,0.267053,0.0399994\r\n7799.21,0.84843995,0.690187,0.267169,0.0399999\r\n7804.16,0.8484489,0.689683,0.267422,0.0400006\r\n7805.52,0.8484654,0.689544,0.267491,0.0400008\r\n7810.76,0.84848495,0.688832,0.267758,0.0400016\r\n7814.17,0.8485045,0.688369,0.267932,0.0400889\r\n7816.71,0.84852405,0.688024,0.268019,0.0401539\r\n7819.11,0.84855005,0.687657,0.268101,0.0402154\r\n7824.17,0.84857755,0.686883,0.268273,0.0403448\r\n7825.39,0.8486023,0.686697,0.268314,0.040376\r\n7825.72,0.84862935,0.686647,0.268337,0.0403844\r\n7831.63,0.8486566,0.685743,0.268738,0.0405858\r\n7834.07,0.8486755,0.684743,0.268904,0.040669\r\n7835.34,0.84869065,0.684222,0.268991,0.0407124\r\n7836.61,0.8487052,0.684028,0.269077,0.0407556\r\n7836.93,0.84870985,0.683978,0.269099,0.0407667\r\n7842.8,0.8487103,0.683081,0.269498,0.0409168\r\n7845.27,0.84871075,0.682068,0.269666,0.0409801\r\n7846.51,0.84871115,0.68156,0.26975,0.0410119\r\n7847.83,0.84871155,0.681019,0.26984,0.0410456\r\n7850.22,0.84871195,0.680038,0.270084,0.0411069\r\n7851.89,0.84871235,0.679699,0.270253,0.0411494\r\n7857.68,0.84871275,0.678517,0.270842,0.0413468\r\n7859.06,0.84871315,0.678093,0.270983,0.0413939\r\n7860.22,0.848712,0.677737,0.271101,0.0414334\r\n7861.4,0.8487087,0.677376,0.271221,0.0414735\r\n7863.1,0.8487028,0.676677,0.271395,0.0415317\r\n7865.11,0.84868575,0.675855,0.271599,0.0416341\r\n7868.82,0.8486653,0.674333,0.271978,0.0418236\r\n7870.29,0.84863825,0.674035,0.272126,0.0418982\r\n7875.16,0.8486008,0.67304,0.27287,0.042147\r\n7876.28,0.84855645,0.672812,0.27304,0.0422039\r\n7877.78,0.8485135,0.672197,0.273269,0.0422805\r\n7878.06,0.8484678,0.672081,0.273312,0.042295\r\n7879.99,0.84840925,0.671291,0.273607,0.042426\r\n7885.27,0.84835515,0.6694,0.274412,0.0427848\r\n7887.42,0.84830515,0.668628,0.274631,0.0429313\r\n7887.45,0.84825045,0.67015,0.274634,0.0429329\r\n7889.28,0.8481937,0.667864,0.274821,0.0430577\r\n7890.1,0.8481352,0.666852,0.274903,0.0430854\r\n7891.11,0.84807665,0.665585,0.275007,0.0431201\r\n7891.14,0.84800905,0.667107,0.275009,0.0431209\r\n7896.5,0.84793785,0.665188,0.275555,0.0433035\r\n7898.57,0.8478634,0.664444,0.275766,0.0433742\r\n7900.5,0.8477891,0.662854,0.275963,0.04344\r\n7901.3,0.8477145,0.662193,0.276044,0.0434945\r\n7902.26,0.8476424,0.661401,0.276142,0.0435598\r\n7902.29,0.84756565,0.662923,0.276145,0.0435613\r\n7905.98,0.8474863,0.659879,0.27652,0.0438122\r\n7907.72,0.8474081,0.659164,0.276698,0.0439309\r\n7909.69,0.8473276,0.658358,0.276998,0.0440647\r\n7911.72,0.8472466,0.657526,0.277308,0.0442027\r\n7913.41,0.84716175,0.656836,0.277565,0.0443171\r\n7915.21,0.8470792,0.656281,0.277841,0.0444401\r\n7916.24,0.84700705,0.655967,0.277945,0.0445097\r\n7917.13,0.84692775,0.655695,0.278036,0.04457\r\n7920.84,0.8468414,0.654174,0.278414,0.0448224\r\n7922.94,0.84674785,0.653744,0.278628,0.0449654\r\n7926.44,0.846653,0.65303,0.278984,0.0452328\r\n7927.44,0.8465588,0.652827,0.279136,0.0453089\r\n7928.29,0.8464599,0.652653,0.279266,0.0453741\r\n7932,0.84636765,0.651131,0.279833,0.045658\r\n7933.93,0.8462739,0.650836,0.280127,0.0458054\r\n7937.91,0.84618145,0.650228,0.280532,0.0461093\r\n7939.46,0.84608665,0.649991,0.28069,0.0462148\r\n7942.38,0.8459953,0.648795,0.280987,0.0464131\r\n7943.18,0.84590655,0.648469,0.281068,0.0464673\r\n7945.16,0.84582935,0.647656,0.28127,0.0466022\r\n7946.89,0.8457595,0.646948,0.281534,0.0467197\r\n7949.13,0.8456965,0.64603,0.281876,0.046872\r\n7950.6,0.8456413,0.645426,0.282101,0.0469721\r\n7952.65,0.8455925,0.643738,0.282413,0.0471113\r\n7954.3,0.8455483,0.642383,0.282664,0.047223\r\n7954.32,0.8455106,0.643905,0.282667,0.0472246\r\n7957.33,0.8454854,0.642825,0.283127,0.0474295\r\n7960.14,0.84547355,0.641819,0.283556,0.0476205\r\n7960.35,0.84547355,0.641743,0.283577,0.0476348\r\n7961.75,0.84549805,0.641242,0.283719,0.0477299\r\n7965.47,0.8455364,0.639721,0.284098,0.0479823\r\n7968.56,0.84557905,0.638456,0.284412,0.0481922\r\n7969.18,0.84563755,0.638199,0.284476,0.0482347\r\n7971.37,0.8456995,0.637864,0.284699,0.0483836\r\n7971.58,0.84579225,0.637833,0.28472,0.0483975\r\n7976.64,0.845903,0.637059,0.285235,0.048913\r\n7979.78,0.84602175,0.63577,0.285555,0.0492332\r\n7980.35,0.84615665,0.635537,0.285613,0.0492911\r\n7982.6,0.84629055,0.635079,0.285842,0.0495199\r\n7982.8,0.84645635,0.635037,0.285863,0.0495408\r\n7987.27,0.8466233,0.634124,0.286318,0.049996\r\n7987.8,0.84680735,0.634016,0.286372,0.0500498\r\n7990.29,0.8469981,0.632998,0.286625,0.0503029\r\n7991.52,0.8472044,0.632495,0.28675,0.050428\r\n7993.83,0.84741785,0.632142,0.286985,0.0506632\r\n7994.76,0.84767015,0.631998,0.287081,0.0507587\r\n7998.97,0.8479388,0.631354,0.28751,0.0511873\r\n8001.52,0.84821425,0.630313,0.287768,0.0514462\r\n8002.26,0.8484893,0.630008,0.287844,0.0514968\r\n8002.69,0.8487758,0.629833,0.287888,0.0515258\r\n8005.05,0.84909955,0.62935,0.288129,0.0516867\r\n8009.76,0.849423,0.62839,0.288448,0.0520064\r\n8010.14,0.84975515,0.628312,0.288474,0.0520323\r\n8012.74,0.8500948,0.627515,0.288651,0.0522089\r\n8013.86,0.85042715,0.627171,0.288727,0.052323\r\n8016.28,0.85075085,0.626678,0.288891,0.0525692\r\n8017.25,0.8511464,0.626479,0.28899,0.0526682\r\n8021.01,0.8515271,0.625712,0.289373,0.053051\r\n8021.31,0.8519265,0.62565,0.289404,0.0530817\r\n8023.97,0.85229985,0.624562,0.289674,0.0533522\r\n8025.02,0.8526881,0.624129,0.289782,0.0534599\r\n8027.5,0.85306495,0.623368,0.290035,0.0537124\r\n8028.5,0.85343495,0.623063,0.290102,0.0538138\r\n8028.74,0.85385285,0.622988,0.290119,0.0538387\r\n8035.19,0.85427175,0.621671,0.290557,0.0544955\r\n8036,0.8546494,0.621507,0.290612,0.0545771\r\n8036.2,0.8550205,0.621467,0.290625,0.0545974\r\n8038.73,0.85539305,0.62043,0.290797,0.0548551\r\n8039.91,0.8557824,0.619945,0.290838,0.0549756\r\n8042.68,0.85615385,0.619096,0.290932,0.0552576\r\n8043.49,0.85647865,0.618848,0.29096,0.0553399\r\n8043.63,0.8568289,0.618804,0.290964,0.0553544\r\n8047.34,0.8571791,0.617283,0.291091,0.0557325\r\n8049.94,0.85752865,0.616218,0.29118,0.0559972\r\n8050.98,0.85786055,0.615794,0.29118,0.0561026\r\n8051.06,0.858214,0.615761,0.29118,0.0561107\r\n8053.91,0.85849985,0.613412,0.29118,0.0564009\r\n8054.75,0.85878365,0.612718,0.29118,0.0564866\r\n8054.77,0.8590847,0.61424,0.29118,0.0564889\r\n8058.46,0.8593778,0.611196,0.291181,0.0568648\r\n8058.47,0.8596802,0.611191,0.291181,0.056866\r\n8062.18,0.85999575,0.609675,0.291181,0.057243\r\n8064.89,0.86031065,0.608563,0.291182,0.0575193\r\n8065.14,0.8605658,0.608463,0.291182,0.0575442\r\n8065.89,0.86081335,0.608153,0.291182,0.0576596\r\n8065.97,0.86107795,0.60813,0.291182,0.0576708\r\n8069.61,0.8613386,0.607012,0.291183,0.058227\r\n8072.63,0.8616135,0.606602,0.291183,0.0586869\r\n8077.19,0.8618628,0.605982,0.291184,0.0591519\r\n8079.84,0.8621043,0.605622,0.291184,0.0594212\r\n8080.8,0.86233955,0.605492,0.291184,0.0595191\r\n8083.85,0.862539,0.605025,0.291185,0.0598301\r\n8088.26,0.8627184,0.604351,0.291185,0.0602784\r\n8088.4,0.86287175,0.604293,0.291185,0.0602929\r\n8091.97,0.86300995,0.60283,0.291186,0.0606566\r\n8094.79,0.8631177,0.602399,0.291186,0.0609433\r\n8095.08,0.86319475,0.602354,0.291186,0.0609734\r\n8099.43,0.8632707,0.601689,0.291187,0.0616363\r\n8099.59,0.8632864,0.601622,0.291187,0.0616614\r\n8102.57,0.86329765,0.600401,0.291188,0.0621161\r\n8103.14,0.86325225,0.600168,0.291188,0.062174\r\n8106.86,0.86318465,0.598646,0.291188,0.0625522\r\n8107.05,0.8630878,0.598567,0.291188,0.0625719\r\n8109.73,0.8629766,0.597467,0.291189,0.0628452\r\n8110.57,0.8628319,0.597125,0.291189,0.0629304\r\n8113.8,0.86266395,0.594461,0.291189,0.0632593\r\n8114.26,0.8624804,0.594081,0.291189,0.0633296\r\n8114.28,0.8622714,0.595603,0.291189,0.0633331\r\n8114.5,0.8620098,0.595526,0.291189,0.0633661\r\n8118.22,0.8617185,0.594194,0.29119,0.0639335\r\n8121.29,0.861428,0.593094,0.29119,0.064402\r\n8121.72,0.8611147,0.592941,0.291191,0.0644455\r\n8124.68,0.86079155,0.592639,0.291191,0.0647472\r\n8125.67,0.86041665,0.592538,0.291191,0.064848\r\n8132.52,0.8600521,0.591841,0.291192,0.0655453\r\n8132.91,0.8596869,0.591801,0.291192,0.0655852\r\n8133.12,0.8592868,0.591769,0.291192,0.0656067\r\n8136.84,0.8588591,0.5912,0.291193,0.0659854\r\n8139.63,0.8584466,0.590774,0.291193,0.0662692\r\n8140.37,0.858004,0.590661,0.291193,0.0663445\r\n8143.75,0.8575842,0.590317,0.291194,0.0666885\r\n8144.29,0.85708985,0.590261,0.291194,0.0667718\r\n8148.01,0.8565982,0.589883,0.291195,0.0673384\r\n8151.24,0.8561126,0.589554,0.291195,0.0678312\r\n8151.56,0.8556369,0.589521,0.291195,0.0678643\r\n8154.58,0.8551591,0.58906,0.291196,0.0681712\r\n8155.46,0.85460875,0.588924,0.291196,0.0682616\r\n8159.02,0.85405735,0.58838,0.291196,0.0686235\r\n8159.18,0.85352945,0.58837,0.291196,0.0686397\r\n8162.47,0.8529904,0.588147,0.291197,0.0689745\r\n8166.63,0.85243355,0.587864,0.291197,0.0696096\r\n8169.52,0.85190365,0.587668,0.291198,0.0700513\r\n8169.96,0.85138185,0.587639,0.291187,0.0701172\r\n8170.22,0.8509004,0.587621,0.29118,0.0701438\r\n8174.09,0.85036865,0.587227,0.291082,0.0705377\r\n8181.18,0.8498373,0.586504,0.290903,0.0712605\r\n8181.41,0.849362,0.586481,0.290897,0.0712952\r\n8181.54,0.8488853,0.586468,0.290894,0.0713151\r\n8184.47,0.8483732,0.58617,0.29082,0.0717611\r\n8188.68,0.8478923,0.585742,0.290534,0.0724031\r\n8188.99,0.8473986,0.585709,0.290513,0.0724356\r\n8192.61,0.8469654,0.585341,0.290268,0.0728033\r\n8195.67,0.8465353,0.585134,0.29006,0.0731149\r\n8199.9,0.84610625,0.584847,0.289773,0.0735464\r\n8200.19,0.845688,0.584827,0.289754,0.0735754\r\n8203.81,0.8452662,0.584582,0.289508,0.0739437\r\n8206.87,0.8449027,0.58427,0.289301,0.0742553\r\n8211.13,0.8445416,0.583836,0.289084,0.0746897\r\n8211.39,0.8442095,0.58381,0.289071,0.0747287\r\n8215,0.8439153,0.583442,0.288888,0.0752797\r\n8218.62,0.8436392,0.582888,0.288704,0.0758324\r\n8221.8,0.8433656,0.582402,0.288542,0.0763174\r\n8222.46,0.84308935,0.582302,0.28852,0.0764172\r\n8226.11,0.8428424,0.582054,0.288396,0.0769751\r\n8226.33,0.8426271,0.582039,0.288389,0.0769971\r\n8233.01,0.84241135,0.581586,0.288163,0.077677\r\n8233.66,0.8421841,0.581542,0.288147,0.077743\r\n8237.34,0.84198605,0.581042,0.288054,0.0781183\r\n8237.53,0.84180305,0.581016,0.288049,0.0781374\r\n8244.84,0.84161145,0.580022,0.287864,0.0788822\r\n8247.95,0.8414359,0.579547,0.287785,0.0791984\r\n8248.57,0.84126165,0.579452,0.287785,0.0792616\r\n8252.3,0.8411084,0.578882,0.287786,0.0798308\r\n8252.46,0.8409291,0.578857,0.287786,0.079855\r\n8256.06,0.8407383,0.578307,0.287786,0.0804043\r\n8259.16,0.8405533,0.577833,0.287787,0.0807199\r\n8259.76,0.8403816,0.577741,0.287817,0.0807807\r\n8259.92,0.84013465,0.577717,0.287825,0.0807969\r\n8263.63,0.83974455,0.577149,0.288015,0.0811751\r\n8267.21,0.839356,0.576601,0.288198,0.08154\r\n8267.29,0.8389701,0.576591,0.288202,0.0815475\r\n8267.32,0.8386611,0.576586,0.288203,0.0815527\r\n8267.34,0.83820455,0.576583,0.288204,0.0815562\r\n8271.04,0.83763725,0.576081,0.288393,0.0821193\r\n8274.12,0.83707045,0.575662,0.28855,0.0825896\r\n8274.73,0.83658195,0.575579,0.28855,0.0826833\r\n8274.76,0.836105,0.575576,0.28855,0.0826868\r\n8274.78,0.83547935,0.575573,0.28855,0.0826902\r\n8278.4,0.83487975,0.57508,0.288551,0.0830591\r\n8278.45,0.8341398,0.575073,0.288551,0.0830638\r\n8282.14,0.8334783,0.574509,0.288551,0.0834397\r\n8282.16,0.83267885,0.574505,0.288551,0.083442\r\n8285.85,0.83188005,0.573941,0.288552,0.0838178\r\n8285.86,0.831109,0.57394,0.288552,0.0838184\r\n8286.01,0.8303669,0.573917,0.288552,0.0838335\r\n8289.07,0.8296366,0.573449,0.288552,0.0843003\r\n8293.32,0.8289078,0.5728,0.288841,0.0849484\r\n8293.5,0.8282612,0.572762,0.288854,0.0849762\r\n8297.07,0.82781125,0.572033,0.289096,0.0853399\r\n8300.29,0.82737165,0.571376,0.289315,0.0856677\r\n8300.77,0.8269343,0.571279,0.289331,0.0857163\r\n8300.82,0.82649695,0.57127,0.289333,0.0857221\r\n8300.85,0.82628795,0.571266,0.289334,0.0857245\r\n8304.61,0.82626605,0.570691,0.289462,0.0861073\r\n8304.63,0.8263424,0.570688,0.289463,0.0861096\r\n8304.65,0.82634415,0.570685,0.289464,0.0861119\r\n8304.73,0.82641765,0.570673,0.289466,0.0861194\r\n8308.22,0.8266443,0.570138,0.289585,0.0864756\r\n8308.41,0.8269489,0.5701,0.289592,0.0864947\r\n8308.43,0.8274818,0.570095,0.289593,0.0864971\r\n8308.46,0.82800035,0.570091,0.289593,0.0864994\r\n8311.51,0.8286855,0.569468,0.289697,0.0868098\r\n8312.22,0.8294467,0.569323,0.289716,0.0868822\r\n8312.24,0.83028375,0.569319,0.289716,0.0868845\r\n8312.26,0.8311847,0.569314,0.289717,0.0868868\r\n8315.67,0.83208405,0.568617,0.289804,0.0872343\r\n8315.95,0.8329845,0.568532,0.289811,0.0872627\r\n8316.02,0.83395495,0.568511,0.289813,0.0872731\r\n8316.04,0.83495255,0.568504,0.289814,0.0872766\r\n8316.07,0.83601025,0.568497,0.289814,0.0872801\r\n8319.39,0.83707585,0.567476,0.289899,0.0877876\r\n8319.83,0.83814145,0.567388,0.28991,0.0878536\r\n8319.85,0.839207,0.567383,0.289911,0.087857\r\n8319.87,0.84026875,0.567379,0.289912,0.0878605\r\n8323.44,0.84125825,0.56665,0.290003,0.0884054\r\n8323.63,0.84232235,0.566611,0.290008,0.0884245\r\n8323.65,0.84339175,0.566607,0.290008,0.0884268\r\n8323.68,0.84451365,0.566602,0.290009,0.0884291\r\n8326.46,0.8455673,0.566034,0.29008,0.0887123\r\n8326.85,0.8465568,0.565955,0.290106,0.0887517\r\n8327.44,0.84756065,0.565713,0.290147,0.088812\r\n8327.46,0.8486262,0.565704,0.290148,0.0888143\r\n8327.48,0.849678,0.565694,0.29015,0.0888166\r\n8330.56,0.8506675,0.564434,0.290359,0.0891299\r\n8331.24,0.851664,0.564154,0.290405,0.0891994\r\n8331.26,0.85273095,0.564145,0.290407,0.0892017\r\n8331.29,0.853857,0.564136,0.290408,0.0892041\r\n8334.27,0.854915,0.562912,0.290611,0.0895081\r\n8334.67,0.8559045,0.562749,0.290638,0.0895487\r\n8335.05,0.8569027,0.562595,0.290664,0.0895869\r\n8335.07,0.85796825,0.562586,0.290665,0.0895892\r\n8335.09,0.8590338,0.562577,0.290667,0.0895915\r\n8335.12,0.8600752,0.562568,0.290669,0.0895938\r\n8337.68,0.8610505,0.561517,0.290843,0.089855\r\n8337.99,0.86203995,0.561391,0.290859,0.0898863\r\n8338.88,0.8630294,0.561255,0.290904,0.0899767\r\n8338.9,0.86401885,0.561252,0.290905,0.089979\r\n8338.92,0.8650281,0.561248,0.290906,0.0899813\r\n8338.94,0.8661056,0.561245,0.290907,0.0899836\r\n8338.97,0.86724725,0.561241,0.290908,0.0899859\r\n8338.99,0.86838895,0.561238,0.29091,0.0899882\r\n8342.75,0.86953065,0.560663,0.291101,0.0903711\r\n8342.77,0.8706861,0.560659,0.291103,0.0903734\r\n8342.79,0.87190385,0.560656,0.291104,0.0903757\r\n8342.82,0.8731216,0.560652,0.291105,0.090378\r\n8342.84,0.87433935,0.560649,0.291106,0.0903803\r\n8342.86,0.8755489,0.560645,0.291107,0.0903826\r\n8345.44,0.8767596,0.56025,0.291239,0.0906456\r\n8345.9,0.87797735,0.560181,0.291262,0.0906919\r\n8346.62,0.8791951,0.56007,0.291299,0.0908021\r\n8346.64,0.88041285,0.560067,0.2913,0.0908056\r\n8346.67,0.8816306,0.560063,0.291301,0.0908091\r\n8346.69,0.8828725,0.56006,0.291303,0.0908125\r\n8346.71,0.8841843,0.560056,0.291304,0.090816\r\n8346.74,0.88555425,0.560053,0.291305,0.0908195\r\n8350.5,0.8869242,0.559478,0.291497,0.091393\r\n8350.52,0.88829415,0.559475,0.291498,0.0913964\r\n8350.54,0.8896641,0.559471,0.291499,0.0913999\r\n8350.56,0.89103405,0.559468,0.2915,0.0914034\r\n8350.59,0.8923695,0.559464,0.291501,0.0914069\r\n8352.64,0.8936341,0.55915,0.291606,0.0917201\r\n8352.9,0.8948325,0.55911,0.291615,0.09176\r\n8353.39,0.89597415,0.55901,0.291632,0.0918346\r\n8354.35,0.8971158,0.558815,0.291664,0.0919319\r\n8354.37,0.89825745,0.55881,0.291665,0.0919342\r\n8354.39,0.8993991,0.558806,0.291666,0.0919365\r\n8354.41,0.90056455,0.558801,0.291667,0.0919389\r\n8354.44,0.90179695,0.558797,0.291667,0.0919412\r\n8358.2,0.9030908,0.558029,0.291796,0.092324\r\n8358.22,0.90438465,0.558024,0.291796,0.0923263\r\n8358.24,0.9056785,0.55802,0.291797,0.0923286\r\n8358.27,0.90697235,0.558015,0.291798,0.092331\r\n8358.29,0.9082319,0.558011,0.291799,0.0923333\r\n8360.35,0.9094497,0.557589,0.291869,0.0925435\r\n8362.05,0.91066745,0.557416,0.291927,0.0927161\r\n8362.07,0.9118852,0.557414,0.291928,0.0927184\r\n8362.09,0.91306255,0.557412,0.291928,0.0927207\r\n8363.86,0.9141792,0.557232,0.291988,0.0929003\r\n8364.62,0.91524475,0.557155,0.29204,0.0929779\r\n8365.85,0.91626875,0.557029,0.292124,0.0931035\r\n8369.59,0.91725745,0.556648,0.292378,0.0934841\r\n8371.55,0.9182092,0.556449,0.292511,0.0936833\r\n8373.31,0.91903155,0.556089,0.292631,0.0938628\r\n8375.08,0.91973,0.555728,0.292751,0.0940429\r\n8375.85,0.9202819,0.555572,0.292771,0.0941211\r\n8376.98,0.9208338,0.55534,0.2928,0.0942364\r\n8377,0.9213857,0.555336,0.2928,0.0942387\r\n8377.02,0.92161115,0.555331,0.292801,0.094241\r\n8379,0.92155425,0.554928,0.292851,0.094442\r\n8380.62,0.92149735,0.554429,0.292893,0.0946076\r\n8380.65,0.92144045,0.554422,0.292894,0.0946099\r\n8380.67,0.92138355,0.554415,0.292894,0.0946123\r\n8380.69,0.92084575,0.554408,0.292895,0.0946146\r\n8382.72,0.9199517,0.553787,0.292947,0.0948208\r\n8384.2,0.91905765,0.553179,0.292985,0.0949719\r\n8384.22,0.9181636,0.55317,0.292985,0.0949742\r\n8384.25,0.91730995,0.553161,0.292986,0.0949766\r\n8384.27,0.916517,0.553151,0.292986,0.0949789\r\n8384.29,0.91577515,0.553142,0.292987,0.0949812\r\n8384.32,0.91510935,0.553133,0.292988,0.0949835\r\n8384.34,0.91401965,0.553123,0.292988,0.0949858\r\n8386.43,0.91281765,0.552266,0.293042,0.0951989\r\n8387.07,0.91155795,0.552003,0.293058,0.0952644\r\n8387.85,0.91042215,0.551686,0.293078,0.0953432\r\n8387.87,0.9094329,0.551676,0.293079,0.0953455\r\n8387.89,0.90844365,0.551667,0.293079,0.0953478\r\n8387.92,0.90732405,0.551658,0.29308,0.0953501\r\n8390.03,0.90645185,0.550791,0.293134,0.0955656\r\n8390.15,0.9056727,0.550744,0.293142,0.0955771\r\n8394.56,0.9047025,0.550295,0.293442,0.0960266\r\n8399.04,0.9035765,0.549838,0.293746,0.0964829\r\n8401.25,0.90237135,0.549613,0.293897,0.0967082\r\n8401.34,0.9015549,0.549604,0.29391,0.0967169\r\n8402.74,0.90113645,0.549462,0.294123,0.0968594\r\n8402.76,0.90025815,0.54946,0.294127,0.0968617\r\n8405.79,0.8992309,0.549152,0.294588,0.0971698\r\n8406.36,0.89820365,0.549093,0.294676,0.0972283\r\n8406.38,0.8971764,0.549091,0.294679,0.0972306\r\n8406.41,0.89614915,0.549088,0.294683,0.097233\r\n8406.43,0.8951219,0.549086,0.294686,0.0972353\r\n8406.45,0.894156,0.549084,0.29469,0.0972376\r\n8408.74,0.8931132,0.54885,0.295039,0.097471\r\n8410.01,0.8923143,0.548722,0.295168,0.0975996\r\n8410.03,0.8915154,0.548719,0.29517,0.0976019\r\n8410.05,0.8907165,0.548717,0.295173,0.0976042\r\n8410.08,0.88947565,0.548715,0.295175,0.0976065\r\n8412.53,0.8881222,0.548465,0.295425,0.0978567\r\n8413.65,0.88684775,0.548293,0.295539,0.0979708\r\n8413.68,0.8857626,0.54829,0.295542,0.0979731\r\n8413.7,0.8848685,0.548286,0.295544,0.0979754\r\n8413.72,0.8834155,0.548283,0.295546,0.0979778\r\n8417.02,0.88191925,0.547779,0.295881,0.0983131\r\n8417.25,0.8805495,0.547743,0.295906,0.0983374\r\n8417.28,0.87917975,0.547739,0.295908,0.0983397\r\n8417.3,0.87826985,0.547736,0.29591,0.0983421\r\n8417.32,0.8775089,0.547732,0.295913,0.0983444\r\n8417.35,0.87674795,0.547729,0.295915,0.0983467\r\n8417.37,0.87522415,0.547725,0.295917,0.098349\r\n8419.97,0.87361975,0.547327,0.296183,0.0986143\r\n8419.99,0.8717173,0.547324,0.296184,0.098616\r\n8420.83,0.8702835,0.547195,0.29627,0.0987017\r\n8420.85,0.86914205,0.547192,0.296272,0.098704\r\n8420.88,0.86800055,0.547188,0.296275,0.0987063\r\n8420.9,0.86685905,0.547185,0.296277,0.0987087\r\n8420.92,0.86571755,0.547182,0.296279,0.098711\r\n8420.95,0.865018,0.547178,0.296282,0.0987133\r\n8420.97,0.8635721,0.547175,0.296284,0.0987156\r\n8424.43,0.8621262,0.546645,0.296637,0.0990683\r\n8424.45,0.8606803,0.546641,0.296639,0.0990706\r\n8424.48,0.8592344,0.546638,0.296641,0.0990729\r\n8424.5,0.8585032,0.546634,0.296644,0.0990753\r\n8424.52,0.8578944,0.546631,0.296646,0.0990776\r\n8424.55,0.85651305,0.546627,0.296648,0.0990799\r\n8427.45,0.85491495,0.546184,0.296944,0.0993753\r\n8428.03,0.85331685,0.546064,0.297003,0.0994349\r\n8428.06,0.85171875,0.54606,0.297006,0.0994372\r\n8428.08,0.85012065,0.546055,0.297008,0.0994396\r\n8428.1,0.8492854,0.54605,0.29701,0.0994419\r\n8428.12,0.8485308,0.546046,0.297013,0.0994442\r\n8428.15,0.8479749,0.546041,0.297015,0.0994465\r\n8428.24,0.8466328,0.546021,0.297025,0.0994564\r\n8431.2,0.8451108,0.545418,0.297326,0.0997575\r\n8431.63,0.8435888,0.545329,0.297392,0.0998015\r\n8431.66,0.8420668,0.545325,0.297395,0.0998038\r\n8431.68,0.84054485,0.54532,0.297399,0.0998062\r\n8431.7,0.8390229,0.545315,0.297402,0.0998085\r\n8431.72,0.83849025,0.545311,0.297406,0.0998108\r\n8431.75,0.83734185,0.545306,0.297409,0.0998131\r\n8434.9,0.8360482,0.544663,0.29789,0.100134\r\n8435.3,0.83475455,0.544372,0.297951,0.100175\r\n8435.32,0.8334609,0.544356,0.297955,0.100177\r\n8435.35,0.83204135,0.544339,0.297958,0.10018\r\n8438.59,0.8313179,0.542,0.298454,0.10051\r\n8438.62,0.83073395,0.543522,0.298457,0.100513\r\n8438.69,0.83006195,0.543461,0.298469,0.10052\r\n8439.47,0.8292703,0.542818,0.298548,0.1006\r\n8442.31,0.82839525,0.540478,0.298837,0.100793\r\n8442.79,0.8274688,0.540283,0.298886,0.100825\r\n8446.02,0.8264945,0.538957,0.299215,0.101045\r\n8449.74,0.82554485,0.537435,0.299593,0.101297\r\n8449.92,0.82530805,0.537398,0.299612,0.10131\r\n8450.28,0.8251782,0.537325,0.299667,0.101334\r\n8450.69,0.8250218,0.53724,0.29973,0.101362\r\n8457.19,0.824791,0.535914,0.300721,0.102353\r\n8457.41,0.82448675,0.535846,0.300755,0.102387\r\n8457.77,0.8241085,0.535736,0.300779,0.102442\r\n8458.18,0.82366805,0.535609,0.300807,0.102505\r\n8460.91,0.82380655,0.534773,0.300992,0.10269\r\n8468.63,0.82401605,0.533987,0.301517,0.103215\r\n8469,0.82415155,0.53395,0.301542,0.10324\r\n8469.41,0.8242247,0.533908,0.30157,0.103268\r\n8472.1,0.82436655,0.533634,0.301753,0.103542\r\n8475.82,0.8245293,0.532112,0.302006,0.103921\r\n8479.85,0.8246951,0.531494,0.30228,0.104332\r\n8480.22,0.82487425,0.531438,0.302305,0.104369\r\n8480.63,0.8251059,0.531375,0.302333,0.104411\r\n8483.27,0.82535745,0.530972,0.302512,0.10459\r\n8486.99,0.82559695,0.52945,0.302765,0.104843\r\n8490.7,0.8258094,0.527929,0.303017,0.105095\r\n8491.08,0.8260217,0.527775,0.303043,0.105121\r\n8491.45,0.8262323,0.527623,0.303062,0.105146\r\n8491.86,0.8264463,0.527456,0.303083,0.105174\r\n8494.42,0.8266202,0.526407,0.303213,0.105434\r\n8498.13,0.8268019,0.524886,0.303403,0.105813\r\n8501.84,0.82698405,0.523364,0.303592,0.106191\r\n8503.08,0.82717045,0.522856,0.303655,0.106317\r\n8505.56,0.82734415,0.521843,0.303782,0.106485\r\n8506.04,0.8274795,0.521794,0.303806,0.106518\r\n8506.41,0.82763545,0.521756,0.303831,0.106543\r\n8513.02,0.8277937,0.521083,0.304281,0.106992\r\n8514.31,0.8279482,0.520908,0.304368,0.10708\r\n8517.26,0.8280851,0.520507,0.304569,0.10738\r\n8517.63,0.8282238,0.520457,0.304594,0.107418\r\n8524.21,0.8283651,0.519562,0.305041,0.108088\r\n8525.53,0.828515,0.519427,0.305131,0.108223\r\n8528.48,0.8286572,0.519127,0.305332,0.108423\r\n8528.85,0.8288012,0.51909,0.305369,0.108448\r\n8531.67,0.8289584,0.518802,0.305656,0.10864\r\n8536.76,0.8291294,0.518285,0.306174,0.108986\r\n8539.71,0.82930105,0.517984,0.306475,0.109286\r\n8540.08,0.8294811,0.517946,0.306512,0.109324\r\n8542.86,0.829672,0.517663,0.306796,0.109608\r\n8547.98,0.8298635,0.517141,0.307318,0.110129\r\n8550.94,0.8300467,0.516841,0.307618,0.11043\r\n8551.3,0.8302408,0.516804,0.307655,0.110467\r\n8554.06,0.83043575,0.516523,0.307936,0.110748\r\n8555.47,0.8306445,0.516307,0.30808,0.110891\r\n8558.42,0.83086805,0.515855,0.30838,0.111092\r\n8561.52,0.83107435,0.515382,0.308591,0.111302\r\n8562.53,0.8312974,0.515331,0.308659,0.111371\r\n8566.69,0.8315255,0.515119,0.308942,0.111654\r\n8569.64,0.83177325,0.514969,0.309143,0.111805\r\n8573.76,0.8320078,0.51476,0.309562,0.112014\r\n8576.45,0.83225285,0.514624,0.309836,0.112152\r\n8580.87,0.83249055,0.514924,0.310286,0.112378\r\n8581.65,0.83273835,0.514977,0.310306,0.112417\r\n8584.98,0.83298545,0.515203,0.310391,0.112756\r\n8587.67,0.8332332,0.515386,0.31046,0.113031\r\n8592.88,0.8334809,0.515916,0.310594,0.113561\r\n8595.82,0.83372865,0.516216,0.310669,0.113761\r\n8596.21,0.83398065,0.516255,0.310695,0.113787\r\n8598.9,0.83424175,0.51653,0.310878,0.11397\r\n8604.1,0.83450445,0.517323,0.311232,0.114323\r\n8606.39,0.8347693,0.517672,0.311387,0.114479\r\n8607.05,0.83502205,0.517805,0.311432,0.114524\r\n8607.43,0.83530045,0.517884,0.311442,0.11455\r\n8613.89,0.8355691,0.519195,0.311607,0.114989\r\n8614.92,0.8358234,0.519352,0.311633,0.115059\r\n8615.32,0.83609305,0.519414,0.311644,0.115086\r\n8621.38,0.8363437,0.520338,0.311799,0.115498\r\n8622,0.83658995,0.520589,0.311815,0.11554\r\n8625.14,0.83681865,0.521861,0.311922,0.115753\r\n8626.14,0.83704025,0.522266,0.311956,0.115821\r\n8626.54,0.83725885,0.52243,0.311969,0.115849\r\n8628.9,0.8374939,0.523383,0.31205,0.116009\r\n8632.65,0.83772045,0.524526,0.312178,0.116264\r\n8633.22,0.8379496,0.524754,0.312197,0.116302\r\n8636.41,0.83815235,0.526048,0.312116,0.11652\r\n8636.44,0.83834485,0.52757,0.312115,0.116521\r\n8637.37,0.83853745,0.52776,0.312092,0.116584\r\n8637.77,0.83874775,0.527841,0.312082,0.116612\r\n8643.93,0.83895065,0.529093,0.311926,0.117031\r\n8647.69,0.83914205,0.530616,0.31183,0.117286\r\n8647.72,0.83933235,0.532138,0.31183,0.117288\r\n8648.16,0.83952295,0.532318,0.311819,0.117318\r\n8648.6,0.83968365,0.532495,0.311808,0.117348\r\n8648.99,0.8398581,0.532654,0.311798,0.117374\r\n8651.47,0.84005975,0.533661,0.311735,0.117543\r\n8655.23,0.84024105,0.534803,0.31164,0.117798\r\n8655.25,0.84042655,0.536325,0.311639,0.1178\r\n8656.09,0.8406433,0.536495,0.311618,0.117857\r\n8660.21,0.84088245,0.537333,0.311513,0.118137\r\n8662.75,0.84111975,0.537848,0.311449,0.118266\r\n8662.77,0.8413484,0.53937,0.311449,0.118268\r\n8663.1,0.84156285,0.539504,0.31144,0.118284\r\n8663.58,0.84183415,0.5397,0.31144,0.118309\r\n8666.53,0.8420896,0.540893,0.311441,0.118459\r\n8666.55,0.84236095,0.542415,0.311441,0.118461\r\n8667.34,0.8426502,0.542735,0.311441,0.118501\r\n8670.31,0.84296605,0.543938,0.311441,0.118652\r\n8674.07,0.84325635,0.54546,0.311442,0.118844\r\n8674.83,0.84353095,0.545769,0.311442,0.118883\r\n8675.17,0.84383245,0.545905,0.311442,0.1189\r\n8677.83,0.8441319,0.546983,0.311443,0.118991\r\n8677.86,0.8444311,0.548505,0.311443,0.118992\r\n8678.05,0.84476355,0.548583,0.311443,0.118998\r\n8681.61,0.84507725,0.550028,0.311564,0.11912\r\n8681.64,0.8453626,0.55155,0.311565,0.119121\r\n8681.66,0.84565455,0.553072,0.311566,0.119121\r\n8682.33,0.84598715,0.553344,0.311589,0.119144\r\n8685.42,0.84628795,0.554594,0.311694,0.11925\r\n8686.09,0.846566,0.554797,0.311717,0.119272\r\n8686.39,0.8468722,0.554889,0.311727,0.119283\r\n8689.17,0.84717595,0.555737,0.311822,0.119472\r\n8689.26,0.8475121,0.555773,0.311825,0.119478\r\n8692.93,0.84778875,0.557259,0.311825,0.119728\r\n8692.96,0.8480713,0.558781,0.311825,0.119729\r\n8693.58,0.8483697,0.559035,0.311826,0.119772\r\n8696.72,0.8486457,0.560304,0.311826,0.119985\r\n8696.74,0.8489019,0.561826,0.311826,0.119986\r\n8697.61,0.8491816,0.562178,0.311826,0.120045\r\n8700.5,0.8494638,0.563349,0.311827,0.120193\r\n8701.08,0.84974285,0.563584,0.311827,0.120222\r\n8704.21,0.85002205,0.564853,0.311827,0.120382\r\n8704.26,0.8503279,0.564871,0.311826,0.120385\r\n8708.02,0.85060175,0.566394,0.311731,0.120576\r\n8708.57,0.8509403,0.566617,0.311717,0.120605\r\n8711.78,0.8512791,0.567917,0.311636,0.120768\r\n8711.8,0.8516218,0.569439,0.311635,0.120769\r\n8712.33,0.8519199,0.569653,0.311622,0.120796\r\n8712.57,0.8522383,0.56975,0.311616,0.120809\r\n8715.56,0.8525902,0.570961,0.31154,0.121012\r\n8719.15,0.8529154,0.572417,0.311449,0.121256\r\n8719.32,0.85324485,0.572484,0.311449,0.121267\r\n8719.83,0.8535864,0.572638,0.311449,0.121302\r\n8723.07,0.8539281,0.573626,0.311449,0.121523\r\n8723.1,0.8542708,0.575148,0.311449,0.121524\r\n8723.58,0.8545859,0.575344,0.31145,0.121557\r\n8723.79,0.85496285,0.575429,0.31145,0.121571\r\n8726.86,0.8553314,0.576671,0.31145,0.121676\r\n8726.88,0.85567995,0.578193,0.31145,0.121677\r\n8727.34,0.8560561,0.578379,0.31145,0.121692\r\n8730.64,0.85644355,0.579715,0.311451,0.121805\r\n8734.1,0.85683365,0.581118,0.311451,0.121923\r\n8734.4,0.85719955,0.581238,0.311451,0.121933\r\n8734.84,0.85756385,0.581415,0.311451,0.121948\r\n8735.01,0.85791115,0.581485,0.311451,0.121954\r\n8738.16,0.8582645,0.582761,0.311452,0.122114\r\n8738.59,0.85864095,0.582936,0.311452,0.122136\r\n8741.92,0.85901525,0.584283,0.311452,0.122306\r\n8741.94,0.85939735,0.585805,0.311452,0.122307\r\n8745.7,0.8597431,0.587328,0.311453,0.122499\r\n8745.72,0.8600909,0.58885,0.311453,0.1225\r\n8746.09,0.8604936,0.588998,0.311453,0.122519\r\n8749.05,0.86085565,0.590198,0.311453,0.12267\r\n8749.48,0.8612178,0.590373,0.311464,0.122692\r\n8749.51,0.8615795,0.591895,0.311465,0.122693\r\n8749.85,0.86193785,0.592033,0.311474,0.122711\r\n8749.97,0.86225925,0.592081,0.311477,0.122717\r\n8753.26,0.8625804,0.593418,0.311561,0.122829\r\n8753.29,0.86292085,0.59494,0.311562,0.12283\r\n8757.04,0.86323005,0.596082,0.311658,0.122958\r\n8757.34,0.86353925,0.596202,0.311666,0.122968\r\n8760.8,0.8638493,0.597604,0.311754,0.123086\r\n8761.18,0.86418355,0.59843,0.311764,0.123099\r\n8764,0.86452365,0.604546,0.311836,0.123291\r\n8764.83,0.86489055,0.606346,0.311836,0.123347\r\n8768.53,0.8652523,0.614348,0.311837,0.123598\r\n8768.59,0.8656084,0.614376,0.311837,0.123603\r\n8772.29,0.86596515,0.615871,0.311837,0.123854\r\n8772.4,0.8663051,0.615889,0.311838,0.123862\r\n8775.21,0.8666515,0.616318,0.311838,0.123934\r\n8776.08,0.8670317,0.616451,0.311816,0.123956\r\n8779.78,0.8673905,0.617014,0.311722,0.124051\r\n8783.58,0.8678153,0.617787,0.311626,0.124148\r\n8787.27,0.86824105,0.618537,0.311533,0.124242\r\n8787.34,0.86866005,0.618547,0.311531,0.124244\r\n8787.36,0.8690995,0.61855,0.311531,0.124245\r\n8790.16,0.8695489,0.618977,0.31146,0.124435\r\n8794.76,0.86999865,0.61968,0.311343,0.124748\r\n8794.83,0.87045725,0.619707,0.311341,0.124753\r\n8798.52,0.87091415,0.621202,0.311248,0.125004\r\n8798.58,0.8713735,0.621225,0.311246,0.125007\r\n8802.28,0.8718292,0.622725,0.311153,0.125102\r\n8802.31,0.8722565,0.624247,0.311152,0.125103\r\n8802.33,0.8726965,0.624256,0.311152,0.125103\r\n8805.1,0.8730691,0.625378,0.311081,0.125174\r\n8806.07,0.8734405,0.62577,0.311049,0.125199\r\n8806.09,0.8738122,0.627292,0.311048,0.1252\r\n8809.82,0.8742207,0.62805,0.310922,0.125295\r\n8813.53,0.87460835,0.628804,0.310796,0.12539\r\n8813.58,0.87498955,0.628815,0.310795,0.125392\r\n8813.61,0.8753335,0.630337,0.310794,0.125393\r\n8813.63,0.87563905,0.631859,0.310793,0.125393\r\n8813.65,0.87587005,0.633381,0.310792,0.125394\r\n8813.68,0.8760999,0.634903,0.310791,0.125395\r\n8813.7,0.87635605,0.636425,0.310791,0.125396\r\n8816.3,0.87660115,0.63748,0.310703,0.125485\r\n8817.32,0.87681225,0.63789,0.310634,0.125519\r\n8817.46,0.87702295,0.637948,0.310624,0.125524\r\n8817.48,0.87719635,0.63947,0.310623,0.125525\r\n8817.5,0.87736935,0.640992,0.310621,0.125525\r\n8817.53,0.87750485,0.642514,0.31062,0.125526\r\n8817.55,0.8776403,0.644036,0.310618,0.125527\r\n8817.57,0.8778303,0.645178,0.310617,0.125528\r\n8824.75,0.8779996,0.646637,0.31013,0.125772\r\n8824.81,0.8781635,0.646649,0.310126,0.125775\r\n8825.06,0.8783458,0.646701,0.310109,0.125788\r\n8827.5,0.8785362,0.647197,0.309943,0.125913\r\n8832.3,0.87869025,0.648171,0.309822,0.126158\r\n8832.56,0.87886285,0.648224,0.309815,0.126171\r\n8836.31,0.87905255,0.649366,0.30972,0.126362\r\n8839.71,0.87925595,0.650056,0.309634,0.126536\r\n8842.44,0.8794646,0.650612,0.309565,0.126629\r\n8843.53,0.879674,0.650832,0.309529,0.126666\r\n8843.81,0.87990735,0.650889,0.309519,0.126675\r\n8850.93,0.8801155,0.651975,0.309278,0.126918\r\n8851.3,0.88032135,0.652032,0.309266,0.126928\r\n8853.65,0.8805298,0.652509,0.309186,0.126988\r\n8854.75,0.88073815,0.652733,0.30913,0.127016\r\n8858.8,0.88094635,0.653555,0.308925,0.127119\r\n8865.88,0.8811544,0.654636,0.308565,0.127301\r\n8866.29,0.8813623,0.654698,0.308544,0.127315\r\n8868.59,0.8815701,0.65501,0.308427,0.127393\r\n8869.7,0.88166605,0.65516,0.308428,0.127431\r\n8873.42,0.8816098,0.655665,0.308428,0.127558\r\n8877.1,0.88155115,0.656164,0.308429,0.127683\r\n8877.11,0.88155,0.656166,0.308429,0.127683\r\n8877.13,0.8814879,0.656169,0.308429,0.127684\r\n8877.52,0.88122055,0.656222,0.308429,0.127694\r\n8880.8,0.8810102,0.656722,0.308429,0.127778\r\n8880.82,0.88067075,0.656725,0.308429,0.127778\r\n8883.53,0.880277,0.657139,0.30843,0.127848\r\n8884.51,0.8798571,0.657288,0.30843,0.127873\r\n8885.01,0.8793047,0.657365,0.30843,0.127886\r\n8888.2,0.8788042,0.657851,0.30843,0.127967\r\n8888.23,0.8781503,0.657855,0.30843,0.127968\r\n8891.92,0.8774857,0.658418,0.308431,0.128062\r\n8892.05,0.8768081,0.658438,0.308431,0.128066\r\n8892.5,0.87608455,0.658507,0.308431,0.128081\r\n8894.74,0.87534275,0.65866,0.308431,0.128158\r\n8895.63,0.8745439,0.65872,0.308409,0.128188\r\n8899.35,0.873669,0.658973,0.308315,0.128315\r\n8903.07,0.872792,0.659225,0.308221,0.128441\r\n8903.27,0.87196735,0.659239,0.308216,0.128448\r\n8903.73,0.8712336,0.65927,0.308204,0.12846\r\n8909.69,0.87049185,0.659271,0.308053,0.128612\r\n8910.52,0.8695979,0.659271,0.308074,0.128634\r\n8914.23,0.8686713,0.659272,0.30817,0.128729\r\n8917.95,0.86794315,0.659272,0.308265,0.128824\r\n8918.22,0.8671296,0.659272,0.308272,0.128831\r\n8918.67,0.86636655,0.659272,0.308283,0.128842\r\n8921.67,0.8655212,0.659171,0.30836,0.128919\r\n8924.64,0.86466695,0.659071,0.308436,0.128995\r\n8925.36,0.86402035,0.659046,0.30846,0.129014\r\n8925.38,0.8630694,0.659046,0.308461,0.129014\r\n8929.05,0.8623467,0.658922,0.308586,0.129108\r\n8929.07,0.8615934,0.658921,0.308587,0.129109\r\n8929.88,0.8607305,0.658893,0.308615,0.129129\r\n8932.76,0.85989665,0.658747,0.308713,0.129203\r\n8933.17,0.8589703,0.658726,0.308727,0.129214\r\n8935.86,0.8580764,0.65859,0.308818,0.129214\r\n8936.45,0.85733465,0.658559,0.308849,0.129214\r\n8936.48,0.85636665,0.658558,0.30885,0.129214\r\n8940.15,0.85547945,0.658372,0.309037,0.129215\r\n8940.17,0.85450095,0.658371,0.309038,0.129215\r\n8943.86,0.85352015,0.658183,0.309227,0.129215\r\n8944.38,0.8526066,0.658157,0.309253,0.129215\r\n8944.82,0.8516936,0.658135,0.309275,0.129226\r\n8947.57,0.85073635,0.658065,0.309416,0.129297\r\n8950.82,0.84978105,0.657983,0.309582,0.12938\r\n8951.29,0.848811,0.657971,0.309598,0.129392\r\n8958.74,0.8479591,0.657782,0.309852,0.129583\r\n8959.34,0.8471335,0.657767,0.309872,0.129598\r\n8959.76,0.8462144,0.657756,0.309886,0.129609\r\n8962.03,0.8454917,0.65768,0.309964,0.129667\r\n8966.2,0.84466865,0.657539,0.310247,0.129774\r\n8970.96,0.84386725,0.657378,0.310571,0.129896\r\n8973.25,0.84317945,0.65732,0.310727,0.129954\r\n8974.29,0.8424927,0.657293,0.310779,0.129981\r\n8977.4,0.84190295,0.657215,0.310938,0.129981\r\n8985.5,0.8413371,0.65701,0.311351,0.129983\r\n8985.9,0.84069125,0.656999,0.311372,0.129993\r\n8988.21,0.84027305,0.656764,0.31149,0.130052\r\n8988.6,0.83975715,0.656725,0.311503,0.130062\r\n8997.1,0.8393875,0.65586,0.311793,0.130279\r\n8999.43,0.839026,0.655503,0.311872,0.130339\r\n9000.45,0.8386655,0.655346,0.311907,0.130365\r\n9003.54,0.8383615,0.654875,0.312012,0.130366\r\n9004.55,0.83810725,0.654719,0.312047,0.130366\r\n9010.65,0.8378604,0.653476,0.312254,0.130367\r\n9012.01,0.8376896,0.653198,0.312289,0.130367\r\n9015.4,0.83752335,0.652852,0.312376,0.130368\r\n9018.48,0.83736035,0.652538,0.312455,0.130447\r\n9023.2,0.8372148,0.652058,0.312576,0.130567\r\n9025.6,0.8371011,0.651691,0.312637,0.130629\r\n9030.36,0.83701165,0.650964,0.31296,0.13075\r\n9030.66,0.8369339,0.650918,0.312981,0.130758\r\n9033.43,0.83686145,0.650351,0.313169,0.130829\r\n9036.82,0.83680485,0.649659,0.3134,0.130916\r\n9038.11,0.83676195,0.649397,0.313465,0.130949\r\n9041.83,0.8367198,0.648256,0.313655,0.131044\r\n9045.31,0.83668155,0.647545,0.313833,0.131133\r\n9048.38,0.83664395,0.646918,0.31399,0.131134\r\n9049.28,0.836628,0.646735,0.314035,0.131134\r\n9051.78,0.83661795,0.646224,0.314163,0.131134\r\n9056.52,0.8366105,0.645257,0.314485,0.131135\r\n9056.73,0.83661095,0.645214,0.314499,0.131135\r\n9060.45,0.8366114,0.644073,0.314752,0.131136\r\n9063,0.8366118,0.643552,0.314926,0.131136\r\n9063.33,0.8366122,0.643486,0.314959,0.131136\r\n9067.9,0.83661265,0.642552,0.315425,0.131137\r\n9071.47,0.83661305,0.641824,0.315788,0.131137\r\n9074.23,0.83661345,0.64126,0.316069,0.131208\r\n9075.35,0.83661385,0.641031,0.316145,0.131237\r\n9078.28,0.83661425,0.640135,0.316344,0.131311\r\n9079.07,0.8366147,0.63989,0.316398,0.131332\r\n9085.45,0.8366151,0.638588,0.316832,0.131495\r\n9086.42,0.8366155,0.63839,0.316898,0.13152\r\n9086.52,0.8366159,0.638369,0.316905,0.13152\r\n9090.24,0.8366163,0.636848,0.317157,0.131521\r\n9093.22,0.8366123,0.636391,0.31736,0.131521\r\n9096.68,0.836607,0.635863,0.317595,0.131522\r\n9097.69,0.83659705,0.635707,0.317664,0.131522\r\n9101.37,0.836587,0.634202,0.317914,0.131522\r\n9101.41,0.8365687,0.634186,0.317916,0.131522\r\n9107.9,0.8365501,0.633193,0.318357,0.131523\r\n9108.17,0.83652905,0.633152,0.318376,0.131523\r\n9108.86,0.83648275,0.633045,0.318423,0.131523\r\n9116.32,0.83642695,0.631525,0.31893,0.131525\r\n9119.12,0.83637025,0.630376,0.31912,0.131525\r\n9119.36,0.8363127,0.630276,0.319133,0.131525\r\n9120.03,0.83625045,0.630003,0.319167,0.131525\r\n9123.74,0.83617885,0.628482,0.319356,0.131526\r\n9131.12,0.836107,0.621633,0.319732,0.131527\r\n9131.26,0.8360316,0.621603,0.31974,0.131527\r\n9134.08,0.83595585,0.621028,0.319883,0.131527\r\n9134.31,0.8358747,0.620982,0.319899,0.131527\r\n9138.57,0.83578885,0.620112,0.320188,0.131528\r\n9142.29,0.83569915,0.618971,0.320441,0.131529\r\n9145.3,0.83560855,0.617737,0.320646,0.131529\r\n9146,0.83552205,0.61745,0.320694,0.131529\r\n9146.21,0.835433,0.617364,0.320708,0.131529\r\n9149.25,0.8353474,0.616119,0.320914,0.13153\r\n9149.71,0.83524455,0.615928,0.320946,0.13153\r\n9156.52,0.83514835,0.614539,0.321409,0.131531\r\n9157.17,0.835043,0.614407,0.321453,0.131531\r\n9160.89,0.83493935,0.613266,0.321705,0.131531\r\n9161.16,0.83485325,0.613155,0.321724,0.131531\r\n9164.18,0.83477525,0.611915,0.32193,0.131532\r\n9164.6,0.83468745,0.611745,0.321958,0.131532\r\n9167.75,0.8345985,0.610456,0.322172,0.131532\r\n9168.31,0.83450175,0.610223,0.32221,0.131532\r\n9172.01,0.83441425,0.60718,0.322461,0.131533\r\n9172.03,0.8343156,0.608702,0.322463,0.131533\r\n9175.38,0.8342205,0.604526,0.322691,0.131534\r\n9175.7,0.8341257,0.604136,0.322712,0.131534\r\n9175.72,0.8340363,0.605658,0.322714,0.131534\r\n9176.11,0.8339516,0.60538,0.32274,0.131534\r\n9178.97,0.8338707,0.603319,0.322934,0.131534\r\n9179.42,0.83379075,0.602995,0.32298,0.131534\r\n9183.13,0.8337111,0.601474,0.323358,0.131535\r\n9186.85,0.8336353,0.599952,0.323737,0.131535\r\n9190.2,0.8335607,0.59858,0.324078,0.131536\r\n9190.33,0.83350285,0.598524,0.324087,0.131536\r\n9190.56,0.8334453,0.598431,0.324102,0.131536\r\n9191.05,0.83337955,0.59833,0.324136,0.131536\r\n9198.01,0.83330585,0.59691,0.324609,0.131537\r\n9201.42,0.83323005,0.596389,0.32484,0.131538\r\n9205.27,0.83315565,0.5958,0.325102,0.131538\r\n9205.47,0.8330919,0.595769,0.325116,0.131538\r\n9206,0.83303015,0.59555,0.325152,0.131538\r\n9209.18,0.8329809,0.594248,0.325368,0.131539\r\n9212.64,0.8329318,0.593719,0.325603,0.131539\r\n9216.64,0.8328941,0.593108,0.32601,0.13154\r\n9220.22,0.8328571,0.591642,0.326375,0.13154\r\n9220.35,0.8328231,0.591586,0.326388,0.13154\r\n9220.95,0.83280395,0.591495,0.326449,0.13154\r\n9223.87,0.8328049,0.591048,0.326746,0.131467\r\n9227.81,0.83282435,0.590446,0.327014,0.131367\r\n9231.44,0.83287155,0.589704,0.327261,0.131275\r\n9235.09,0.83292,0.58896,0.327509,0.131183\r\n9235.26,0.83297325,0.588925,0.327527,0.131178\r\n9235.89,0.83304975,0.588828,0.327591,0.131162\r\n9238.93,0.83315565,0.588363,0.327901,0.131163\r\n9242.72,0.8332903,0.587784,0.328286,0.131163\r\n9246.32,0.8334434,0.587234,0.328652,0.131164\r\n9246.43,0.83362425,0.587217,0.32866,0.131164\r\n9250.17,0.8338183,0.586644,0.328915,0.131164\r\n9250.84,0.83402785,0.586371,0.32896,0.131165\r\n9253.89,0.834256,0.585122,0.329167,0.131165\r\n9257.54,0.8344847,0.584751,0.329415,0.131166\r\n9257.65,0.83475805,0.584739,0.329423,0.131166\r\n9262.05,0.83506215,0.584291,0.329722,0.131166\r\n9265.08,0.8353669,0.583983,0.329928,0.131089\r\n9265.15,0.8357259,0.583955,0.329932,0.131088\r\n9268.76,0.8360824,0.582475,0.330178,0.130996\r\n9268.8,0.8364257,0.582461,0.330181,0.130995\r\n9268.9,0.83682355,0.582444,0.330192,0.130993\r\n9276.25,0.8372044,0.581321,0.330941,0.130807\r\n9276.4,0.83756945,0.58129,0.330956,0.130803\r\n9276.99,0.8379939,0.58117,0.331016,0.130788\r\n9279.99,0.838417,0.580558,0.331321,0.130712\r\n9280.16,0.83884385,0.580523,0.331333,0.130708\r\n9283.7,0.83927195,0.5798,0.331574,0.130618\r\n9287.65,0.8397082,0.578994,0.331842,0.130518\r\n9291.16,0.84014425,0.578279,0.33208,0.130429\r\n9291.21,0.84056855,0.578261,0.332084,0.130428\r\n9291.93,0.8410193,0.57804,0.332157,0.13041\r\n9294.88,0.84145735,0.577138,0.332457,0.13041\r\n9295.15,0.8419339,0.577082,0.332485,0.13041\r\n9298.9,0.84243305,0.576316,0.332867,0.130411\r\n9302.33,0.84288835,0.575617,0.333216,0.130411\r\n9302.44,0.84334295,0.575599,0.333227,0.130411\r\n9306.88,0.84381655,0.57492,0.333679,0.130412\r\n9309.78,0.84423615,0.574476,0.333975,0.130412\r\n9309.93,0.8446567,0.574455,0.333989,0.130412\r\n9310.14,0.84512975,0.574422,0.334005,0.130413\r\n9317.24,0.8455311,0.573336,0.334548,0.130414\r\n9317.63,0.84595235,0.573257,0.334578,0.130414\r\n9321.83,0.8463762,0.572399,0.334899,0.130414\r\n9324.69,0.84674035,0.571815,0.335118,0.130318\r\n9324.89,0.8471212,0.571785,0.335133,0.130311\r\n9328.85,0.8475002,0.571178,0.335537,0.130177\r\n9332.15,0.84785085,0.570675,0.335872,0.130065\r\n9332.38,0.8481709,0.570628,0.335895,0.130058\r\n9333.03,0.8485156,0.570493,0.33594,0.130035\r\n9336.35,0.848853,0.569818,0.336165,0.130036\r\n9339.6,0.84916055,0.569154,0.336386,0.130036\r\n9343.6,0.84946535,0.568746,0.336658,0.130037\r\n9347.98,0.84971305,0.5683,0.337105,0.130038\r\n9350.79,0.84993755,0.568014,0.337391,0.129967\r\n9351.29,0.8501493,0.567997,0.337442,0.129954\r\n9354.83,0.85030665,0.567877,0.337801,0.129865\r\n9362,0.8504426,0.567635,0.338532,0.129683\r\n9362.49,0.8505743,0.567635,0.338582,0.12967\r\n9362.92,0.85068065,0.567635,0.338626,0.12966\r\n9366.05,0.85070305,0.567636,0.338945,0.12966\r\n9369.95,0.8506511,0.567636,0.339209,0.129661\r\n9376.95,0.85057435,0.567637,0.339685,0.129662\r\n9377.28,0.85048175,0.567629,0.339707,0.129662\r\n9377.4,0.85037845,0.567626,0.339716,0.129662\r\n9377.87,0.8501886,0.567614,0.339748,0.129662\r\n9381.11,0.84991715,0.567532,0.339968,0.129552\r\n9388.5,0.8496434,0.567345,0.34047,0.129303\r\n9388.57,0.84935945,0.567343,0.340475,0.1293\r\n9389.08,0.8490216,0.56733,0.34051,0.129283\r\n9391.89,0.8486415,0.567259,0.3407,0.129212\r\n9396.02,0.8482331,0.567154,0.340982,0.129107\r\n9399.72,0.8477959,0.567061,0.341233,0.129014\r\n9403.48,0.84734765,0.566966,0.341329,0.128918\r\n9404.02,0.84684185,0.566952,0.341343,0.128905\r\n9406.83,0.84634055,0.566881,0.341415,0.128834\r\n9407.19,0.8457875,0.566862,0.341424,0.128824\r\n9414.64,0.84523555,0.566484,0.341615,0.128636\r\n9414.67,0.84461145,0.566482,0.341616,0.128635\r\n9418.36,0.844002,0.566295,0.341616,0.128542\r\n9418.96,0.8433793,0.566264,0.341616,0.128526\r\n9421.77,0.84282325,0.566122,0.341617,0.128527\r\n9422.08,0.84219455,0.566111,0.341617,0.128527\r\n9429.53,0.8415664,0.56586,0.341618,0.128528\r\n9429.62,0.8409223,0.565856,0.341618,0.128528\r\n9432.97,0.84033745,0.565743,0.341533,0.128529\r\n9433.91,0.8397855,0.565648,0.341509,0.128529\r\n9436.99,0.83916085,0.565334,0.341431,0.128425\r\n9444.16,0.8385386,0.564603,0.34125,0.128182\r\n9444.44,0.83794405,0.564576,0.341243,0.128173\r\n9444.56,0.8373875,0.564563,0.34124,0.128169\r\n9445.11,0.8368074,0.564507,0.341221,0.12815\r\n9451.89,0.8362443,0.563816,0.340992,0.128151\r\n9455.36,0.83569085,0.563464,0.340875,0.128152\r\n9455.77,0.8351804,0.563422,0.340861,0.128152\r\n9460.06,0.83466715,0.562985,0.340752,0.128152\r\n9463.09,0.83422555,0.562676,0.340675,0.128075\r\n9466.55,0.8337793,0.562324,0.340588,0.127988\r\n9470.71,0.8334043,0.561688,0.340483,0.127883\r\n9474.01,0.8330373,0.561183,0.340483,0.127799\r\n9475,0.8327095,0.561031,0.340483,0.127774\r\n9478.04,0.83239085,0.560567,0.340484,0.127671\r\n9481.47,0.8321536,0.560043,0.340484,0.127556\r\n9485.66,0.83191775,0.559616,0.340485,0.127414\r\n9486.21,0.83171585,0.55956,0.340447,0.127395\r\n9492.66,0.8315215,0.558903,0.34001,0.127232\r\n9492.99,0.83135075,0.558853,0.339988,0.127224\r\n9496.86,0.83125325,0.558261,0.339726,0.127126\r\n9500.12,0.83115855,0.557763,0.339615,0.127043\r\n9501.15,0.83106485,0.557551,0.33958,0.127017\r\n9507.57,0.83097535,0.556242,0.339363,0.127018\r\n9507.94,0.8309373,0.556185,0.339351,0.127018\r\n9508.06,0.8308874,0.556166,0.339347,0.127018\r\n9515.02,0.83083525,0.555101,0.338993,0.127019\r\n9516.1,0.83078735,0.554882,0.338938,0.127019\r\n9519.14,0.8306985,0.554262,0.338784,0.126942\r\n9522.48,0.8306082,0.55358,0.338615,0.126858\r\n9522.86,0.83052025,0.553522,0.338595,0.126848\r\n9523,0.83036335,0.5535,0.338588,0.126845\r\n9526.57,0.8301388,0.552954,0.338467,0.126754\r\n9529.93,0.8299109,0.55244,0.338354,0.126669\r\n9530.28,0.8296631,0.552368,0.338342,0.12666\r\n9531.04,0.8293494,0.552213,0.338316,0.126641\r\n9534,0.82902655,0.55161,0.338216,0.126541\r\n9534.2,0.8285643,0.551568,0.338209,0.126534\r\n9537.38,0.828089,0.550919,0.337994,0.126427\r\n9537.69,0.82768975,0.550872,0.337973,0.126416\r\n9537.71,0.8271383,0.550868,0.337971,0.126415\r\n9541.4,0.82656965,0.550304,0.337721,0.126291\r\n9542.25,0.8259478,0.550175,0.337664,0.126262\r\n9544.84,0.82532025,0.549779,0.337488,0.126196\r\n9545.12,0.82468725,0.549736,0.337469,0.126189\r\n9545.4,0.8240076,0.549692,0.33745,0.126182\r\n9548.83,0.8232785,0.549168,0.337276,0.126095\r\n9552.3,0.82255575,0.548638,0.3371,0.126008\r\n9552.55,0.8219078,0.548612,0.337087,0.126001\r\n9557.19,0.82128025,0.54814,0.336851,0.125884\r\n9560.04,0.8207317,0.54785,0.336706,0.125788\r\n9560.06,0.82026195,0.547848,0.336705,0.125787\r\n9560.34,0.8199249,0.547819,0.336691,0.125777\r\n9563.49,0.8196017,0.547498,0.336585,0.125671\r\n9563.82,0.8193701,0.547448,0.336573,0.12566\r\n9563.84,0.81927515,0.547444,0.336573,0.125659\r\n9563.87,0.8192648,0.547441,0.336572,0.125658\r\n9567.63,0.81946165,0.546866,0.336445,0.125531\r\n9567.65,0.81974725,0.546862,0.336444,0.12553\r\n9567.67,0.82003285,0.546859,0.336443,0.12553\r\n9567.69,0.82048485,0.546855,0.336442,0.125529\r\n9568.39,0.82100575,0.546748,0.336419,0.125505\r\n9570.95,0.82159005,0.546358,0.336332,0.12544\r\n9571.45,0.82225615,0.546306,0.336315,0.125428\r\n9571.48,0.8230042,0.546304,0.336315,0.125427\r\n9571.5,0.8238986,0.546302,0.336314,0.125426\r\n9571.52,0.82484655,0.546299,0.336313,0.125426\r\n9571.54,0.82587415,0.546297,0.336312,0.125425\r\n9575.28,0.82695425,0.545916,0.336059,0.125331\r\n9575.3,0.8280959,0.545914,0.336057,0.12533\r\n9575.33,0.82923755,0.545912,0.336056,0.12533\r\n9575.35,0.8304497,0.54591,0.336054,0.125329\r\n9579.11,0.83167415,0.545527,0.335799,0.125234\r\n9579.13,0.83296805,0.545524,0.335798,0.125233\r\n9579.15,0.83426195,0.545522,0.335796,0.125233\r\n9579.18,0.83553975,0.54552,0.335795,0.125232\r\n9582.14,0.8367536,0.545218,0.335594,0.125157\r\n9582.74,0.8378953,0.545126,0.335553,0.125142\r\n9582.94,0.839037,0.545096,0.335533,0.125137\r\n9582.96,0.8401787,0.545093,0.335531,0.125136\r\n9582.98,0.84131335,0.545089,0.335529,0.125136\r\n9583.34,0.84246525,0.545035,0.335493,0.125127\r\n9586.74,0.843683,0.544514,0.335146,0.125127\r\n9586.77,0.84490075,0.544511,0.335143,0.125127\r\n9586.79,0.84611855,0.544507,0.335141,0.125127\r\n9586.81,0.84731665,0.544504,0.335139,0.125127\r\n9589.6,0.848534,0.544078,0.334855,0.125128\r\n9590.57,0.84975175,0.543879,0.334756,0.125128\r\n9590.59,0.8509695,0.543874,0.334754,0.125128\r\n9590.62,0.85218725,0.54387,0.334751,0.125128\r\n9590.64,0.8533957,0.543865,0.334749,0.125128\r\n9593.94,0.85453735,0.543192,0.334413,0.125128\r\n9594.4,0.855679,0.543098,0.334366,0.125129\r\n9594.42,0.8570481,0.543093,0.334364,0.125129\r\n9597.05,0.8584837,0.542557,0.334096,0.125129\r\n9598.28,0.86030745,0.542368,0.333971,0.125129\r\n9602.16,0.8621951,0.541775,0.333576,0.124998\r\n9602.18,0.8641549,0.541771,0.333573,0.124997\r\n9602.21,0.8660851,0.541768,0.333571,0.124996\r\n9604.51,0.8679519,0.541416,0.333337,0.124919\r\n9605.13,0.86982845,0.541321,0.333273,0.124898\r\n9605.97,0.8717122,0.541193,0.333217,0.124869\r\n9605.99,0.87359595,0.541189,0.333215,0.124869\r\n9606.01,0.8754797,0.541186,0.333213,0.124868\r\n9606.04,0.8773572,0.541182,0.333212,0.124867\r\n9609.49,0.8791845,0.540654,0.332978,0.12475\r\n9609.8,0.8810682,0.540607,0.332957,0.124743\r\n9609.82,0.8829519,0.540604,0.332955,0.124742\r\n9609.84,0.88483565,0.5406,0.332954,0.124741\r\n9609.86,0.8866858,0.540597,0.332952,0.124741\r\n9611.96,0.88850275,0.540276,0.33281,0.124688\r\n9613.62,0.89038645,0.540022,0.332697,0.124646\r\n9613.65,0.89227015,0.540018,0.332696,0.124645\r\n9613.67,0.8939264,0.540015,0.332694,0.124644\r\n9613.69,0.89551615,0.540011,0.332693,0.124644\r\n9613.71,0.89671075,0.540008,0.332691,0.124643\r\n9616.33,0.8978524,0.539608,0.332514,0.124577\r\n9617.47,0.89899405,0.539433,0.332397,0.124548\r\n9617.5,0.9001653,0.539429,0.332395,0.124548\r\n9617.52,0.90139995,0.539426,0.332393,0.124547\r\n9617.54,0.9026557,0.539422,0.332391,0.124546\r\n9619.42,0.90387345,0.539135,0.332199,0.124499\r\n9621.3,0.9050912,0.53888,0.332008,0.124451\r\n9621.32,0.90630895,0.538876,0.332005,0.124451\r\n9621.35,0.90751925,0.538873,0.332003,0.12445\r\n9624.43,0.90873705,0.538454,0.331689,0.124372\r\n9625.11,0.90995485,0.538362,0.33162,0.124355\r\n9625.13,0.91117265,0.538359,0.331618,0.124354\r\n9625.15,0.9123623,0.538356,0.331616,0.124354\r\n9627.52,0.9135376,0.538034,0.331374,0.124294\r\n9628.91,0.9147554,0.537845,0.33128,0.124259\r\n9628.93,0.9159732,0.537842,0.331278,0.124258\r\n9628.96,0.9171483,0.537839,0.331277,0.124257\r\n9630.61,0.91829,0.537615,0.331165,0.124216\r\n9632.72,0.9194317,0.537292,0.331022,0.124162\r\n9632.74,0.92038725,0.537289,0.33102,0.124162\r\n9636.44,0.92139575,0.536723,0.33077,0.124068\r\n9636.46,0.92219425,0.53672,0.330768,0.124068\r\n9638.06,0.9228616,0.536475,0.330659,0.124027\r\n9638.72,0.9233987,0.536374,0.330615,0.12401\r\n9639.37,0.92383655,0.536275,0.330571,0.123994\r\n9640.11,0.92431235,0.536162,0.330521,0.123969\r\n9640.13,0.92478815,0.536159,0.330519,0.123968\r\n9640.15,0.92473125,0.536155,0.330518,0.123967\r\n9643.75,0.924688,0.535605,0.330274,0.123846\r\n9643.77,0.92470715,0.535601,0.330272,0.123845\r\n9643.8,0.9247263,0.535598,0.330271,0.123844\r\n9643.82,0.9244537,0.535594,0.330269,0.123843\r\n9645.52,0.92389215,0.535334,0.330154,0.123786\r\n9647.42,0.92337855,0.534946,0.330025,0.123722\r\n9647.44,0.92286495,0.534942,0.330024,0.123721\r\n9647.47,0.92196365,0.534937,0.330022,0.12372\r\n9649.92,0.920946,0.534436,0.329855,0.123637\r\n9650.58,0.91982365,0.534302,0.329811,0.123615\r\n9651.07,0.9187013,0.534202,0.329778,0.123603\r\n9651.09,0.9177882,0.534198,0.329776,0.123602\r\n9651.11,0.9168751,0.534193,0.329775,0.123601\r\n9651.13,0.91578125,0.534188,0.329773,0.123601\r\n9652.97,0.9144483,0.533813,0.329649,0.123554\r\n9654.69,0.9132456,0.533551,0.329532,0.123511\r\n9654.71,0.91218025,0.533547,0.329531,0.12351\r\n9654.74,0.9111149,0.533544,0.329529,0.12351\r\n9654.76,0.91004955,0.53354,0.329528,0.123509\r\n9654.78,0.9087559,0.533537,0.329526,0.123509\r\n9658.31,0.90746225,0.532996,0.329287,0.123419\r\n9658.34,0.9061686,0.532993,0.329285,0.123419\r\n9658.36,0.90487495,0.532989,0.329283,0.123418\r\n9658.38,0.90387305,0.532986,0.329282,0.123417\r\n9658.4,0.9028664,0.532982,0.32928,0.123417\r\n9660.43,0.90169945,0.532673,0.329143,0.123366\r\n9661.12,0.90034875,0.532531,0.329096,0.123348\r\n9662.01,0.89938575,0.53235,0.329074,0.123326\r\n9662.03,0.89836195,0.532345,0.329073,0.123325\r\n9665.52,0.8973917,0.531634,0.328985,0.123237\r\n9665.72,0.8961654,0.531592,0.32898,0.12323\r\n9667.88,0.8947386,0.531152,0.328925,0.123157\r\n9669.37,0.8933118,0.530924,0.328887,0.123107\r\n9669.39,0.89227575,0.53092,0.328887,0.123106\r\n9669.42,0.89160995,0.530917,0.328886,0.123105\r\n9669.44,0.89025925,0.530914,0.328886,0.123104\r\n9672.99,0.88890855,0.53037,0.328796,0.122984\r\n9673.02,0.88755785,0.530366,0.328795,0.122983\r\n9673.04,0.88620715,0.530363,0.328794,0.122983\r\n9673.06,0.88561745,0.530359,0.328794,0.122982\r\n9673.08,0.8846163,0.530356,0.328793,0.122981\r\n9675.34,0.88343065,0.530011,0.328736,0.122905\r\n9676.06,0.88207995,0.529863,0.328718,0.12288\r\n9676.66,0.88072925,0.529741,0.328677,0.12286\r\n9676.68,0.87970035,0.529736,0.328676,0.122859\r\n9676.71,0.87875425,0.529732,0.328674,0.122859\r\n9676.72,0.87761285,0.529728,0.328673,0.122858\r\n9680.35,0.87647145,0.528987,0.328427,0.122766\r\n9680.38,0.87527245,0.528983,0.328426,0.122766\r\n9682.79,0.87397885,0.528491,0.328262,0.122704\r\n9684.02,0.8729413,0.528302,0.328178,0.122673\r\n9684.04,0.87210425,0.528298,0.328177,0.122673\r\n9684.07,0.87078575,0.528295,0.328175,0.122672\r\n9687.26,0.8693399,0.527806,0.327958,0.122591\r\n9687.67,0.86789405,0.527744,0.327959,0.122581\r\n9687.69,0.8671331,0.527741,0.327959,0.12258\r\n9687.71,0.8663722,0.527737,0.327959,0.12258\r\n9687.74,0.8651056,0.527734,0.327959,0.122579\r\n9690.24,0.86358365,0.52735,0.327959,0.122516\r\n9691.29,0.86206175,0.526921,0.327959,0.122489\r\n9691.31,0.8609513,0.526912,0.327959,0.122489\r\n9691.34,0.86002535,0.526903,0.327959,0.122488\r\n9691.36,0.85926445,0.526893,0.327959,0.122487\r\n9691.38,0.85837875,0.526884,0.327959,0.122487\r\n9691.67,0.8570244,0.526768,0.327959,0.12248\r\n9693.96,0.8555039,0.525829,0.32796,0.122422\r\n9694.94,0.85436245,0.525428,0.32796,0.122397\r\n9694.96,0.853221,0.525419,0.32796,0.122396\r\n9694.98,0.852357,0.525409,0.32796,0.122396\r\n9695.01,0.85125775,0.5254,0.32796,0.122395\r\n9697.67,0.8499641,0.524307,0.32796,0.122328\r\n9698.61,0.84867045,0.524117,0.32796,0.122304\r\n9698.63,0.84785825,0.524112,0.32796,0.122303\r\n9698.65,0.84680005,0.524107,0.32796,0.122303\r\n9702.21,0.84565865,0.523381,0.327961,0.122213\r\n9702.3,0.84451725,0.523363,0.327959,0.122211\r\n9702.32,0.84313705,0.523359,0.327958,0.12221\r\n9705.12,0.8421003,0.522786,0.327887,0.122139\r\n9706.61,0.8408964,0.522559,0.327849,0.122101\r\n9712.58,0.8393511,0.521646,0.327698,0.122102\r\n9717.15,0.8376911,0.52118,0.327583,0.122103\r\n9717.82,0.8357887,0.521112,0.32756,0.122103\r\n9720.69,0.8338338,0.52082,0.327463,0.121958\r\n9723.77,0.83196115,0.520506,0.327359,0.121801\r\n9728.18,0.8304829,0.519833,0.32721,0.121577\r\n9728.36,0.8292014,0.519805,0.327204,0.121568\r\n9731.23,0.82785925,0.519365,0.327058,0.121422\r\n9732.76,0.82647065,0.519262,0.32698,0.121344\r\n9735.67,0.82504035,0.519064,0.326832,0.121246\r\n9742.43,0.8239521,0.518606,0.326489,0.121017\r\n9743.16,0.82298305,0.518532,0.326452,0.120993\r\n9743.3,0.8219424,0.518518,0.326445,0.120988\r\n9743.96,0.8208912,0.51845,0.326445,0.120966\r\n9753.62,0.82014245,0.517466,0.326447,0.120967\r\n9754.39,0.8194012,0.517349,0.326447,0.120967\r\n9754.51,0.8185988,0.517331,0.326447,0.120967\r\n9758.91,0.81796645,0.516658,0.326335,0.120968\r\n9761.08,0.81743565,0.516326,0.32628,0.120857\r\n9765.61,0.81726115,0.515401,0.326165,0.120627\r\n9768.53,0.81735505,0.514805,0.326092,0.120479\r\n9769.45,0.81750245,0.514743,0.326068,0.120432\r\n9773.84,0.81782635,0.514445,0.326069,0.120209\r\n9776.84,0.8181365,0.514242,0.32607,0.120108\r\n9779.73,0.81842895,0.514046,0.32607,0.12001\r\n9784.4,0.81872175,0.513729,0.326071,0.119852\r\n9785.05,0.8189956,0.513685,0.326071,0.11983\r\n9788.06,0.81926375,0.513481,0.326071,0.119754\r\n9790.93,0.8195306,0.513286,0.326072,0.119681\r\n9799.28,0.8197634,0.512862,0.326073,0.11947\r\n9799.34,0.81999575,0.512859,0.326073,0.119468\r\n9799.99,0.8202571,0.512826,0.32609,0.119452\r\n9805.87,0.8205376,0.512528,0.32624,0.119253\r\n9810.51,0.82075605,0.512646,0.326359,0.119097\r\n9811.2,0.82098505,0.512664,0.326376,0.119073\r\n9814.3,0.8212325,0.512743,0.326456,0.118995\r\n9817.99,0.8214794,0.512838,0.326331,0.118901\r\n9820.82,0.821738,0.51291,0.326235,0.11883\r\n9824.57,0.8219859,0.514052,0.326108,0.118735\r\n9825.5,0.82222715,0.514241,0.326077,0.118711\r\n9826.14,0.82248095,0.51437,0.326093,0.118695\r\n9829.22,0.82273435,0.514997,0.326172,0.118591\r\n9832.07,0.8230044,0.515576,0.326245,0.118494\r\n9837.34,0.8232759,0.516647,0.32638,0.118316\r\n9839.57,0.82353605,0.517099,0.326437,0.11826\r\n9840.45,0.82379405,0.517367,0.32646,0.118238\r\n9840.46,0.82406365,0.517369,0.32646,0.118237\r\n9843.32,0.82435875,0.518241,0.326557,0.118165\r\n9847.94,0.824645,0.519179,0.326715,0.118048\r\n9850.82,0.8249367,0.519764,0.326813,0.117975\r\n9851.67,0.82522925,0.52011,0.326842,0.117953\r\n9852.29,0.82550745,0.520359,0.326842,0.117938\r\n9854.58,0.8257932,0.521287,0.326842,0.11788\r\n9859.17,0.826126,0.521755,0.326843,0.117764\r\n9865.81,0.82644925,0.52243,0.326844,0.117596\r\n9866.62,0.826754,0.522554,0.326844,0.117575\r\n9866.66,0.82704865,0.522561,0.326844,0.117574\r\n9867.23,0.8273705,0.522647,0.326844,0.11756\r\n9873.3,0.8276942,0.523573,0.326845,0.117355\r\n9874.16,0.82803645,0.523747,0.326845,0.117325\r\n9878.43,0.828375,0.524616,0.326846,0.117181\r\n9880.79,0.828703,0.525096,0.326846,0.117121\r\n9881.57,0.82902355,0.525214,0.326847,0.117102\r\n9885.38,0.8293623,0.525796,0.326847,0.117005\r\n9888.28,0.82974325,0.526239,0.326848,0.116932\r\n9892.88,0.83012665,0.527173,0.326848,0.116815\r\n9893.38,0.83050045,0.527273,0.326848,0.116803\r\n9895.78,0.83084275,0.527762,0.326849,0.116721\r\n9896.51,0.83118095,0.528059,0.326849,0.116697\r\n9899.54,0.83151755,0.529285,0.326926,0.116594\r\n9900.37,0.83187975,0.529535,0.326947,0.116566\r\n9903.29,0.83223945,0.530427,0.327022,0.116467\r\n9904.58,0.8325931,0.530947,0.327055,0.116424\r\n9907.05,0.8329044,0.531949,0.327119,0.116361\r\n9907.86,0.833232,0.532276,0.327139,0.116341\r\n9910.81,0.83356425,0.533472,0.327215,0.116266\r\n9911.47,0.8339159,0.533737,0.327232,0.116249\r\n9914.57,0.8342272,0.534995,0.327337,0.116171\r\n9915.35,0.8345923,0.53531,0.327364,0.116151\r\n9918.33,0.83493605,0.536517,0.327466,0.116076\r\n9918.36,0.8352831,0.538039,0.327466,0.116075\r\n9919.11,0.83562935,0.538346,0.327492,0.116056\r\n9919.52,0.83597595,0.538512,0.327506,0.116046\r\n9922.12,0.8362974,0.539562,0.327594,0.115958\r\n9922.68,0.83660205,0.540161,0.327614,0.115939\r\n9926.6,0.83693575,0.544285,0.327614,0.115806\r\n9929.71,0.83726755,0.547554,0.327615,0.115701\r\n9930.73,0.83762345,0.547966,0.327615,0.115667\r\n9933.47,0.83797035,0.549077,0.327615,0.115528\r\n9934.1,0.8383369,0.549332,0.327616,0.115495\r\n9937.23,0.8386768,0.550599,0.327616,0.115337\r\n9937.63,0.83903385,0.550661,0.327616,0.115316\r\n9941.59,0.8393975,0.551265,0.327617,0.115115\r\n9944.72,0.83976255,0.551742,0.327617,0.114956\r\n9945.66,0.8401337,0.552124,0.327617,0.114908\r\n9948.48,0.84050605,0.553265,0.327618,0.114813\r\n9949.09,0.84087875,0.553511,0.327618,0.114792\r\n9952.24,0.8412481,0.554787,0.327618,0.114686\r\n9952.58,0.84158315,0.554925,0.327618,0.114674\r\n9956,0.8419222,0.55631,0.327532,0.114559\r\n9956.58,0.8422489,0.557101,0.327517,0.114539\r\n9956.87,0.84264565,0.557496,0.32751,0.114529\r\n9964.08,0.84304835,0.567322,0.327327,0.114347\r\n9967.44,0.8434473,0.571913,0.327242,0.114262\r\n9967.52,0.8438444,0.571945,0.32724,0.11426\r\n9971.2,0.84421275,0.573435,0.327147,0.114166\r\n9971.57,0.8445732,0.573491,0.327138,0.114157\r\n9971.81,0.84497575,0.573528,0.327132,0.114151\r\n9978.69,0.84537565,0.574578,0.326957,0.113977\r\n9979.06,0.84577755,0.574728,0.326948,0.113967\r\n9982.45,0.84617645,0.576101,0.326862,0.113882\r\n9982.46,0.84657375,0.576105,0.326862,0.113881\r\n9986.21,0.8469419,0.577623,0.326608,0.113786\r\n9986.55,0.84730445,0.577762,0.326585,0.113778\r\n9986.75,0.8477036,0.577842,0.326571,0.113773\r\n9989.97,0.84810345,0.579146,0.326353,0.113664\r\n9990.31,0.8485471,0.579284,0.32633,0.113652\r\n9993.66,0.84898955,0.580641,0.326103,0.113539\r\n9993.73,0.84941285,0.580669,0.326098,0.113537\r\n9994.07,0.84985775,0.580807,0.326075,0.113525\r\n9997.49,0.85030325,0.582191,0.325843,0.11341\r\n9997.96,0.8507007,0.58238,0.325811,0.113394\r\n10001.3,0.85107485,0.583714,0.325588,0.113311\r\n10001.6,0.8514819,0.583809,0.325567,0.113303\r\n10004.9,0.85186235,0.584813,0.325343,0.113219\r\n10005,0.8522812,0.584856,0.325334,0.113216\r\n10009.1,0.85272385,0.58568,0.325059,0.113113\r\n10012.5,0.85309705,0.586379,0.324825,0.113026\r\n10012.6,0.853469,0.590946,0.324821,0.113024\r\n10012.9,0.8538413,0.591079,0.324798,0.113016\r\n10016.3,0.85421365,0.592468,0.324566,0.112929\r\n10016.4,0.8545589,0.59399,0.324564,0.112928\r\n10016.6,0.8549777,0.607308,0.324551,0.112923\r\n10020.3,0.855211526,0.59399,0.324296,0.11282";
    }
}